package dataquality.webservices.imp_catalogo;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.Period;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter;

@WebService
public class DataQualityImpCatalogoMI {

	/* NO ESTÁ EN EL CATÁLOGO */
	/**
	 * Descripcion: Encuentra caracteres incorrectos. Los valores validos son: a-zA-Z0-9áéíóúñ
	 * Granularidad: Celda Dimension: Exactitud Factor: Correctitud sintáctica Metrica: Booleano de
	 * correctitud sintactica
	 * 
	 * @param valueCell
	 *            valor a evaluar
	 * @return El valor tiene caracteres validos
	 * @throws Exception
	 */
	@WebMethod
	public boolean wrongCodedCharacters(@WebParam(name = "valueCell") String valueCell) throws Exception {
		try {

			Pattern patron = Pattern.compile("[^a-zA-Z0-9áéíóúñ ]");
			Matcher match = patron.matcher(valueCell);
			return match.find();

		} catch (Exception e) {
			e.printStackTrace();
			throw new Exception(e.getMessage());
		}
	}

	

	/* CASO DE ESTUDIO - MI 5*/
	@WebMethod
	public double foreignKeyValidPercentage(@WebParam(name = "databaseUrl") String databaseUrl,
			@WebParam(name = "table") String table, @WebParam(name = "column") String column,
			@WebParam(name = "referenceTable") String referenceTable,
			@WebParam(name = "referenceColumn") String referenceColumn) throws Exception {
		try {
			Connection con = DriverManager.getConnection(databaseUrl);
			Statement s1 = con.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_READ_ONLY);

			String sql1 = "SELECT ((SELECT count(*) FROM " + table + " WHERE " + column + " IN (SELECT " + referenceColumn + " FROM " + referenceTable + "))" 
					+ " / " + " (SELECT count(*) FROM " + table + "))";

			ResultSet rs1 = s1.executeQuery(sql1);
			rs1.first();
			return rs1.getDouble(1);

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	/* ESTÁ EN EL CATÁLOGO (método 4) */
	/* CASO DE ESTUDIO - MI 2*/
	@WebMethod
	public double percentageNotNullValues(@WebParam(name = "databaseUrl") String databaseUrl,
			@WebParam(name = "tableName") String tableName, @WebParam(name = "columnName") String columnName) throws Exception {

		try {
			
			Connection con = DriverManager.getConnection(databaseUrl);
			Statement s1 = con.createStatement();
			
			String sql1 = "SELECT ((SELECT count(*) FROM "+ tableName +" WHERE "+ columnName +" IS NOT NULL)/(SELECT count(*) FROM "+ tableName +"))*100";
			ResultSet rs1 = s1.executeQuery(sql1);
			rs1.first();
			return rs1.getDouble(1);

		} catch (Exception e) {
			e.printStackTrace();
			return 0;
		}
	}

	/* CATALOGO (metodo 1) */
	/**
	 * Verificar el dígito verificador de las CI uruguayas Recibe como parámetro un string que
	 * representa un número de cédula uruguaya, se verifica si el dígito verificador es correcto
	 * 
	 * @param ci
	 * @return Devuelve 1 en caso que sea válido y si no 0.
	 * @throws Exception
	 */
	@WebMethod
	public Integer checkUruguayanCI(@XmlElement(name = "ci") String ci) throws Exception {

		try {
			int a = 0;
			int i = 0;
			int checknum = 0;
			if (ci.length() <= 6) {
				for (i = ci.length(); i < 7; i++) {
					ci = '0' + ci;
				}
			}
			for (i = 0; i < 7; i++) {
				a += (Integer.valueOf("2987634".substring(i, i + 1)) * Integer.valueOf(ci.substring(i, i + 1))) % 10;
			}

			if (a % 10 == 0) {
				checknum = 0;
			} else {
				checknum = 10 - a % 10;
			}

			if (checknum == Integer.valueOf(ci.substring(7, 8))) {
				return 1;
			} else {
				return 0;
			}

		} catch (Exception e) {
			return 0;
		}

	}

	/* CATALOGO (metodo 2) */
	@WebMethod
	public Integer satisfyRegularExpression(@XmlElement(name = "data") String data,
			@XmlElement(name = "regularExpression") String regularExpression) throws Exception {

		Pattern patron = Pattern.compile(regularExpression);
		Matcher match = patron.matcher(data);
		if (match.find()) {
			return 1;
		} else {
			return 0;
		}

	}

	/* CATALOGO (metodo 3) */
	@WebMethod
	public int checkStringLength(@XmlElement(name = "data") String data,
			@XmlElement(name = "dataLength") int dataLength) throws Exception {

		if (data != null && data.length() == dataLength) {
			return 1;
		} else {
			return 0;
		}

	}

	/* CATALOGO (metodo 5) */
	@WebMethod
	public double percentageWeightedNotNullValues(@XmlElement(name = "databaseUrl") String databaseUrl,
			@XmlElement(name = "columnName1") String columnName1, @XmlElement(name = "tableName1") String tableName1,
			@XmlElement(name = "weight1") int weight1, @XmlElement(name = "columnName2") String columnName2,
			@XmlElement(name = "tableName2") String tableName2, @XmlElement(name = "weight2") int weight2)
			throws Exception {

		Connection con = null;
		try {
			// Obtener datos de la tabla

			// Cantidad de registros no nulos y total de registros de la columna 1
			con = DriverManager.getConnection(databaseUrl);
			Statement s1 = con.createStatement();

			String sql1 = "SELECT count(*) FROM " + tableName1 + " WHERE " + columnName1 + "  IS NOT NULL";
			ResultSet rs1 = s1.executeQuery(sql1);
			double countNotNull1;
			if (rs1.next()) {
				countNotNull1 = rs1.getInt(1);
			} else {
				return -1;
			}

			String sqlTotal1 = "SELECT count(*) FROM " + tableName1;
			ResultSet rsTotal1 = s1.executeQuery(sqlTotal1);
			double countTotal1;
			if (rsTotal1.next()) {
				countTotal1 = rsTotal1.getInt(1);
			} else {
				return -1;
			}

			// Cantidad de registros no nulos y total de registros de la columna 2
			String sql2 = "SELECT count(*) FROM " + tableName2 + " WHERE " + columnName2 + "  IS NOT NULL";
			ResultSet rs2 = s1.executeQuery(sql2);

			double countNotNull2;
			if (rs2.next()) {
				countNotNull2 = rs2.getInt(1);
			} else {
				return -1;
			}

			String sqlTotal2 = "SELECT count(*) FROM " + tableName2;
			ResultSet rsTotal2 = s1.executeQuery(sqlTotal2);

			double countTotal2;
			if (rsTotal2.next()) {
				countTotal2 = rsTotal2.getInt(1);
			} else {
				return -1;
			}

			// Calculos
			double resultado1 = (countNotNull1 / countTotal1) * weight1;
			double resultado2 = (countNotNull2 / countTotal2) * weight2;

			double resultado = (resultado1 + resultado2) / (weight1 + weight2);

			return resultado;

		} catch (Exception e) {

			return 0;
		} finally {
			con.close();
		}

	}

	/* CATALOGO (metodo 6) */
	@WebMethod
	public boolean isLessThan(
			@XmlElement(required = false, defaultValue = "null", name = "value", nillable = true) Integer value,
			@XmlElement(name = "max") int max) throws Exception {
		return value != null && value < max;

	}

	/* CATALOGO (metodo 7) */
	@WebMethod
	public boolean isvalue1LessAndValue2Greater(
			@XmlElement(required = false, defaultValue = "null", name = "value1", nillable = true) Integer value1,
			@XmlElement(name = "max1") int max1,
			@XmlElement(required = false, defaultValue = "null", name = "value2", nillable = true) Integer value2,
			@XmlElement(name = "min2") int min2) throws Exception {
		return ((value1 != null && value1 < max1) && (value2 != null && min2 < value2));
	}

	/* CATALOGO (metodo 8) */
	@WebMethod
	public boolean verifyYearsFromADate(@XmlElement(name = "age") Integer age, @XmlElement(name = "date") Date date)
			throws Exception {
		if (date == null) {
			return false;
		}
		LocalDate now = LocalDate.now();
		LocalDate dateBirth = LocalDate.parse(new SimpleDateFormat("yyyy-MM-dd").format(date));
		int years = Period.between(dateBirth, now).getYears();
		return years == age;

	}

	/* CATALOGO (metodo 9) */
	@WebMethod
	public double percentageNonDuplicates(@XmlElement(name = "databaseUrl") String databaseUrl,
			@XmlElement(name = "columnName") String columnName, @XmlElement(name = "tableName") String tableName)
			throws Exception {
		Connection con = null;
		try {
			con = DriverManager.getConnection(databaseUrl);
			Statement s1 = con.createStatement();

			String sql1 = "SELECT count(*) FROM (SELECT " + columnName + " FROM " + tableName + " GROUP BY "
					+ columnName + " HAVING count(*) = 1) as result";

			ResultSet rs1 = s1.executeQuery(sql1);

			double countNotDuplicated;
			if (rs1.next()) {
				countNotDuplicated = rs1.getInt(1);
			} else {
				return -1;
			}

			String sql2 = "SELECT count(*) FROM " + tableName;

			ResultSet rsTotal2 = s1.executeQuery(sql2);

			double countTotal;
			if (rsTotal2.next()) {
				countTotal = rsTotal2.getInt(1);
			} else {
				return -1;
			}

			return (countNotDuplicated * 100) / countTotal;

		} catch (Exception e) {

			return 0;
		} finally {
			con.close();
		}

	}

	/* CATALOGO AGGREGATION SUM */
	@WebMethod
	public double aggSum(
			@XmlElement(name = "databaseUrl") String databaseUrl,
			@XmlElement(name = "columnName") String columnName, 
			@XmlElement(name = "tableName") String tableName)
			throws Exception {
		Connection con = null;
		try {
			con = DriverManager.getConnection(databaseUrl);
			Statement s1 = con.createStatement();

			String sql1 = "SELECT sum(" + columnName + ") FROM " + tableName;

			ResultSet rs1 = s1.executeQuery(sql1);

			double sum;
			if (rs1.next()) {
				sum = rs1.getDouble(1);
			} else {
				return -1;
			}

			return sum;

		} catch (Exception e) {

			return 0;
		} finally {
			con.close();
		}

	}

	/* CATALOGO AGGREGATION PROMEDIO */
	@WebMethod
	public double aggAvg(
			@XmlElement(name = "databaseUrl") String databaseUrl,
			@XmlElement(name = "columnName") String columnName, 
			@XmlElement(name = "tableName") String tableName)
			throws Exception {
		Connection con = null;
		try {
			con = DriverManager.getConnection(databaseUrl);
			Statement s1 = con.createStatement();


			String sql2 = "SELECT AVG(" + columnName + ") FROM " + tableName;

			ResultSet rs2 = s1.executeQuery(sql2);

			double avg;
			if (rs2.next()) {
				avg = rs2.getDouble(1);
			} else {
				return -1;
			}

			return avg;

		} catch (Exception e) {

			return 0;
		} finally {
			con.close();
		}

	}

	/* CATALOGO AGGREGATION OR */
	@WebMethod
	public boolean aggOr(
			@XmlElement(name = "databaseUrl") String databaseUrl,
			@XmlElement(name = "columnName") String columnName, 
			@XmlElement(name = "tableName") String tableName)
			throws Exception {
		Connection con = null;
		try {
			con = DriverManager.getConnection(databaseUrl);
			Statement s1 = con.createStatement();

			String sql1 = "SELECT " + columnName + " FROM " + tableName;

			ResultSet rs1 = s1.executeQuery(sql1);

			boolean result = true;
			while (rs1.next()) {
				result = result || rs1.getBoolean(1);
			}

			return result;

		} catch (Exception e) {

			return false;
		} finally {
			con.close();
		}

	}

	/* CATALOGO AGGREGATION AND */
	@WebMethod
	public boolean aggAnd(@XmlElement(name = "databaseUrl") String databaseUrl,
			@XmlElement(name = "columnName") String columnName, @XmlElement(name = "tableName") String tableName)
			throws Exception {
		Connection con = null;
		try {
			con = DriverManager.getConnection(databaseUrl);
			Statement s1 = con.createStatement();

			String sql1 = "SELECT " + columnName + " FROM " + tableName;

			ResultSet rs1 = s1.executeQuery(sql1);

			boolean result = true;
			while (rs1.next()) {
				result = result && rs1.getBoolean(1);
			}

			return result;

		} catch (Exception e) {

			throw e;
		} finally {
			con.close();
		}

	}

	/* CASO DE ESTUDIO - MI 3*/
	@WebMethod
	public boolean startsWith(
			@XmlElement(required = false, defaultValue = "null") @WebParam(name = "data") String data,
			@WebParam(name = "prefix") String prefix) 
			throws Exception {
		if (data != null) {
			return data.startsWith(prefix);
		} else {
			return false;
		}
	}

	/* CASO DE ESTUDIO - MI 1*/
	@WebMethod
	public boolean isValidState(
			@XmlElement(required = false, defaultValue = "null") @WebParam(name = "data") String data)
			throws Exception {
		if (data != null) {
			return data.matches(
					"FaltaDespachar|Despachado|Cerrado|TomadoSupervisor|Archivado");/*
																					 * Para ignorar
																					 * mayusculas y
																					 * minusculas
																					 * deberiamos
																					 * ser: (?i)
																					 * FaltaDespachar
																					 * |Despachado|
																					 * Cerrado|
																					 * TomadoSupervisor
																					 * |Archivado
																					 */
		} else {
			return false;
		}
	}

	/*CASO DE ESTUDIO - MI 4*/
	@WebMethod
	public boolean endLessStartEqualsDuration(
			@XmlElement(required = false, defaultValue = "null") @WebParam(name = "startDate") @XmlJavaTypeAdapter(DateAdapter.class) Date startDate,
			@XmlElement(required = false, defaultValue = "null") @WebParam(name = "endDate") @XmlJavaTypeAdapter(DateAdapter.class) Date endDate,
			@XmlElement(required = true) @WebParam(name = "duration") double duration) throws Exception {
		if ((startDate != null) && (endDate != null)) {
			 return (((endDate.getTime() - startDate.getTime())/1000) == duration);
		} else {
			return false;
		}
	}
	
	/*CASO DE ESTUDIO - MI 6*/
	@WebMethod
	public boolean isEqualsOrGreaterThan(
			@XmlElement(required = false, defaultValue = "null", name = "value", nillable = true) Double value,
			@XmlElement(name = "ref") double ref) throws Exception {
		return value != null && value >= ref;

	}
	
	/* CATALOGO AGGREGATION SUM */
	@WebMethod
	public double sum(
			@XmlElement(name = "sum1") int sum1, 
			@XmlElement(name = "sum2") int sum2)
			throws Exception {
		try {
			return (sum1+sum2);

		} catch (Exception e) {
			return 0;
		}
	}
}