package dataqulity.webservices.dataqulity.webservices;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;

//URL: http://localhost:8081/DataQuality?wsdl
//@WebService
public class DataQuality {

	/**
	 * Celda DomainIntegrity
	 * 
	 * @param age
	 * @param minAge
	 * @param maxAge
	 * @return la edad esta en el rango [minAge, maxAge]
	 */
	@WebMethod
	public Boolean ageRank(@WebParam(name = "age") Integer age, @WebParam(name = "minAge") int minAge,
			@WebParam(name = "maxAge") int maxAge) {
		return age >= minAge && age <= maxAge;

	}

	/**
	 * Conjunto de celdas RelationIntegrity
	 * 
	 * @param age
	 * @param birthDate
	 * @return la fecha actual menos la fecha de nacimiento es igual a la edad
	 */
	@WebMethod
	public Boolean ageBirthDate(@WebParam(name = "age") Integer age, @WebParam(name = "birthDate") Date birthDate) {
		return (new Date()).getTime() - birthDate.getTime() == age;
	}

	List<Student> students = new ArrayList<Student>();

	/**
	 * Conjunto de celdas o tupla - DuplicateFree
	 * 
	 * @param ci
	 * @param name
	 * @param age
	 * @param birthDate
	 * @return La tupla esta duplicada
	 */
	@WebMethod
	public boolean studentDuplicate(@WebParam(name = "ci") String ci, @WebParam(name = "name") String name,
			@WebParam(name = "age") int age, @WebParam(name = "birthDate") Date birthDate, @WebParam(name = "phone") String phone) {
		Student student = new Student(ci, name, birthDate, age, phone);

		boolean contiene = this.students.contains(student);
		this.students.add(student);
		return contiene;
	}

	/**
	 * Tabla
	 * 
	 * @param urlBD
	 *            Url con el string de conexion incluyendo el usuario y la contraseña,
	 *            "jdbc:mysql://localhost:3306/facultad?user=root&password=admin"
	 * @param tableName
	 *            Nombre de la tabla
	 * @return Porcentaje de los valores no nulos en la tabla
	 * @throws Exception
	 * @throws SQLException
	 */
	@WebMethod
	public double valuesNotNullPorcentage(@WebParam(name = "urlBD") String urlBD,
			@WebParam(name = "tableName") String tableName) throws Exception {
		try {
			// Registrar el driver
			Class.forName("com.mysql.jdbc.Driver");
			Connection con = DriverManager.getConnection(urlBD);
			Statement s = con.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_READ_ONLY);
			String sql = "SELECT * FROM " + tableName;

			ResultSet rs = s.executeQuery(sql);
			ResultSetMetaData rsmd = rs.getMetaData();
			int columnsCount = rsmd.getColumnCount();
			int rowcount = 0;
			int nullCount = 0;

			while (rs.next()) {
				rowcount++;
				for (int i = 1; i <= columnsCount; i++) {
					if (rs.getObject(i) == null) {
						nullCount++;
					}
				}
			}

			return rowcount != 0 && columnsCount != 0 ? 100 - (nullCount * 100) / (rowcount * columnsCount) : 100;
		} catch (SQLException e) {
			e.printStackTrace();
			return -1;
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			throw new Exception(e);
		}

	}

	/**
	 * Integridad inter-relación - Ratio de integridad Conjunto de columnas
	 * 
	 * @param url
	 * @param column1
	 * @param table1
	 * @param column2
	 * @param table2
	 * @return Porcentaje de atributos de la columna 1 estan en la columna2
	 * @throws Exception
	 */
	@WebMethod
	public double studentSchool(@WebParam(name = "url") String url, @WebParam(name = "column1") String column1,
			@WebParam(name = "table1") String table1, @WebParam(name = "column2") String column2,
			@WebParam(name = "table2") String table2) throws Exception {

		// Registrar el driver
		try {
			Class.forName("com.mysql.jdbc.Driver");

			Connection con1 = DriverManager
					.getConnection("jdbc:mysql://localhost:3306/facultad?user=root&password=admin");
			Statement s1 = con1.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_READ_ONLY);
			String sql1 = "SELECT " + column1 + " FROM " + table1;

			ResultSet rsColumn1 = s1.executeQuery(sql1);

			int containsCount = 0;
			int columns1RegisterCount = 0;

			while (rsColumn1.next()) {
				columns1RegisterCount++;
				Connection con2 = DriverManager
						.getConnection("jdbc:mysql://localhost:3306/facultad?user=root&password=admin");
				Statement s2 = con2.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_READ_ONLY);

				String sql2 = "SELECT " + column2 + " FROM " + table2;

				ResultSet rsColumn2 = s2.executeQuery(sql2);
				boolean contains = false;
				while (rsColumn2.next() && !contains) {
					if (rsColumn1.getObject(1).equals(rsColumn2.getObject(1))) {
						contains = true;
					}
				}
				if (contains)
					containsCount++;
			}

			return columns1RegisterCount != 0 ? containsCount * 100 / columns1RegisterCount : 100;
		} catch (ClassNotFoundException | SQLException ex) {
			ex.printStackTrace();
			throw new Exception(ex);
		}
	}

	/**
	 * Correctitud sintáctica Columna
	 * 
	 * @param url
	 * @param columnName
	 * @param tableName
	 * @param longPhoneNumber
	 *            largo esperado de los numeros de telefono
	 * @return Porcentaje de numeros de telefono que tienen el largo esperado
	 * @throws Exception
	 */
	@WebMethod
	public double phoneNumberCorrect(@WebParam(name = "url") String url,
			@WebParam(name = "columnName") String columnName, @WebParam(name = "tableName") String tableName,
			@WebParam(name = "longPhoneNumber") int longPhoneNumber) throws Exception {
		try {
			Class.forName("com.mysql.jdbc.Driver");

			Connection con = DriverManager
					.getConnection("jdbc:mysql://localhost:3306/facultad?user=root&password=admin");
			Statement s = con.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_READ_ONLY);
			String sql = "SELECT " + columnName + " FROM " + tableName;

			ResultSet rs = s.executeQuery(sql);

			int rowcount = 0;
			int phoneOk = 0;

			while (rs.next()) {
				rowcount++;

				if (!rs.getString(1).isEmpty() && rs.getString(1).length() == longPhoneNumber) {
					phoneOk++;
				}
			}

			return rowcount != 0 ? (phoneOk * 100) / (rowcount) : 100;
		} catch (ClassNotFoundException | SQLException e) {
			//
			e.printStackTrace();
			throw new Exception(e.getMessage());
		}
	}

	/**
	 * Base de datos
	 * 
	 * @param url
	 * @return Porcentaje de registros vacios
	 * @throws Exception
	 */
	@WebMethod
	public double emptyRegisters(@WebParam(name = "url") String url) throws Exception {
		try {

			int registerCount = 0;
			int registerEmptyCount = 0;

			Class.forName("com.mysql.jdbc.Driver");

			Connection con = DriverManager
					.getConnection("jdbc:mysql://localhost:3306/facultad?user=root&password=admin");
			Statement s = con.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_READ_ONLY);
			DatabaseMetaData metaDatos = con.getMetaData();
			ResultSet tables = metaDatos.getTables(null, null, "%", null);

			while (tables.next()) {
				String sql = "SELECT * FROM " + tables.getString(3);
				ResultSet rs = s.executeQuery(sql);

				while (rs.next()) {
					registerCount++;

					if (!rs.getString(1).isEmpty()) {
						registerEmptyCount++;
					}
				}
			}

			return registerCount != 0 ? (registerEmptyCount * 100) / (registerCount) : 100;
		} catch (ClassNotFoundException |

				SQLException e) {
			//
			e.printStackTrace();
			throw new Exception(e.getMessage());
		}
	}

}
