package com.kiuplus.logic.dataAccess;

import java.util.*;

import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.extractors.*;
import com.kiuplus.logic.model.AttributeColumn;
import com.kiuplus.logic.model.AttributeDatabase;
import com.kiuplus.logic.model.AttributeTable;
import com.kiuplus.logic.objects.*;

public class DataAccess {
	
	/**********************
     * Atributos privados *
     **********************/
	private String dataBaseConnectionString;
	private String dataBaseUser;
	private String dataBasePassword;
	private Extractor dataBase;
	
	/***************
     * Constructor *
     ***************/
	public DataAccess(String server, String user, String password){
		this.dataBaseConnectionString = server;
		this.dataBaseUser = user;
		this.dataBasePassword = password;
	}
	
	
	/***********
     * Getters *
     ***********/
	public String getDataBaseConnectionString() {
		return dataBaseConnectionString;
	}
	public String getDataBaseUser() {
		return dataBaseUser;
	}
	public String getDataBasePassword() {
		return dataBasePassword;
	}
	public Extractor getDataBase() {
		return dataBase;
	}
	
	/***********
     * Setters *
     ***********/
	public void setDataBaseConnectionString(String dataBaseConnectionString) {
		this.dataBaseConnectionString = dataBaseConnectionString;
	}
	public void setDataBaseUser(String dataBaseUser) {
		this.dataBaseUser = dataBaseUser;
	}
	public void setDataBasePassword(String dataBasePassword) {
		this.dataBasePassword = dataBasePassword;
	}
	public void setDataBase(Extractor dataBase) {
		this.dataBase = dataBase;
	}

	/**************************
     * Conexin y Desconexin *
     **************************/
	public void connect () throws Exception{
		this.dataBase.Connect();
	}
	public void disconnect ()throws Exception{
		this.dataBase.Disconnect();
	}
	
	/*****************
     * Transacciones *
     *****************/
	public void setAutoCommit(boolean autoCommit) throws Exception{
		this.dataBase.setAutoCommit(autoCommit);
	}
	public void commit()throws Exception{
		this.dataBase.commit();
	}
	public void rollback()throws Exception{
		this.dataBase.rollback();
	}
	
	/**********************************
     * DataBase y Table (create/drop) *
     **********************************/
	/*
	public void createDataBase(String dataBaseName){
		this.dataBase.ExecuteUpdate(String.format("CREATE DATABASE %s", dataBaseName));
	}
	public void dropDataBase(String dataBaseName){
		this.dataBase.ExecuteUpdate(String.format("DROP DATABASE %s", dataBaseName));
	}
	*/
	public void dropTable(String tableName){
		this.dataBase.ExecuteUpdate(String.format("DROP TABLE IF EXISTS %s", tableName));
	}
	/*
	public void truncateTable(String tableName){
		this.dataBase.ExecuteUpdate(String.format("TRUNCATE TABLE %s", tableName));
	}
	*/
	
	/***************************************************************
	 *                                                             *
     * Source DataBase: ExecuteServiceDAOImpl, DatabaseServiceImpl *
     *                                                             * 
     ***************************************************************
     ***  Analizar si hay que re-implementar para un nuevo motor ***
     ***************************************************************/
	public List<String> getTablesName(String schemaName){
		//Hay que implementarla para otras BDs
		return null;
	}
	
	public List<GenericObject> getColumnsNamesCompleteTable(String schemaName, String tableName){
		Map<Integer, Object> parameters = new HashMap<Integer, Object>();
		parameters.put(1, schemaName);
		parameters.put(2, tableName);
		String query = "SELECT * "
						+"FROM INFORMATION_SCHEMA.COLUMNS "
						+"WHERE TABLE_SCHEMA = ? AND TABLE_NAME = ?";
		List<GenericObject> result = this.dataBase.ExecuteQuery(query, parameters);
		return result;
	}
	
	public List<GenericObject> getPKColumnsAndColumnsNamesPartialTable(String schemaName, String tableName, List<String> columnsName){
		//Hay que implementarla para otras BDs
		return null;
	}
	
	public List<String> getTableAttributeStringList (List<GenericObject> attributeList){
		//Hay que implementarla para otras BDs
		return null;
	}
	
	public List<GenericObject> getValues(String schemaName, String tableName, List<String> attributeStringList){
		//Hay que implementarla para otras BDs
		return null;
	}
	
	public List<AttributeColumn> getColumnsNamesTable(AttributeDatabase database, AttributeTable attributeTable) throws ExceptionKiuPlus{
		return null;
	}
	
	public AttributeColumn getColumnTypeParser(String schemaName, String tableName, String columnName)throws ExceptionKiuPlus{
		//Hay que implementarla para otras BDs
		return null;
	}
	
	public List<GenericObject> getColumnType(String schemaName, String tableName, String columnName){	
		Map<Integer, Object> parameters = new HashMap<Integer, Object>();
		parameters.put(1, schemaName);
		parameters.put(2, tableName);
		parameters.put(3, columnName);
		String query = "SELECT * "
						+"FROM INFORMATION_SCHEMA.COLUMNS "
						+"WHERE TABLE_SCHEMA = ? AND TABLE_NAME = ? AND COLUMN_NAME = ?";
		List<GenericObject> result = this.getDataBase().ExecuteQuery(query, parameters);
		return result;
	}
	
	public List<String> attributeListToAttributeTypeStringList(List<GenericObject> attributeList){
		//Hay que implementarla para otras BDs
		return null;
	}
	
	/******************************
     * Target DataBase (MetaData) *
     ******************************/
	public void createDataTable(String tableName, List<String> attributeTypeStringList){
		this.dataBase.ExecuteUpdate("DROP TABLE IF EXISTS " + tableName + ";"+
									"CREATE TABLE " + tableName + " " +
									"( " +
									    "id_metadata_" + tableName + " int(11) NOT NULL AUTO_INCREMENT PRIMARY KEY, " +
									    String.join(", ", attributeTypeStringList) +
									");");
	}
	
	public void createIMResultTableG1(int imId, String resultType){
		String tableName = "metadata_im_" + imId;
		this.dataBase.ExecuteUpdate("DROP TABLE IF EXISTS " + tableName + "; " +
									"CREATE TABLE " + tableName + " " +
									"( " +
									    "value int(11) NOT NULL, " +
									    "result " + resultType + " DEFAULT NULL" +
									") ");
	}
	
	public void createIMResultTableG2(int imId, String resultType){
		String tableName = "metadata_im_" + imId;
		this.dataBase.ExecuteUpdate("CREATE TABLE IF NOT EXISTS " + tableName + " " +
									"( " +
									    "id int(11) NOT NULL AUTO_INCREMENT PRIMARY KEY," +
									    "result " + resultType + " DEFAULT NULL," +
									    "timestamp datetime(1) NOT NULL" +
									") ");
	}
	
	public void saveIMResultG1(int imId, int resultId, Object result){
		String tableName = "metadata_im_" + imId;
		Map<Integer, Object> parameters = new HashMap<Integer, Object>();
		parameters.put(1, Integer.toString(resultId));
		parameters.put(2, result);
		String query = "INSERT INTO " + tableName + " (value, result) VALUES(?, ?)";
		this.dataBase.ExecuteUpdate(query, parameters);
	}
	
	public void saveIMResultG2(int imId, Object result, Date timestamp){
		String tableName = "metadata_im_" + imId;
		Map<Integer, Object> parameters = new HashMap<Integer, Object>();
		parameters.put(1, result);
		parameters.put(2, timestamp);
		String query = "INSERT INTO " + tableName + " (result, timestamp) VALUES(?, ?)";
		this.dataBase.ExecuteUpdate(query, parameters);
	}
	
	public void save(List<String> attributeStringList, GenericObject tuple, String tableName){
		try {
			int key = 1;
			String query = "";
			String values = "";
			Map<Integer, Object> parameters = new HashMap<Integer, Object>();
			for (String columnName : attributeStringList){
					query += columnName + ",";
					values += "?,"; 
					parameters.put(key, tuple.getProperty(columnName));
					key++;
			}
			query = query.substring(0,query.length()-1);
			values = values.substring(0,values.length()-1);
			query = "INSERT INTO " + tableName + " (" + query + ") VALUES(" + values + ")";
			this.dataBase.ExecuteUpdate(query, parameters);
		} catch (Exception e) {
			throw e;
		}
	}
	
	public List<GenericObject> getResults(String tableResultName, String tableDataName, List<String> attributeStringList){
		
		String query = String.format("SELECT %s, %s.result "
									 + "FROM %s "
									 + 		"left join %s on %s.value = %s.id_metadata_%s",
									 String.join(", ", attributeStringList), tableResultName, tableResultName,tableDataName, tableResultName,tableDataName, tableDataName);
		
		List<GenericObject> resultList = dataBase.ExecuteQuery(query);
		//Coloca el resultado en la ltima columna
		for (GenericObject go : resultList){
			String result = go.remove("result");
			go.setProperty("Result", result);
		}
		return resultList;
	}
	
	public List<GenericObject> getColumnResults(String tableResultName){
		
		
		List<GenericObject> resultList = dataBase.ExecuteQuery("SELECT result, date_format(timestamp, '%d/%m/%Y %T') as 'Execution date' "
				 + "FROM " + tableResultName);
		//Coloca el resultado en la ltima columna
		for (GenericObject go : resultList){
			String result = go.remove("result");
			go.setProperty("Result", result);
			
		}
		return resultList;
	}
	
	public void setDataBase(String dataBaseName){
		this.dataBase.ExecuteUpdate(String.format("CREATE DATABASE IF NOT EXISTS %s; USE %s", dataBaseName, dataBaseName));
	}
	
	public void createAggregationResultTable(int aggId, String resultType){
		String tableName = "metadata_agg_" + aggId;
		this.dataBase.ExecuteUpdate("CREATE TABLE IF NOT EXISTS " + tableName + " " +
									"( " +
									    "id int(11) NOT NULL AUTO_INCREMENT PRIMARY KEY," +
									    "result " + resultType + " DEFAULT NULL," +
									    "im_dateTime_result datetime(1) NOT NULL," +
									    "timestamp datetime(1) NOT NULL" +
									") ");
	}
	
	public void saveAggregationResult(int aggId, Object result, Date imDateTimeResult){
		String tableName = "metadata_agg_" + aggId;
		Map<Integer, Object> parameters = new HashMap<Integer, Object>();
		parameters.put(1, result);
		parameters.put(2, imDateTimeResult);
		String query = "INSERT INTO " + tableName + " (result, im_dateTime_result, timestamp) VALUES(?, ?, NOW())";
		this.dataBase.ExecuteUpdate(query, parameters);
	}

	public List<GenericObject> getAggregationResults(String tableName){		
		List<GenericObject> result = dataBase.ExecuteQuery("SELECT result as 'Result', date_format(timestamp, '%d/%m/%Y %T') as 'Aggregation execution date', date_format(im_dateTime_result, '%d/%m/%Y %T') as 'Instanced metric execution date' FROM " + tableName + " ORDER BY timestamp desc");
		return result;
	}
	/*********
	 * Otros *
	 *********/
	public Schema getDataBaseSchema(String string) {
		return null;
	}

	
}
