package com.kiuplus.logic.dataAccess;

import java.util.*;
import com.kiuplus.logic.objects.*;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.extractors.*;
import com.kiuplus.logic.model.AttributeColumn;
import com.kiuplus.logic.model.AttributeDatabase;
import com.kiuplus.logic.model.AttributeTable;

public class MySQLDataAccess extends DataAccess{
	
	/***************
     * Constructor *
     ***************/
	public MySQLDataAccess(String server, String user, String password) {
		super(server, user, password);
		super.setDataBase(new MySQL(server, user, password));
	}
	
	/***************************************************************
     * Source DataBase: ExecuteServiceDAOImpl, DatabaseServiceImpl *
     ***************************************************************/
	@Override
	public List<String> getTablesName(String schemaName){
		List<String> result = new ArrayList<String>();
		Map<Integer, Object> parameters = new HashMap<Integer, Object>();
		parameters.put(1, schemaName);
		String query = "SELECT * FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = ?";
		for (GenericObject go : this.getDataBase().ExecuteQuery(query, parameters)){
			result.add(go.getProperty("TABLE_NAME"));
		}
		return result;
	}
	
	//public List<GenericObject> getColumnsNamesCompleteTable(String schemaName, String tableName)
	
	@Override
	public List<GenericObject> getPKColumnsAndColumnsNamesPartialTable(String schemaName, String tableName, List<String> columnsName){
		String query = "";
		Map<Integer, Object> parameters = new HashMap<Integer, Object>();
		parameters.put(1, schemaName);
		parameters.put(2, tableName);
		int key = 3;
		query += "AND (TABLE_NAME = ? AND "
				+"(COLUMN_KEY IN('PRI', 'UNI') ";
		for (String columnName : columnsName){
			parameters.put(key, columnName);
			key++;
			query += "OR COLUMN_NAME = ? ";
		}
		query = "SELECT * "
				+"FROM INFORMATION_SCHEMA.COLUMNS "
				+"WHERE TABLE_SCHEMA = ? "
				+ query +  "))";
		List<GenericObject> result = super.getDataBase().ExecuteQuery(query, parameters);
		return result;
	}
	
	@Override
	public List<String> getTableAttributeStringList (List<GenericObject> attributeList){
		List<String> result = new ArrayList<String>();
		attributeList.forEach((temp) -> {
			result.add(temp.getProperty("COLUMN_NAME"));
		});
		return result;
	}
	
	@Override
	public List<GenericObject> getValues(String schemaName, String tableName, List<String> attributeStringList){
		return super.getDataBase().ExecuteQuery(String.format("SELECT %s FROM %s", String.join(", ", String.join(", ", attributeStringList)), tableName));
	}
	
	@Override
	public List<AttributeColumn> getColumnsNamesTable(AttributeDatabase database, AttributeTable attributeTable) throws ExceptionKiuPlus{
		List<AttributeColumn> columns = new ArrayList<>();
		for (GenericObject object : this.getColumnsNamesCompleteTable(database.getName(), attributeTable.getName())) {
			java.util.Map<String, String> o = object.getProperties();
			AttributeColumn attribute = new AttributeColumn(o.get("COLUMN_NAME"), AttributeColumn.dataTypeColumn(o.get("COLUMN_TYPE")));
			attribute.setAttributeTable(attributeTable);
			columns.add(attribute);
		}
		return columns;
	}
	
	@Override
	public AttributeColumn getColumnTypeParser(String schemaName, String tableName, String columnName) throws ExceptionKiuPlus{	
		AttributeColumn attribute = null;
		List<GenericObject> columnType = super.getColumnType(schemaName, tableName, columnName);
		if (!columnType.isEmpty()) {
			java.util.Map<String, String> o = columnType.get(0).getProperties();
			attribute =  new AttributeColumn(o.get("COLUMN_NAME"), AttributeColumn.dataTypeColumn(o.get("COLUMN_TYPE"))); 
		}
		return attribute;
	}
	
	@Override
	public List<String> attributeListToAttributeTypeStringList(List<GenericObject> attributeList){
		List<String> result = new ArrayList<String>();
		attributeList.forEach((temp) -> {
			result.add("" + temp.getProperty("COLUMN_NAME") + " " + temp.getProperty("COLUMN_TYPE"));
		});
		return result;
	}
	
	/*********
	 * Otros *
	 *********/
	public Schema getDataBaseSchema(String schemaName){
		Map<Integer, Object> parameters = new HashMap<Integer, Object>();
		parameters.put(1, schemaName);
		String query = "SELECT TABLE_NAME, COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = ? ";
		
		Schema result = new Schema();
		for (GenericObject go : super.getDataBase().ExecuteQuery(query, parameters)){
			if (!go.getProperty("TABLE_NAME").startsWith("metadata_agg_")
					&& !go.getProperty("TABLE_NAME").startsWith("metadata_im_")
					&& !go.getProperty("TABLE_NAME").startsWith("metadata_cim_")
					&& !go.getProperty("COLUMN_NAME").startsWith("id_metadata_")){
				result.add(go.getProperty("TABLE_NAME"), go.getProperty("COLUMN_NAME"));
			}
		}
		return result;
	}


	/*
	public List<String> getttributeStringList (List<GenericObject> attributeList){
		List<String> result = new ArrayList<String>();
		attributeList.forEach((temp) -> {
			result.add(temp.getProperty("COLUMN_NAME"));
		});
		return result;
	}
	public String dataBaseType(String enumDataType){
		switch (enumDataType) {
        	case "int": return "INT";
        	case "double": return "DOUBLE";
        	case "bool": return "BIT";
        	case "nvarchar": return "TEXT";
        	case "datetime": return "DATETIME(3)";
        	default: return "TEXT";
		}
	}
	*/
}
