package com.kiuplus.logic.extractors;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import com.kiuplus.logic.objects.*;

public class Extractor {
	
	/**********************
     * Atributos privados *
     **********************/
	private String  user;
    private String  password;
    private String  server;
    private Connection connection  = null;
    
    /*****************
     * Constructores *
     *****************/
    public Extractor(String user, String password, String server) {
		this.user = user;
		this.password = password;
		this.server = server;
		this.connection = null;
	}
	
    /***********
     * Getters *
     ***********/
    public String getUser() {
		return user;
	}
    public String getPassword() {
		return password;
	}
    public String getServer() {
		return server;
	}
	public Connection getConnection() {
		return connection;
	}
	
	/***********
     * Setters *
     ***********/	
	public void setUser(String user) {
		this.user = user;
	}
	public void setPassword(String password) {
		this.password = password;
	}
	public void setServer(String server) {
		this.server = server;
	}
	public void setConnection(Connection connection) {
		this.connection = connection;
	}
	
	
	/**************************
     * Conexin y desconexin *
     **************************/
	public void Connect() throws Exception{
		//Se tiene que implemetar para cada extractor
	};
	public void Disconnect() throws Exception{
    	try {
			connection.close();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw e;
		}
    }
	
	/*****************
     * Transacciones *
     *****************/
	public void setAutoCommit(boolean autoCommit) throws Exception{
		this.connection.setAutoCommit(autoCommit);
	}
	public void commit()throws Exception{
		this.connection.commit();
	}
	public void rollback()throws Exception{
		this.connection.rollback();
	}
	
	/*********
     * Query *
     *********/
	public List<GenericObject> ExecuteQuery(String query){
    	return this.ExecuteQuery(query, null);
    }
	public List<GenericObject> ExecuteQuery(String query, Map<Integer, Object> parameters){
    	PreparedStatement pstmt = null;
    	ResultSet rs = null;
    	List<GenericObject> result = new ArrayList<GenericObject>();
    	try {
    	    pstmt = connection.prepareStatement(query);
    	    if(parameters != null){
                for(Integer key : parameters.keySet()){
                	Object obj = parameters.get(key);
                	if (obj == null){
                		pstmt.setNull(key, java.sql.Types.INTEGER);
                	} else if (obj.getClass().equals(String.class)){
                		pstmt.setString(key, (String) obj);
                	} else if (obj.getClass().equals(Boolean.class)){
                		pstmt.setBoolean(key, (Boolean) obj);
                	} else if (obj.getClass().equals(Integer.class)){
                		pstmt.setInt(key, (Integer) obj);
                	} else if (obj.getClass().equals(Double.class)){
                		pstmt.setDouble(key, (Double) obj);
                	} else{
                		pstmt.setObject(key, obj);
                	}
                }
            }    
    	    rs = pstmt.executeQuery();
    	    int columnCount = rs.getMetaData().getColumnCount();
    	    while ( rs.next() ) {
    	    	GenericObject row = new GenericObject();
	    	    for (int i = 1; i <= columnCount; ++i) {
	    	        row.setProperty(rs.getMetaData().getColumnName(i), rs.getString(i)); 
	    	    }
	    	    result.add(row);
    	    }
    	}
    	catch (Exception ex){
    		System.err.println("ERROR");
            System.err.println(ex);
    	}
    	finally {
    	    if (rs != null) {
    	        try {
    	            rs.close();
    	        } catch (SQLException sqlEx) { } // ignore

    	        rs = null;
    	    }

    	    if (pstmt != null) {
    	        try {
    	            pstmt.close();
    	        } catch (SQLException sqlEx) { } // ignore
    	        pstmt = null;
    	    }
    	}
		return result;
    }
	
	/*********
     * Update *
     *********/
	public int ExecuteUpdate(String query){
		return this.ExecuteUpdate(query, null);
    }    
    public int ExecuteUpdate(String query, Map<Integer, Object> parameters){
    	PreparedStatement pstmt = null;
    	int result = 0;
    	try {
    	    pstmt = connection.prepareStatement(query);
    	    if(parameters != null){
                for(Integer key : parameters.keySet()){
                	Object obj = parameters.get(key);
                	if (obj == null){
                		pstmt.setNull(key, java.sql.Types.INTEGER);
                	} else if (obj.getClass().equals(String.class)){
                		pstmt.setString(key, (String) obj);
                	} else if (obj.getClass().equals(Boolean.class)){
                		pstmt.setBoolean(key, (Boolean) obj);
                	} else if (obj.getClass().equals(Integer.class)){
                		pstmt.setInt(key, (Integer) obj);
                	} else if (obj.getClass().equals(Double.class)){
                		pstmt.setDouble(key, (Double) obj);
                	} else{
                		pstmt.setObject(key, obj);
                	}
                }
            }    
    	    result = pstmt.executeUpdate();
    	    
    	}
    	catch (Exception ex){
    		System.err.println("ERROR");
            System.err.println(ex);
    	}
    	finally {
    	    if (pstmt != null) {
    	        try {
    	            pstmt.close();
    	        } catch (SQLException sqlEx) { } // ignore
    	        pstmt = null;
    	    }
    	}
		return result;
    }

}
