package com.kiuplus.logic.model;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.print.attribute.standard.DateTimeAtCompleted;

import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.Parameter.EnumDataType;

@Entity
@Table(name = "attributes_column")
@PrimaryKeyJoinColumn(name = "idAttribute", referencedColumnName = "id")
public class AttributeColumn extends Attribute {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private EnumDataType type;

	@ManyToOne(targetEntity = AttributeTable.class, optional = false, fetch = FetchType.EAGER, cascade=CascadeType.ALL)
	@JoinColumn(name = "idAttributeTable", insertable = true, updatable = true)
	private AttributeTable attributeTable;

	public AttributeColumn() {
	}

	public AttributeColumn(Integer id, String name, AttributeTable attributeTable) {
		super(id, name);
		this.attributeTable = attributeTable;
	}

	public AttributeColumn(Integer id, String name, EnumDataType type, AttributeTable attributeTable) {
		super(id, name);
		this.type = type;
		this.attributeTable = attributeTable;
	}

	public AttributeColumn(String name, EnumDataType type) {
		super(name);
		this.type = type;
		this.attributeTable = new AttributeTable();
	}

	
	// public AttributeColumn(int id, String name, InstancedMetric instancedMetric, String type,
	// ParameterAttribute parameterAttribute, AttributeTable attributeTable) {
	// super(id, name, instancedMetric, EnumAttributeType.COLUMNS, parameterAttribute);
	// this.type = type;
	// this.attributeTable = attributeTable;
	// }

	// public AttributeColumn(int id, String name, InstancedMetric instancedMetric, String type,
	// AttributeTable attributeTable) {
	// super(id, name, instancedMetric, EnumAttributeType.COLUMNS);
	// this.type = type;
	// this.attributeTable = attributeTable;
	// }

	public EnumDataType getType() {
		return type;
	}

	public void setType(EnumDataType type) {
		this.type = type;
	}

	public AttributeTable getAttributeTable() {
		return attributeTable;
	}

	public void setAttributeTable(AttributeTable attributeTable) {
		this.attributeTable = attributeTable;
	}

	// public String getFullName() {
	//
	// return (this.attributeTable != null ? this.attributeTable.getName() : "") + " " +
	// this.getName();
	// }

	public EnumAttributeType getTypeAttribute() {
		return EnumAttributeType.COLUMNS;
	}

	public Integer databaseIDGet() {
		return this.attributeTable.getAttributeDatabase().getId();
	}
	
	public Integer tableIDGet() {
		return this.attributeTable.getId();
	}
	// public void setFullName(String fullName) {
	// String[] fullNameArray = fullName.split(" ");
	//
	// // Esto no deberia ser asi, pero cuando edito una metrica instanciada puede venir el
	// // attributeColumn null
	// if (fullNameArray.length > 1) {
	// this.setName(fullNameArray[1]);
	// } else {
	// this.setName("");
	// }
	// // this.attributeTable.setName(fullNameArray[0]);
	// }

	// public String getTableName() {
	// return this.attributeTable != null ? this.attributeTable.getName() : "";
	// }

	// public void setTableName(String name) {
	// this.attributeTable.setName(name);
	// }

	// public String getColumnName() {
	// return this.getName();
	// }
	//
	// public void setTableColumn(String name) {
	// this.setName(name);
	// }

	/**
	 * Se utiliza en la interfaz para mostrar las opciones al seleccionar la columna
	 * 
	 * @return nombre (type)
	 */
	public String getNameType() {
		return this.getName() + " (" + this.type.getText() + ")";
	}

	// TODO: Esto no va aca, Para mysql
	public static EnumDataType dataTypeColumn(String text) throws ExceptionKiuPlus {
		text = text.toUpperCase();
		if (text.equals("BIT(1)"))
			return EnumDataType.BOOL;
		if (text.contains("INT") || text.contains("BIT") || text.contains("YEAR"))
			return EnumDataType.INTEGER;
		if (text.contains("DATE") || text.contains("TIME"))
			return EnumDataType.DATETIME;
		if (text.contains("DECIMAL") || text.contains("FLOAT") || text.contains("DOUBLE") || text.contains("NUMERIC")
				|| text.contains("REAL"))
			return EnumDataType.DOUBLE;
		if (text.contains("CHAR") || text.contains("BINARY") || text.contains("BLOB") || text.contains("TEXT")
				|| text.contains("ENUM") || text.contains("SET") || text.contains("ENUM") || text.contains("SET")
				|| text.contains("JSON"))
			return EnumDataType.STRING;

		throw new ExceptionKiuPlus("The data type " + text + " is not supported");
	}
// TODO: Esto no va aca
	public static EnumDataType dataTypeColumnPostgres(String text) throws ExceptionKiuPlus {
		text = text.toUpperCase();
		if (text.equals("BOOLEAN"))
			return EnumDataType.BOOL;
		if (text.contains("BIT") || text.contains("BIGSERIAL") || text.contains("BYTEA") || text.contains("CHARACTER")
				|| text.contains("INTEGER") || text.contains("SMALLINT") || text.contains("SMALLSERIAL")
				|| text.contains("SERIAL"))
			return EnumDataType.INTEGER;
		if (text.contains("DATE") || text.contains("INTERVAL") || text.contains("TIME"))
			return EnumDataType.DATETIME;
		if (text.contains("DOUBLE") || text.contains("MONEY") || text.contains("NUMERIC") || text.contains("REAL"))
			return EnumDataType.DOUBLE;
		if (text.contains("CHARACTER") || text.contains("CIDR") || text.contains("INET") || text.contains("JSON")
				|| text.contains("MACADDR") || text.contains("TEXT") || text.contains("TSQUERY")
				|| text.contains("TSVECTOR") || text.contains("UUID") || text.contains("XML"))
			return EnumDataType.STRING;

		throw new ExceptionKiuPlus("The data type " + text + " is not supported");
	}
}
