package com.kiuplus.logic.model;

import java.nio.charset.StandardCharsets;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.List;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.SecretKeySpec;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.hibernate.validator.constraints.NotBlank;

import com.fasterxml.jackson.annotation.JsonBackReference;
import com.kiuplus.logic.exception.ExceptionKiuPlus;

@Entity
@Table(name = "attributes_database")
@PrimaryKeyJoinColumn(name = "idAttribute", referencedColumnName = "id")
public class AttributeDatabase extends Attribute {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private static final String KEY = "juw!5s5sDw32ow/l";

	
	public enum EnumDatabaseSystem {
		MySQL, PostgreSQL
	}

	@Size(max = 2000)
	@NotBlank
	@Column(name = "url")
	private String url;

	@Size(max = 45)
	@NotBlank
	@Column(name = "user")
	private String user;

//	@Size(max = 45)
//	@NotBlank
//	@Column(insertable = false, updatable = false)
//	private String password;

	@Column(name = "password")
	private byte[] passwordEncrypted;

	@NotNull(message="Select a system")
	@Column(name = "_system")
	private EnumDatabaseSystem system;

	@Size(max = 200)
	@NotBlank
	@Column(name = "_schema")
	private String schema;
	
	@JsonBackReference
	@OneToMany(targetEntity = AttributeTable.class, fetch = FetchType.EAGER, orphanRemoval=true, cascade = CascadeType.REFRESH)
	@JoinColumn(name = "idAttributeDatabase", insertable = false, updatable = false)
	private List<AttributeTable> attributesTable;

	public AttributeDatabase() {
		this.attributesTable = new ArrayList<>();
	}

	public AttributeDatabase(Integer id, String name, String url, String user, String password, String schema, EnumDatabaseSystem system) throws ExceptionKiuPlus {
		super(id, name);
		this.url = url;
		this.user = user;
		this.system = system;
		this.attributesTable = new ArrayList<>();
		this.schema = schema;
		this.setPassword(password);
	}

	public AttributeDatabase(String name, String url, String user, String password, String schema, EnumDatabaseSystem system) throws ExceptionKiuPlus {
		super(name);
		this.url = url;
		this.user = user;
		this.system = system;
		this.schema = schema;
		this.attributesTable = new ArrayList<>();
		this.setPassword(password);
	}
	
	public String getUrl() {
		return url;
	}

	public void setUrl(String url) {
		this.url = url;
	}

	public EnumDatabaseSystem getSystem() {
		return system;
	}

	public void setSystem(EnumDatabaseSystem system) {
		this.system = system;
	}

	public String getUser() {
		return user;
	}

	public void setUser(String user) {
		this.user = user;
	}

	public String getPassword() throws ExceptionKiuPlus {
		return this.decryptedPassword(this.passwordEncrypted);
	}

	public void setPassword(String password) throws ExceptionKiuPlus {
		this.passwordEncrypted = this.encpyptPassword(password);
	}

	public byte[] getPasswordEncrypted() {
		return passwordEncrypted;
	}

	public void setPasswordEncrypted(byte[] passwordEncrypted) {
		this.passwordEncrypted = passwordEncrypted;
	}

	public List<AttributeTable> getAttributesTable() {
		return attributesTable;
	}

	public void setAttributesTable(List<AttributeTable> attributesTable) {
		this.attributesTable = attributesTable;
	}

	public Integer tableIDGet() {
		return -1;
	}
	
//	public boolean validateDatabase(BindingResult result, String field) {
//
//		boolean error = false;
//		// if (this.name == "") {
//		// result.rejectValue(field + ".name", "blanckValue", new Object[] { "'"
//		// + field + ".name'" },
//		// "Can not be empty.");
//		// error = true;
//		// }
//
//		if (this.url == "") {
//			result.rejectValue(field + ".url", "blanckValue", new Object[] { "'" + field + ".url'" },
//					"Can not be empty.");
//			error = true;
//		}
//
//		if (this.system == null) {
//			result.rejectValue(field + ".system", "blanckValue", new Object[] { "'" + field + ".system'" },
//					"Select a opcion.");
//			error = true;
//		}
//
//		return error;
//	}

	public EnumAttributeType getTypeAttribute() {
		return EnumAttributeType.DATABASE;
	}

	public String getServer() {
		return this.url + "/" + this.getName();
	}

	
	public String getSchema() {
		return schema;
	}

	public void setSchema(String schema) {
		this.schema = schema;
	}

	/** 
	 * @param name
	 * @return atributo identificado con el nombre, si no lo enceuntra retorna null
	 */
	public AttributeTable findAttributeTable(String name) {

		for (AttributeTable attributeTable : this.attributesTable) {
			if (attributeTable.getName().equals(name)) {
				return attributeTable;
			}
		}

		return null;
	}

	// Encriptar y desencriptar contrasea
	/**
	 * @param password
	 * @return Encripta la password con el estandar AES
	 * @throws ExceptionKiuPlus
	 */
	private byte[] encpyptPassword(String password) throws ExceptionKiuPlus {

		try {
			SecretKeySpec keyAux = new SecretKeySpec((KEY).getBytes(StandardCharsets.UTF_8), "AES");

			Cipher aes = Cipher.getInstance("AES/ECB/PKCS5Padding");
			aes.init(Cipher.ENCRYPT_MODE, keyAux);
			return aes.doFinal(password.getBytes(StandardCharsets.UTF_8));
		} catch (NoSuchAlgorithmException | NoSuchPaddingException | InvalidKeyException | IllegalBlockSizeException
				| BadPaddingException e) {

			e.printStackTrace();

			throw new ExceptionKiuPlus("Could not encrypt password");

		}
	}

	/**
	 * @param passwordEncrypt
	 * @return Desencripta la contrasea con el estandar AES
	 * @throws ExceptionKiuPlus
	 */
	private String decryptedPassword(byte[] passwordEncrypt) throws ExceptionKiuPlus {
		try {
			if (passwordEncrypt != null) {
				SecretKeySpec keyAux = new SecretKeySpec((KEY).getBytes(StandardCharsets.UTF_8), "AES");
				Cipher aes = Cipher.getInstance("AES/ECB/PKCS5Padding");
				aes.init(Cipher.DECRYPT_MODE, keyAux);
				byte[] decrypted = aes.doFinal(passwordEncrypt);
				return new String(decrypted, StandardCharsets.UTF_8);
			} else {
				return "";
			}
		} catch (NoSuchAlgorithmException | NoSuchPaddingException | InvalidKeyException | IllegalBlockSizeException
				| BadPaddingException e) {

			e.printStackTrace();
			throw new ExceptionKiuPlus("Could not decrypt the password");
		}

	}

	

}
