package com.kiuplus.logic.model;

import java.io.Serializable;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.validation.Valid;
import javax.validation.constraints.Size;

import org.hibernate.validator.constraints.NotBlank;

import com.fasterxml.jackson.annotation.JsonBackReference;
import com.fasterxml.jackson.annotation.JsonManagedReference;

/**
 * @author Beatriz
 *
 */
@Entity
@Table(name = "data_models")
public class DataModel implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "id")
	private int id;

	@Size(max=45)
	@NotBlank
	private String name;

	@Valid
	@JsonManagedReference
	@OneToOne(optional = false, cascade = CascadeType.ALL, orphanRemoval = true)
	@JoinColumn(name = "idDatabaseSource", unique = true)
	private AttributeDatabase databaseSource;

	@JsonManagedReference
	@OneToOne(optional = true, cascade = CascadeType.ALL, orphanRemoval = true)
	@JoinColumn(name = "idDatabaseMetadata", unique = true)
	private AttributeDatabase databaseMetadata;
	

	@JsonBackReference
	@OneToMany(targetEntity=InstancedMetric.class,  fetch=FetchType.LAZY)
	@JoinColumn(name="idDataModel", insertable = false, updatable = false)
	private List<InstancedMetric> instancedMetrics;

	@JsonBackReference
	@OneToMany(targetEntity=ResultGroup.class,  fetch=FetchType.LAZY)
	@JoinColumn(name="idDataModel", insertable = false, updatable = false)
	private List<ResultGroup> resultGroups;
	
	
	public DataModel() {
		this.databaseMetadata = new AttributeDatabase();
	}
	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public AttributeDatabase getDatabaseSource() {
		return databaseSource;
	}

	public void setDatabaseSource(AttributeDatabase databaseSource) {
		this.databaseSource = databaseSource;
	}

	public AttributeDatabase getDatabaseMetadata() {
		return databaseMetadata;
	}

	public void setDatabaseMetadata(AttributeDatabase databaseMetadata) {
		this.databaseMetadata = databaseMetadata;
	}
	
	public List<InstancedMetric> getInstancedMetrics() {
		return instancedMetrics;
	}

	public void setInstancedMetrics(List<InstancedMetric> instancedMetrics) {
		this.instancedMetrics = instancedMetrics;
	}

	public List<ResultGroup> getResultGroups() {
		return resultGroups;
	}

	public void setResultGroups(List<ResultGroup> resultGroups) {
		this.resultGroups = resultGroups;
	}
	
	

}
