package com.kiuplus.logic.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.validation.constraints.Size;

import org.hibernate.validator.constraints.NotBlank;

import com.fasterxml.jackson.annotation.JsonManagedReference;



/**
 * Dimensions generated by hbm2java
 */
@Entity
@Table(name = "DIMENSIONS")
public class Dimension implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "ID")
	private int id;
	
	@Size(max=45)
	@NotBlank
	private String name;
	
	
	@OneToMany(targetEntity=Factor.class,  cascade = CascadeType.ALL, fetch=FetchType.LAZY)
	@JoinColumn(name="iddimension", insertable = false, updatable = false)
	@JsonManagedReference
	private List<Factor> factors;

	public Dimension() {
		this.factors = new ArrayList<Factor>();
	}

	public Dimension(String name, List<Factor> factors) {
		this.name = name;
		this.factors = factors;
	}
	
	public Dimension(int id, String name) {
		this.id = id;
		this.name = name;
		this.factors = new ArrayList<Factor>();
	}

	
	public int getId() {
		return this.id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}
	

	public List<Factor> getFactors() {
		return this.factors;
	}

	public void setFactors(List<Factor> factors) {
		this.factors = factors;
	}

	@Override
	public boolean equals(Object object){
		
		if (object == this) return true;
		Dimension dimension = (Dimension) object;
		
		return dimension.getId() == this.id && dimension.getName().equals(this.name); 
		
	}
	
	/** Busca el factor de la dimension identificado con el id
	 * @param idFactor
	 * @return 
	 */
	public Factor getFactor(int idFactor){
		for(Factor f : this.factors){
			if(f.getId() == idFactor){
				return f;
			}
		}
		return new Factor();
	}
}
