package com.kiuplus.logic.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.validation.constraints.Size;

import org.hibernate.annotations.Where;
import org.hibernate.validator.constraints.NotBlank;

import com.fasterxml.jackson.annotation.JsonBackReference;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonManagedReference;
import com.kiuplus.logic.model.Attribute.EnumAttributeType;

@Entity
@Table(name = "instanced_metrics")
public class InstancedMetric implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue
	@Column(name = "ID")
	private int id;

	@NotBlank
	@Size(max = 45)
	private String name;

	@JsonManagedReference
	@ManyToOne(optional = false, fetch = FetchType.EAGER)
	@JoinColumn(name = "idDataModel", insertable = true, updatable = true)
	private DataModel dataModel;

	@JsonBackReference
	@ManyToOne(optional = false, fetch = FetchType.EAGER)
	@JoinColumn(name = "idMethod", insertable = true, updatable = true)
	private Method method;

	@JsonBackReference
	@OneToMany(targetEntity = ParameterValue.class, cascade = CascadeType.ALL, fetch = FetchType.EAGER, orphanRemoval = true)
	@JoinColumn(name = "idInstancedMetric", insertable = true, updatable = true)
	@Where(clause = "idResultGroup IS NULL")
	private List<ParameterValue> parameterValues;

	private String metadataTableResult;

	private Date dateTimeResult;


	
	public InstancedMetric() {
		this.parameterValues = new ArrayList<>();
		this.method = new Method();
	}

	public InstancedMetric(int id, String name, DataModel dataModel, Method method) {
		this.id = id;
		this.name = name;
		this.dataModel = dataModel;
		this.method = method;
		this.parameterValues = new ArrayList<>();

	}

	public InstancedMetric(DataModel dataModel) {

		this.dataModel = dataModel;
		this.method = new Method();
		this.parameterValues = new ArrayList<>();

	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public DataModel getDataModel() {
		return dataModel;
	}

	public void setDataModel(DataModel dataModel) {
		this.dataModel = dataModel;
	}

	public Method getMethod() {
		return method;
	}

	public void setMethod(Method method) {
		this.method = method;
	}

	public List<ParameterValue> getParameterValues() {
		return parameterValues;
	}

	public void setParameterValues(List<ParameterValue> parameterValues) {
		this.parameterValues = parameterValues;
	}

	public EnumAttributeType getTypeAttribute() {
		if (this.method != null && this.method.getMetrics() != null) {
			return this.method.getMetrics().getTypeAttribute();
		}
		return EnumAttributeType.UNDEFINED;

	}

	/**
	 * @return la metrica instanciada tiene que conectarse a la base de datos, depende del tipo de
	 *         la metrica
	 */
	public boolean getConnectToDB() {
		if (method != null) {
			return this.method.getMetrics().getConnectToDB();
		}
		
		return false;
	}

	public String getMetadataTableResult() {
		return metadataTableResult;
	}

	public void setMetadataTableResult(String metadataTableResult) {
		this.metadataTableResult = metadataTableResult;
	}

	public String getFullName() {
		if (this.getMethod() != null) {
			return this.name + " (" + this.getMethod().getMetrics().getGranularity() + ")";
		}
		return this.name;
	}

	public Date getDateTimeResult() {
		return dateTimeResult;
	}

	public void setDateTimeResult(Date dateTimeResult) {
		this.dateTimeResult = dateTimeResult;
	}
	
	/*************************
	 * ExecuteServiceDAOImpl *
	 *************************/
	@JsonIgnore
	public List<ParameterAttribute> getParameterAttributeList(){
		return this.getParameterValues().stream()
					.filter(p -> p.getClass().equals(ParameterAttribute.class))
					.map(m -> (ParameterAttribute) m)
					.collect(Collectors.toList());
	}
	
	@JsonIgnore
	public List<Reference> getReferencedParameterList(){
		return this.getParameterValues().stream()
					.filter(p -> p.getClass().equals(Reference.class))
					.map(m -> (Reference) m)
					.collect(Collectors.toList());
	}

}
