package com.kiuplus.logic.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.hibernate.validator.constraints.NotBlank;

import com.fasterxml.jackson.annotation.JsonBackReference;
import com.fasterxml.jackson.annotation.JsonManagedReference;
import com.kiuplus.logic.model.Attribute.EnumAttributeType;

/**
 * Metrics generated by hbm2java
 */

@Entity
@Table(name = "metrics")
public class Metrics implements Serializable {

	public enum EnumGranularity {
		Undefined("..."), Cell("Cell"), CellSet("Cell set"), Tuple ("Tuple"), Column("Column"), ColumnSet ("Column set"), Table ("Table"), Database ("Database");

		private String text;

		EnumGranularity(String text) {
			this.text = text;
		}

		public String getText() {
			return this.text;
		}

		public static EnumGranularity fromString(String text) {
			for (EnumGranularity b : EnumGranularity.values()) {
				if (b.text.equalsIgnoreCase(text)) {
					return b;
				}
			}
			return null;
		}
		
		public static EnumGranularity[] getGroup1(){
			EnumGranularity[] group1 = new EnumGranularity[3];
			group1[0] = EnumGranularity.Cell;
			group1[1] = EnumGranularity.CellSet;
			group1[2] = EnumGranularity.Tuple;
			
			return group1;
		}
		
		public static EnumGranularity[] getGroup2(){
			EnumGranularity[] group2 = new EnumGranularity[4];
			group2[0] = EnumGranularity.Column;
			group2[1] = EnumGranularity.ColumnSet;
			group2[2] = EnumGranularity.Table;
			group2[3] = EnumGranularity.Database;
			
			return group2;
		}
		
		public boolean isGroup1() {
			return (this.equals(EnumGranularity.Cell) || this.equals(EnumGranularity.CellSet) || this.equals(EnumGranularity.Tuple));
		}
		
		public boolean isGroup2() {
			return !isGroup1();
		}

	}

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "ID")
	private int id;

	@Size(max = 45)
	@NotBlank
	private String name;

	@Size(max = 1000)
	@NotBlank
	private String description;
	
	@Column(name = "granularity")
	@Enumerated(EnumType.ORDINAL)
	@NotNull
	private EnumGranularity granularity;

	@JsonManagedReference
	@OneToMany(targetEntity = Method.class, cascade = CascadeType.ALL, fetch = FetchType.LAZY)
	@JoinColumn(name = "idmetrics", insertable = false, updatable = false)
	private List<Method> methods = new ArrayList<Method>(0);

	@JsonBackReference
	@ManyToOne(optional = false, fetch = FetchType.EAGER)
	@JoinColumn(name = "idfactor")
	private Factor factor;

	public Metrics() {
		this.factor = new Factor();
		this.granularity = EnumGranularity.Undefined;
	}

	public Metrics(String name, EnumGranularity granularity, List<Method> methods, Factor factor) {
		this.name = name;
		this.granularity = granularity;
		this.methods = methods;
		this.factor = factor;
	}

	public Metrics(int id, String name, EnumGranularity granularity) {
		this.id = id;
		this.name = name;
		this.granularity = granularity;
	}

	public Metrics(Factor factor) {
		this.factor = factor;
		this.granularity = EnumGranularity.Undefined;
	}

	public int getId() {
		return this.id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public EnumGranularity getGranularity() {
		return this.granularity;
	}

	public void setGranularity(EnumGranularity granularity) {
		this.granularity = granularity;
	}

	public List<Method> getMethods() {
		return this.methods;
	}

	public void setMethods(List<Method> methods) {
		this.methods = methods;
	}

	public Factor getFactor() {
		return factor;
	}

	public void setFactor(Factor factor) {
		this.factor = factor;
	}

	@Override
	public boolean equals(Object object) {

		if (object == this)
			return true;
		Metrics metrics = (Metrics) object;

		return metrics.getId() == this.id && metrics.getName().equals(this.name);

	}

	/**
	 * Busca el metodo de la metrica identificado con el id
	 * 
	 * @param idMethod
	 * @return
	 */
	public Method getMethod(int idMethod) {
		for (Method m : this.methods) {
			if (m.getId() == idMethod) {
				return m;
			}
		}
		return new Method();
	}
	
	

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public EnumAttributeType getTypeAttribute() {
		if (this.granularity != null) {
			switch (this.getGranularity()) {
			case Cell:
			case CellSet:
			case Column:
			case ColumnSet:
			case Tuple:
				return EnumAttributeType.COLUMNS;
			case Table:
				return EnumAttributeType.TABLES;
			case Database:
				return EnumAttributeType.DATABASE;
			default:
				return EnumAttributeType.UNDEFINED;

			}
		} else {
			return EnumAttributeType.UNDEFINED;
		}
	}
	
	
	/** Determina si la operacion tiene que conectarse a la base de datos directamente
	 * @return
	 */
	public boolean getConnectToDB() {
		return granularity == EnumGranularity.Column || granularity == EnumGranularity.ColumnSet
				|| granularity == EnumGranularity.Table || granularity == EnumGranularity.Database;
	}

}
