package com.kiuplus.logic.model;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.validation.constraints.Size;

import org.hibernate.validator.constraints.NotBlank;

import com.fasterxml.jackson.annotation.JsonBackReference;
import com.kiuplus.logic.model.Parameter.EnumDataType;

@Entity
@Table(name = "operations")
public class Operation implements java.io.Serializable, Cloneable {

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "id")
	private int id;

	@Size(max=45)
	@NotBlank
	private String name;
	
	private EnumDataType dataTypeReturn;
	

	@ManyToOne(cascade = CascadeType.ALL, optional = false, fetch = FetchType.EAGER) 
	@JoinColumn(name = "idWebService")
	@JsonBackReference
	private WebService webService;


	@OneToMany(targetEntity = Parameter.class, cascade = CascadeType.ALL, fetch = FetchType.EAGER, orphanRemoval = true)
	@JoinColumn(name = "idOperation", insertable = true, updatable = true, nullable=false)
	private List<Parameter> parameters;

	
	public Operation() {
		this.parameters = new ArrayList<>();
		this.webService = new WebService();
	}

	public Operation(int id, String name, EnumDataType dataTypeReturn,  WebService webService) {
		this.id = id;
		this.name = name;
		this.dataTypeReturn = dataTypeReturn;
		this.parameters = new ArrayList<>();
		this.webService = webService;
	}

	public Operation(int id, String name, EnumDataType dataTypeReturn) {
		this.id = id;
		this.name = name;
		this.dataTypeReturn = dataTypeReturn;
		this.parameters = new ArrayList<>();
	}


	public int getId() {
		return this.id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public EnumDataType getDataTypeReturn() {
		return dataTypeReturn;
	}

	public void setDataTypeReturn(EnumDataType dataTypeReturn) {
		this.dataTypeReturn = dataTypeReturn;
	}
	
	public WebService getWebService() {
		return webService;
	}

	public void setWebService(WebService webService) {
		this.webService = webService;
	}
	
	public List<Parameter> getParameters() {
		return parameters;
	}

	public void setParameters(List<Parameter> parameters) {
		this.parameters = parameters;
	}

	/**
	 * Busca el parametro de la operacion identificado con el nombre
	 * 
	 * @param nameParameter
	 * @return
	 */
	public Parameter getParameter(String nameParameter) {
		for (Parameter p : this.parameters) {
			if (p.getName().equals(nameParameter)) {
				return p;
			}
		}
		return new Parameter();
	}
	
	
	/**
	 * Busca el parametro de la operacion identificado con el id
	 * 
	 * @param nameParameter
	 * @return
	 */
//	public Parameter getParameter(int idParameter) {
//		for (Parameter p : this.parameters) {
//			if (p.getId() == idParameter) {
//				return p;
//			}
//		}
//		return new Parameter();
//	}

}
