package com.kiuplus.logic.model;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import com.kiuplus.logic.exception.ExceptionKiuPlus;

@Entity
@Table(name = "parameters")
@Inheritance(strategy = InheritanceType.JOINED)
public class Parameter implements java.io.Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	
	
	
	public enum EnumParameterType {
		REFERENCE("Reference"), ATTRIBUTE("Attribute"), CONNECT_TO_DB("Connect to DB") , TABLE_NAME("Table name"), COLUMN_NAME("Column name");
//		, INSTANCED_METRIC("Instanced metric")
		private String text;

		EnumParameterType(String text) {
			this.text = text;
		}

		public String getText() {
			return this.text;
		}

		public static EnumParameterType fromString(String text) {
			for (EnumParameterType b : EnumParameterType.values()) {
				if (b.text.equalsIgnoreCase(text)) {
					return b;
				}
			}
			return null;
		}
	}

	
	
	public enum EnumDataType {
		STRING("String"), INTEGER("Integer"), BOOL("Boolean"), DATETIME("DateTime"), DOUBLE("Double"), UNDEFINED("");

		private String text;

		EnumDataType(String text) {
			this.text = text;
		}

		public String getText() {
			return this.text;
		}

		/** Convierte el string en EnumDataType 
		 * @param text
		 * @return
		 * @throws ExceptionKiuPlus
		 */
		public static EnumDataType fromString(String text) throws ExceptionKiuPlus {
			text = text.toUpperCase();

			if (text.contains("INT"))
				return INTEGER;
			if (text.contains("BOOL"))
				return BOOL;
			if (text.contains("DATE"))
				return DATETIME;
			if (text.contains("DOUBLE"))
				return DOUBLE;
			if (text.contains("STRING"))
				return STRING;

			throw new ExceptionKiuPlus("The data type " + text + " is not supported");
		}

//		// TODO: falta hacer el caso que sea postgres
//		public static EnumDataType fromString(String text) throws ExceptionKiuPlus {
//			text = text.toUpperCase();
//			if (text.equals("BIT(1)"))
//				return BOOL;
//			if (text.equals("TINYINT") || text.equals("SMALLINT") || text.equals("MEDIUMINT") || text.equals("INT") || text.equals("INTEGER") || text.equals("BIGINT") || text.equals("BIT") || text.equals("YEAR"))
//				return INTEGER;
//			if (text.equals("DATE") || text.equals("TIME") || text.equals("DATETIME") || text.equals("TIMESTAMP"))
//				return DATETIME;
//			if (text.equals("DECIMAL") || text.equals("FLOAT") || text.equals("DOUBLE") || text.equals("NUMERIC") || text.equals("REAL"))
//				return DOUBLE;
//			if (text.equals("CHAR") || text.equals("VARCHAR") || text.equals("BINARY") || text.equals("VARBINARY") || text.equals("BLOB") || text.equals("TEXT") || text.equals("ENUM") || text.equals("SET") || text.equals("TINYBLOB") || text.equals("BLOB") || text.equals("MEDIUMBLOB") || text.equals("LONGBLOB") || text.equals("TINYTEXT") || text.equals("MEDIUMTEXT") || text.equals("LONGTEXT") || text.equals("ENUM") || text.equals("SET") || text.equals("JSON"))
//				return STRING;
//
//			throw new ExceptionKiuPlus("The data type " + text + " is not supported");
//		}

	}

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "id")
	private int id;

	@Size(max = 45)
	private String name;

	@Column(name = "type")
	private EnumParameterType parameterType;

	@NotNull
	private EnumDataType dataType;

	public Parameter(int id, String name, EnumParameterType parameterType, EnumDataType datatype) {
		this.id = id;
		this.name = name;
		this.parameterType = parameterType;
		this.dataType = datatype;
	}

	public Parameter() {
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public EnumParameterType getParameterType() {
		return parameterType;
	}

	public void setParameterType(EnumParameterType parameterType) {
		this.parameterType = parameterType;
	}

	public EnumDataType getDataType() {
		return dataType;
	}

	public void setDataType(EnumDataType dataType) {
		this.dataType = dataType;
	}

//	public static EnumDataType DataTypeGet(String dataType) throws ExceptionKiuPlus {
//		if (dataType != null) {
//			switch (dataType.toUpperCase().trim()) {
//			case "STRING":
//				return EnumDataType.STRING;
//			case "BOOLEAN":
//			case "BOOL":
//				return EnumDataType.BOOL;
//			case "DATETIME":
//				return EnumDataType.DATETIME;
//			case "DOUBLE":
//			case "FLOAT":
//				return EnumDataType.DOUBLE;
//			default:
//				throw new ExceptionKiuPlus("The data type " + dataType + " is not supported");
//			}
//		} else {
//			return EnumDataType.UNDEFINED;
//		}

	//}

	// public Parameter parameterGet() {
	//
	// switch (this.parameterType) {
	// case REFERENTIAL:
	// return new ParameterReferential(0, this.name, this.dataType);
	// case ATTRIBUTE:
	// return new ParameterAttribute(0, this.name, this.dataType);
	// case CONNECT_TO_DB:
	// return new ParameterConnectToDB(0, this.getName(), this.dataType);
	// }
	// return this;
	// }

	// public void setValue(String value) {
	// this.name = value;
	// }

	public String getDataTypeText() {
		return this.dataType.text;
	}
	
	public String getParameterTypeText() {
		
		return this.parameterType != null ? this.parameterType.text : "";
	}
}
