package com.kiuplus.logic.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.hibernate.validator.constraints.NotBlank;

import com.fasterxml.jackson.annotation.JsonBackReference;
import com.kiuplus.logic.model.Metrics.EnumGranularity;
import com.kiuplus.logic.model.Parameter.EnumParameterType;

@Entity
@Table(name = "result_group")
public class ResultGroup implements Serializable{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	public enum EnumResultGroup {
		Aggregation, ComposedInstancedMetric
	}
		
//	public enum EnumAggregationType {
//		Undefined(""), Granularity("Granularity"), Measurement("Measurement"), Partial_Measurement("Partial measurement");
//		
//		private String text;
//
//		EnumAggregationType(String text) {
//			this.text = text;
//		}
//
//		public String getText() {
//			return this.text;
//		}
//
//		public static EnumAggregationType fromString(String text) {
//			for (EnumAggregationType b : EnumAggregationType.values()) {
//				if (b.text.equalsIgnoreCase(text)) {
//					return b;
//				}
//			}
//			return null;
//		}
//	}

	
	@Id
	@GeneratedValue
	@Column(name = "id")
	private int id;
	
	@Size(max = 45)
	@NotBlank
	private String name;

	@ManyToOne(optional = false, fetch = FetchType.EAGER)
	@JoinColumn(name = "idDataModel")
	private DataModel dataModel;

	@ManyToOne(optional = false, fetch = FetchType.LAZY, cascade= CascadeType.ALL)
	@JoinColumn(name = "idOperation")
	private Operation operation;

	@JsonBackReference
	@OneToMany(targetEntity = ParameterValue.class, cascade = CascadeType.ALL, fetch = FetchType.EAGER, orphanRemoval = true)
	@JoinColumn(name = "idResultGroup", insertable = true, updatable = true)
	private List<ParameterValue> parameterValues;
		
	@Enumerated(EnumType.ORDINAL)
	@NotNull
	private EnumResultGroup type;
		
	@Enumerated(EnumType.ORDINAL)
	@NotNull
	private EnumGranularity granularity;
	
	private String metadataTableResult;
	
	private Date dateTimeResult;
	
	public ResultGroup() {
		this.parameterValues = new ArrayList<ParameterValue>();
		//this.type = EnumAggregationType.Undefined;
	}

	public ResultGroup(DataModel dataModel) {
		this.dataModel = dataModel;
		this.parameterValues = new ArrayList<ParameterValue>();
		//this.type = EnumAggregationType.Undefined;
	}
	
	public ResultGroup(DataModel dataModel, EnumResultGroup type) {
		this.dataModel = dataModel;
		this.parameterValues = new ArrayList<ParameterValue>();
		this.type = type;
	}

	public ResultGroup(String name) {
		this.name = name;
	}
	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public DataModel getDataModel() {
		return dataModel;
	}

	public void setDataModel(DataModel dataModel) {
		this.dataModel = dataModel;
	}

	public Operation getOperation() {
		return operation;
	}

	public void setOperation(Operation operation) {
		this.operation = operation;
	}	
	
	public List<ParameterValue> getParameterValues() {
		return parameterValues;
	}

	public void setParameterValues(List<ParameterValue> parameterValues) {
		this.parameterValues = parameterValues;
	}

//	public InstancedMetric getInstancedMetric1() {
//		return instancedMetric1;
//	}
//
//	public void setInstancedMetric1(InstancedMetric instancedMetric1) {
//		this.instancedMetric1 = instancedMetric1;
//	}
//
//	public InstancedMetric getInstancedMetric2() {
//		return instancedMetric2;
//	}
//
//	public void setInstancedMetric2(InstancedMetric instancedMetric) {
//		this.instancedMetric2 = instancedMetric;
//	}

//	public EnumAggregationType getType() {
//		return type;
//	}
//
//	public void setType(EnumAggregationType type) {
//		this.type = type;
//	}


	public String getMetadataTableResult() {
		return metadataTableResult;
	}

	public EnumResultGroup getType() {
		return type;
	}

	public void setType(EnumResultGroup type) {
		this.type = type;
	}

	public EnumGranularity getGranularity() {
		return granularity;
	}

	public void setGranularity(EnumGranularity granularity) {
		this.granularity = granularity;
	}

	public void setMetadataTableResult(String metadataTableResult) {
		this.metadataTableResult = metadataTableResult;
	}

	public Date getDateTimeResult() {
		return dateTimeResult;
	}

	public void setDateTimeResult(Date dateTimeResult) {
		this.dateTimeResult = dateTimeResult;
	}
	
	/**************
	 * Auxiliares *
	 **************/
	public boolean isInstancedMetricsExecuted() {
		for(ParameterValue pv : this.parameterValues) {
			if(pv.getInstancedMetric() != null) {
				if(pv.getInstancedMetric().getDateTimeResult() != null) {
					return true;
				}
			}
		}
		return false;	
	}
	
	public Date getInstancedMetricsExecutedDateTime() {
		for(ParameterValue pv : this.parameterValues) {
			if(pv.getInstancedMetric() != null) {
				if(pv.getInstancedMetric().getDateTimeResult() != null) {
					return pv.getInstancedMetric().getDateTimeResult();
				}
			}
		}
		return null;	
	}
	
	public InstancedMetric getInstancedMetric() {
		for(ParameterValue pv : this.parameterValues) {
			if(pv.getInstancedMetric() != null) {
				return pv.getInstancedMetric();
			}
		}
		return null;	
	}
	
}
