package com.kiuplus.logic.objects;

import java.util.*;
import com.kiuplus.logic.dataAccess.DataAccess;

public class Schema {
	
	/**********************
     * Atributos privados *
     **********************/
	//Nombre del esquema
	private String name = "";
	
	//Diccionario de tablas y columnas de cada tabla (son las tablas que no se van a copiar enteras)
	private Map<String,GenericObject> partialTables = new HashMap<String, GenericObject>();
	
	//Diccionario de tablas (son las que se copian enteras)
	private List<String> completeTables = new ArrayList<String>();
	
	//Indica si hay que copiar toda la base
	private boolean copyDataBase = false; 
	
	/***********
     * Getters *
     ***********/
	public String getName(){
		return this.name;
	}
	public boolean getCopyDataBase(){
		return this.copyDataBase;
	}
	
	/***********
     * Setters *
     ***********/
	public void setName(String name){
		this.name = name;
	}
	public void setCopyDataBase(boolean b){
		this.copyDataBase = b;
	}

	/***************
     * Operaciones *
     ***************/
	
	/*
	 * Descripcin: Agrega tableName a la coleccin de tablas enteras 
	*/
 	public void add(String tableName){
		if (!completeTables.contains(tableName)){
			completeTables.add(tableName);
			if (partialTables.containsKey(tableName)){
				partialTables.remove(tableName);
			}
		}
	}
	
	/*
	 * Descripcin: Agrega (tableName, columnName) a la coleccin de tablas parciales 
	*/
	public void add(String tableName, String columnName){
		if (partialTables.containsKey(tableName)){
			if (! partialTables.get(tableName).containsProperty(columnName)){
				partialTables.get(tableName).setProperty(columnName, columnName);
			}
		}else{
			GenericObject toInsert = new GenericObject();
			toInsert.setProperty(columnName, columnName);
			partialTables.put(tableName, toInsert);
		}
	}
	
	public List<String> getPartialTablesName(){
		return new ArrayList<String>(partialTables.keySet());
	}
	
	public List<String> getTablesName(){
		List<String> result = new ArrayList<String>(partialTables.keySet());
		result.addAll(completeTables);
		return result;
	}
	
	public List<String> getTableColumns(String tableName){
		if (partialTables.containsKey(tableName)){
			return new ArrayList<String>(partialTables.get(tableName).getProperties().values());
		} else {
			return null;
		}
	}

	public void copy(DataAccess source, DataAccess metaData){
		//Si hay que copiar la base de datos entera obtengo la lista de tablas de la base de datos origen
		//Si no la obtengo de la informacin de las mtricas instanciadas
		List<String> tablesName;
		if (this.getCopyDataBase()){
			tablesName = source.getTablesName(this.getName());
		}else{
			tablesName = this.getTablesName();
		}
		
		for(String tableName : tablesName){
			List<GenericObject> tableAttributesList = null;
			//Si hay que copiar la base entera --> hay que obtener todas las columnas de la base
			if (this.getCopyDataBase()){
				tableAttributesList = source.getColumnsNamesCompleteTable(this.getName(), tableName);
			}else{
				//Obtengo la lista de caracteristicas de los atributos de tableName (PK + involucrados en las mtricas a ejecutar)
				List<String> columnsName = this.getTableColumns(tableName);
				//Si columnsName es vaco --> hay que copiar la tabla entera
				if (columnsName == null){
					tableAttributesList = source.getColumnsNamesCompleteTable(this.getName(), tableName);
				} else{ //Si no --> hay que copiar solo las columnas involucradas en las mi
					tableAttributesList = source.getPKColumnsAndColumnsNamesPartialTable(this.getName(), tableName, columnsName); //TO DO: Analizar si se puede hacer en un nica consulta
				}
			}
			//Obtengo lista de nombre los atributos de tableName a copiar
			List<String> tableAttributesStringList = source.getTableAttributeStringList(tableAttributesList);
			List<String> tableAttributeTypeStringList = source.attributeListToAttributeTypeStringList(tableAttributesList);
			
			//Creo la tabla en la base de metadatos donde se van a copiar los datos a evaluar
			metaData.createDataTable(tableName, tableAttributeTypeStringList);
			//Obtengo los valores a copiar
			List<GenericObject> toCopy = source.getValues(this.name, tableName, tableAttributesStringList);
			//Guardamos la informacin en la base de datos de metadatos
			toCopy.forEach(valueToCopy -> {
				metaData.save(tableAttributesStringList, valueToCopy, tableName);
			});
		}
    }
}
