package com.kiuplus.configuration.logic.domain;

import java.util.List;

import org.hibernate.Hibernate;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.query.Query;
import org.springframework.stereotype.Repository;

import com.kiuplus.logic.domain.HibernateUtils;
import com.kiuplus.logic.model.Factor;

@Repository
public class FactorsDAOImpl implements FactorsDAO {

	private SessionFactory sessionFactory;

	public FactorsDAOImpl() {
		this.sessionFactory = (SessionFactory) HibernateUtils.getSessionFactory();
	}

	@SuppressWarnings("unchecked")
	public List<Factor> factorGetAll(int idDimension) {

		Session session = sessionFactory.openSession();
		try {
			Query<Factor> query = session
					.createQuery("from " + Factor.class.getName() + " where idDimension= :idDimension ");
			query.setParameter("idDimension", idDimension);

			List<Factor> factors = query.getResultList();

			return factors;

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		} finally {
			session.close();
		}
	}

	@SuppressWarnings("unchecked")
	public List<Factor> factorFindByName(String name) {

		Session session = sessionFactory.openSession();
		try {
			Query<Factor> query = session.createQuery("from " + Factor.class.getName() + " where name= :name ");
			query.setParameter("name", name);
			List<Factor> factors = query.getResultList();

			return factors;
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		} finally {
			session.close();
		}
	}

	public Factor factorFindById(int id) {

		Session session = sessionFactory.openSession();
		try {
			Factor factor = (Factor) session.get(Factor.class, id);

			return factor;
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		} finally {
			session.close();
		}
	}

	public Factor factorFindFullById(int id) {

		Session session = sessionFactory.openSession();
		try {
			Factor factor = (Factor) session.get(Factor.class, id);
			if(factor != null)
				Hibernate.initialize(factor.getMetrics());

			return factor;
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		} finally {
			session.close();
		}
	}

	public void factorAdd(Factor factor) {

		Session session = sessionFactory.openSession();
		try {
			session.beginTransaction();

			factor.setId((int) session.save(factor));

			session.getTransaction().commit();
		} catch (Exception e) {
			e.printStackTrace();
			session.getTransaction().rollback();
			throw e;
		} finally {
			session.close();
		}
	}

	public void factorUpdate(Factor factor) {

		Session session = sessionFactory.openSession();
		try {
			session.beginTransaction();

			session.update(factor);

			session.getTransaction().commit();
		} catch (Exception e) {
			e.printStackTrace();
			session.getTransaction().rollback();
			throw e;
		} finally {
			session.close();
		}
	}

	public void factorDelete(Factor factor) {

		Session session = sessionFactory.openSession();
		try {
			session.beginTransaction();

			session.delete(factor);

			session.getTransaction().commit();
		} catch (Exception e) {
			e.printStackTrace();
			session.getTransaction().rollback();
			throw e;
		} finally {
			session.close();
		}

	}

}
