package com.kiuplus.configuration.logic.domain;

import java.util.List;

import org.hibernate.Hibernate;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.query.Query;
import org.springframework.stereotype.Repository;

import com.kiuplus.logic.domain.HibernateUtils;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.InstancedMetric;
import com.kiuplus.logic.model.Method;
import com.kiuplus.logic.model.Operation;

@Repository
public class MethodsDAOImpl implements MethodsDAO {

	private SessionFactory sessionFactory;

	public MethodsDAOImpl() {
		this.sessionFactory = (SessionFactory) HibernateUtils.getSessionFactory();
	}

	@SuppressWarnings("unchecked")
	public List<Method> methodsGetAll(int idMetrics) {

		Session session = sessionFactory.openSession();
		try {
			Query<Method> query = session
					.createQuery("from " + Method.class.getName() + " where idMetrics= :idMetrics ");
			query.setParameter("idMetrics", idMetrics);

			List<Method> methods = query.getResultList();

			return methods;
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		} finally {
			session.close();
		}
	}

	@SuppressWarnings("unchecked")
	public List<Method> methodsFindByName(String name) {

		Session session = sessionFactory.openSession();
		try {
			Query<Method> query = session.createQuery("from " + Method.class.getName() + " where name= :name ");
			query.setParameter("name", name);
			List<Method> methods = query.getResultList();

			return methods;
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		} finally {
			session.close();
		}
	}

	public Method methodFindById(int id) {

		Session session = sessionFactory.openSession();
		try {
			Method method = (Method) session.get(Method.class, id);

			return method;
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		} finally {
			session.close();
		}
	}

	public Method methodFindFullById(int id) {

		Session session = sessionFactory.openSession();
		try {
			Method method = (Method) session.get(Method.class, id);

			return method;
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		} finally {
			session.close();
		}
	}

	public void methodAdd(Method method) throws ExceptionKiuPlus {
		Session session = sessionFactory.openSession();
		try {

			session.beginTransaction();

			method.setId((int) session.save(method));

			session.flush();
			session.getTransaction().commit();
			session.clear();

		} catch (Exception e) {
			e.printStackTrace();
			session.getTransaction().rollback();
			throw new ExceptionKiuPlus(e.getMessage());
		} finally {
			session.close();
		}

	}

	public void methodUpdate(Method method) throws ExceptionKiuPlus {

		Session session = sessionFactory.openSession();
		try {
			session.beginTransaction();

			// Cuando el metodo tiene MI asociadas solo se edita el nombre
			if (method.getInstancedMetrics().isEmpty()) {
				Query<Operation> query = session
						.createQuery("from " + Operation.class.getName() + " where idWebService= :idWebService ");
				query.setParameter("idWebService", method.getOperation().getWebService().getId());

				List<Operation> operations = query.getResultList();

				for (int i = 0; i < operations.size(); i++) {
					if (operations.get(i).getId() != method.getOperation().getId()) {
						session.remove(operations.get(i));

					}

				}
				//
				method = (Method) session.merge(method);

			}
			session.update(method);
			session.getTransaction().commit();
			session.clear();

		} catch (Exception e) {
			e.printStackTrace();
			session.getTransaction().rollback();
			throw new ExceptionKiuPlus(e.getMessage());
		} finally {
			session.close();
		}
	}

	public void methodDelete(Method method) {

		Session session = sessionFactory.openSession();
		try {
			session.beginTransaction();

			session.delete(method);

			session.getTransaction().commit();
		} catch (Exception e) {
			e.printStackTrace();
			session.getTransaction().rollback();
			throw e;
		} finally {
			session.close();
		}

	}

	public List<InstancedMetric> instanedMetricsFind(int idMethod) {
		Session session = sessionFactory.openSession();
		try {
			Query<InstancedMetric> query = session
					.createQuery("FROM " + InstancedMetric.class.getName() + " WHERE idMethod = :idMethod ");

			query.setParameter("idMethod", idMethod);
			List<InstancedMetric> instancedMetrics = query.getResultList();

			return instancedMetrics;
		} catch (Exception e) {
			e.printStackTrace();
			session.getTransaction().rollback();
			throw e;
		} finally {
			session.close();
		}
	}

}
