package com.kiuplus.configuration.logic.service;

import java.util.Iterator;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.kiuplus.configuration.logic.domain.DimensionsDAO;
import com.kiuplus.configuration.logic.domain.DimensionsDAOImpl;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.Dimension;
import com.kiuplus.logic.model.Factor;
import com.kiuplus.logic.model.Method;
import com.kiuplus.logic.model.Metrics;

/**
 * @author Beatriz
 *
 */

@Service
public class DimensionsServiceImpl implements DimensionsService {

	@Autowired
	private DimensionsDAO dao;

	public DimensionsServiceImpl() {
		this.dao = new DimensionsDAOImpl();
	}

	public List<Dimension> dimensionsGetAll() {
		return dao.getAllDimensions();
	}

	public List<Dimension> dimensionsDataQualityModelGet(int idDataModel) {
		List<Dimension> dimensions = this.dimensionsGetAll();

		// Se remueven los elementos que no tienen metricas instanciadas

		Iterator<Dimension> itDimension = dimensions.iterator();
		while (itDimension.hasNext()) {
			// Elimina las dimensiones que no tienen factores
			Dimension d = itDimension.next();
			Iterator<Factor> itFactor = d.getFactors().iterator();

			while (itFactor.hasNext()) {
				// Elimina los factores que no tienen metricas
				Factor f = itFactor.next();
				Iterator<Metrics> itMetrics = f.getMetrics().iterator();

				while (itMetrics.hasNext()) {
					// Elimina las metricas que no tienen metodos
					Metrics me = itMetrics.next();
					Iterator<Method> itMethods = me.getMethods().iterator();

					while (itMethods.hasNext()) {
						// Elimina los metodos que no tienen metricas instanciadas
						Method method = itMethods.next();
						method.getInstancedMetrics().removeIf(im -> im.getDataModel().getId() != idDataModel);

					}

					me.getMethods().removeIf(method -> method.getInstancedMetrics().isEmpty());
				}

				f.getMetrics().removeIf(me -> me.getMethods().isEmpty());
			}

			d.getFactors().removeIf(fa -> fa.getMetrics().isEmpty());
		}

		dimensions.removeIf(d -> d.getFactors().isEmpty());
		return dimensions;
	}

	public void dimensionCreate(Dimension dimension) throws ExceptionKiuPlus {

		// Verifica que no exista una dimension con el mismo nombre
		if (dao.findDimensionByName(dimension.getName()).size() > 0) {
			throw new ExceptionKiuPlus("There is a dimension with the same name.");
		}

		this.dao.dimensionAdd(dimension);

	}

	public void dimensionUpdate(Dimension dimension) throws ExceptionKiuPlus {

		List<Dimension> dimDAO = dao.findDimensionByName(dimension.getName());

		// Verifica que no exista una dimension con el mismo nombre
		if (dimDAO.size() > 0 && (!dimDAO.contains(dimension)
				|| dimDAO.get(dimDAO.indexOf(dimension)).getId() != dimension.getId())) {

			throw new ExceptionKiuPlus("There is a dimension with the same name.");
		}

		// Verifica que  exista la dimension
		Dimension d = this.dimensionFindFullDById(dimension.getId());
		if (d == null) {
			throw new ExceptionKiuPlus("The dimension not exists");
		}
		
		this.dao.dimensionUpdate(dimension);

	}

	public Dimension dimensionGet(int id) {

		return this.dao.dimensionFindById(id);

	}

	public Dimension dimensionFindFullDById(int id) {

		return this.dao.dimensionFindFullById(id);

	}

	public List<Dimension> dimensionFindByName(String name){
		return dao.findDimensionByName(name);
	}
	
	public void dimensionDelete(int id) throws ExceptionKiuPlus {
		Dimension dimension = this.dimensionFindFullDById(id);
		if (dimension == null) {
			throw new ExceptionKiuPlus("there is no dimension");
		}
		if (dimension.getFactors().size() > 0) {
			throw new ExceptionKiuPlus("dimension has factors");
		}
		this.dao.dimensionDelete(dimension);

	}

}
