package com.kiuplus.configuration.logic.service;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.kiuplus.configuration.logic.domain.FactorsDAO;
import com.kiuplus.configuration.logic.domain.FactorsDAOImpl;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.Factor;

@Service
public class FactorsServiceImpl implements FactorsService{

	@Autowired
	private FactorsDAO dao;
	
	public FactorsServiceImpl(){
		this.dao = new FactorsDAOImpl();
	}

	public List<Factor> factorsGetAll(int idDimension) {
		return dao.factorGetAll(idDimension);
	}

	public Factor factorGet(int id) {
		return this.dao.factorFindById(id);
	}
	
	public Factor factorGetFull(int id) {
		return this.dao.factorFindFullById(id);
	}

	public void factorCreate(Factor factor) throws ExceptionKiuPlus{
		// Verifica que no exista un factor con el mismo nombre
				if (dao.factorFindByName(factor.getName()).size() > 0) {
					throw new ExceptionKiuPlus("There is a factor with the same name.");
				}

				this.dao.factorAdd(factor);
	}

	public void factorUpdate(Factor factor) throws ExceptionKiuPlus{
		List<Factor> factorDAO = dao.factorFindByName(factor.getName());

		// Verifica que no exista un factor con el mismo nombre
		if (factorDAO.size() > 0 && (!factorDAO.contains(factor)
				|| factorDAO.get(factorDAO.indexOf(factor)).getId() != factor.getId())) {

			throw new ExceptionKiuPlus("There is a factor with the same name.");
		}

		this.dao.factorUpdate(factor);
		
	}

	public Factor factorDelete(int id) throws ExceptionKiuPlus{
		Factor factor = this.factorGetFull(id);
		if (factor == null) {
			throw new ExceptionKiuPlus("there is no factor");
		}
		if(factor.getMetrics().size() > 0){
			
			throw new ExceptionKiuPlus("the factor has metrics");
		}
		this.dao.factorDelete(factor);
		return factor;
	}

}
