package com.kiuplus.configuration.logic.service;


import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.kiuplus.configuration.logic.domain.MethodsDAO;
import com.kiuplus.configuration.logic.domain.MethodsDAOImpl;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.InstancedMetric;
import com.kiuplus.logic.model.Method;


@Service
public class MethodsServiceImpl implements MethodsService {

	@Autowired
	private MethodsDAO dao;

	public MethodsServiceImpl() {
		this.dao = new MethodsDAOImpl();
	}

	public List<Method> methodsGetAll(int idMetrics) {
		return dao.methodsGetAll(idMetrics);
	}

	public Method methodGet(int id) {
		return this.dao.methodFindById(id);
	}

	public Method methodGetFull(int id) {
		return this.dao.methodFindFullById(id);
	}

	public void methodCreate(Method method) throws ExceptionKiuPlus {
		// Verifica que no exista un method con el mismo nombre
		if (dao.methodsFindByName(method.getName()).size() > 0) {
			throw new ExceptionKiuPlus("There is a method with the same name.");
		}

		this.dao.methodAdd(method);
	}

	public void methodUpdate(Method method) throws ExceptionKiuPlus {
		List<Method> methodDAO = dao.methodsFindByName(method.getName());

		// Verifica que no exista otro metodo con el mismo nombre
		if (methodDAO.size() > 0 && (!methodDAO.contains(method)
				|| methodDAO.get(methodDAO.indexOf(method)).getId() != method.getId())) {

			throw new ExceptionKiuPlus("There is a method with the same name.");
		}

		this.dao.methodUpdate(method);

	}

	public Method methodDelete(int id) throws ExceptionKiuPlus {
		Method method = this.methodGetFull(id);
		if (method == null) {
			throw new ExceptionKiuPlus("there is no method");
		}

		List<InstancedMetric> instancedMetrics = dao.instanedMetricsFind(id);
		if (instancedMetrics.size() > 0) {
			throw new ExceptionKiuPlus("there is an instantiated metric associated with the method.");
		}
		this.dao.methodDelete(method);
		return method;
	}

}
