package com.kiuplus.configuration.logic.service;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.kiuplus.configuration.logic.domain.MetricsDAO;
import com.kiuplus.configuration.logic.domain.MetricsDAOImpl;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.Metrics;

@Service
public class MetricsServiceImpl implements MetricsService {

	@Autowired
	private MetricsDAO dao;

	public MetricsServiceImpl(){
		this.dao = new MetricsDAOImpl();
	}
	
	public List<Metrics> metricsGetAll(int idDimension) {
		return dao.metricsGetAll(idDimension);
	}

	public Metrics metricsGet(int id) {
		return this.dao.metricsFindById(id);
	}

	public Metrics metricsGetFull(int id) {
		return this.dao.metricsFindFullById(id);
	}

	public void metricsCreate(Metrics metrics) throws ExceptionKiuPlus {
		// Verifica que no exista un metrics con el mismo nombre
		if (dao.metricsFindByName(metrics.getName()).size() > 0) {
			throw new ExceptionKiuPlus("There is a metrics with the same name.");
		}

		this.dao.metricsAdd(metrics);
	}

	public void metricsUpdate(Metrics metrics) throws ExceptionKiuPlus {
		List<Metrics> metricsDAO = dao.metricsFindByName(metrics.getName());

		// Verifica que no exista un metrics con el mismo nombre
		if (metricsDAO.size() > 0 && (!metricsDAO.contains(metrics)
				|| metricsDAO.get(metricsDAO.indexOf(metrics)).getId() != metrics.getId())) {

			throw new ExceptionKiuPlus("There is a metrics with the same name.");
		}

		this.dao.metricsUpdate(metrics);

	}

	public Metrics metricsDelete(int id) throws ExceptionKiuPlus {
		Metrics metrics = this.metricsGetFull(id);
		if (metrics == null) {
			throw new ExceptionKiuPlus("there is no metrics");
		}

		if (metrics.getMethods().size() > 0) {
			throw new ExceptionKiuPlus("the metrics has methods");
		}

		this.dao.metricsDelete(metrics);
		return metrics;
	}

}
