package test.java;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import com.kiuplus.configuration.logic.service.DimensionsService;
import com.kiuplus.configuration.logic.service.DimensionsServiceImpl;
import com.kiuplus.configuration.logic.service.FactorsService;
import com.kiuplus.configuration.logic.service.FactorsServiceImpl;
import com.kiuplus.configuration.logic.service.MethodsService;
import com.kiuplus.configuration.logic.service.MethodsServiceImpl;
import com.kiuplus.configuration.logic.service.MetricsService;
import com.kiuplus.configuration.logic.service.MetricsServiceImpl;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.Dimension;
import com.kiuplus.logic.model.Factor;
import com.kiuplus.logic.model.Method;
import com.kiuplus.logic.model.Metrics;
import com.kiuplus.logic.model.Operation;
import com.kiuplus.logic.model.Parameter;
import com.kiuplus.logic.model.Parameter.EnumDataType;
import com.kiuplus.logic.model.Parameter.EnumParameterType;
import com.kiuplus.logic.model.WebService;

public class testConfiguration {

	private DimensionsService dimensionsService;
	private FactorsService factorsService;
	private MetricsService metricsService;
	private MethodsService methodsService;

	private Integer idDelete = 10000;

	@Before
	public void initialize() {
		dimensionsService = new DimensionsServiceImpl();
		factorsService = new FactorsServiceImpl();
		metricsService = new MetricsServiceImpl();
		methodsService = new MethodsServiceImpl();
	}

	@Test
	public void test() {
		this.dimensionsT();
		this.dimensionsDeleteT();
	}

	public void dimensionsT() {
		try {

			// Crear dimension
			Dimension dimension = new Dimension();
			String name = "dimensionTest";
			dimension.setName(name);
			dimensionsService.dimensionCreate(dimension);

			// Consultar dimension
			List<Dimension> dimensions = dimensionsService.dimensionFindByName(name);

			assertEquals(dimensions.size(), 1);

			// Actualizar dimension
			Dimension dimensionCreate = dimensions.get(0);
			name = "dimensionTest2";
			dimensionCreate.setName(name);
			dimensionsService.dimensionUpdate(dimensionCreate);
			dimensions = dimensionsService.dimensionsGetAll();
			assertEquals(dimensions.size(), 1);

			assertEquals(dimensions.get(0).getName(), name);

			assertEquals(dimensionsService.dimensionGet(dimension.getId()).getName(), name);

			Dimension dimensionduplicate = new Dimension();
			dimensionduplicate.setName(name);
			try {

				dimensionsService.dimensionCreate(dimensionduplicate);
				fail();
			} catch (ExceptionKiuPlus e) {

			}

			try {
				dimensionsService.dimensionUpdate(dimensionduplicate);
				fail();
			} catch (ExceptionKiuPlus e) {

			}

			try {
				dimensionsService.dimensionDelete(idDelete);
				fail();
			} catch (ExceptionKiuPlus e) {

			}
			this.factorT(dimensionCreate);
		} catch (Exception e) {
			fail();
		}

	}

	public void factorT(Dimension dimension) {
		try {
			String name = "Factor";
			Factor factor = new Factor(dimension);
			factor.setName(name);
			factorsService.factorCreate(factor);

			// Consultar
			List<Factor> factors = factorsService.factorsGetAll(dimension.getId());

			assertEquals(1, factors.size());

			// Actualizar
			Factor factorCreate = factors.get(0);
			name = "factorTest2";
			factorCreate.setName(name);
			factorsService.factorUpdate(factorCreate);
			factor = factorsService.factorGet(factorCreate.getId());

			assertEquals(name, factor.getName());
			assertEquals(dimension.getId(), factor.getDimension().getId());

			assertEquals(1, dimensionsService.dimensionGet(dimension.getId()).getFactors().size());

			// Duplicados

			Factor factor3 = new Factor(dimension);
			factor3.setName("factor3");
			this.factorsService.factorCreate(factor3);

			factor3.setName(name);

			// Actualizar nombre duplicado en oto
			try {
				this.factorsService.factorUpdate(factor3);
				fail();
			} catch (ExceptionKiuPlus e) {
			}

			// Crear nombre duplicado
			Factor factor2 = new Factor(dimension);
			factor2.setName(name);
			try {

				factorsService.factorCreate(factor2);
				fail();
			} catch (ExceptionKiuPlus e) {

			}

			// Actualizar factor que no existe
			try {
				factorsService.factorUpdate(factor2);
				fail();
			} catch (ExceptionKiuPlus e) {

			}

			// Borrar factor que no existe
			try {
				factorsService.factorDelete(idDelete);
				fail();
			} catch (ExceptionKiuPlus e) {

			}
			metricsT(factor3);

		} catch (ExceptionKiuPlus e) {
			fail();
		}

	}

	public void metricsT(Factor factor) throws ExceptionKiuPlus {
		try {
			String name = "Metrics";
			Metrics metric = new Metrics(factor);
			metric.setName(name);
			this.metricsService.metricsCreate(metric);

			// Consultar
			List<Metrics> metrics = this.metricsService.metricsGetAll(factor.getId());

			assertEquals(1, metrics.size());

			// Actualizar
			Metrics metricCreate = metrics.get(0);
			name = "metricsTest2";
			metricCreate.setName(name);
			this.metricsService.metricsUpdate(metricCreate);
			metric = this.metricsService.metricsGet(metricCreate.getId());

			assertEquals(name, metric.getName());
			assertEquals(factor.getId(), metric.getFactor().getId());

			assertEquals(1, factorsService.factorGet(factor.getId()).getMetrics().size());

			// Duplicados

			Metrics metrics3 = new Metrics(factor);
			metrics3.setName("metrics3");
			this.metricsService.metricsCreate(metrics3);

			metrics3.setName(name);

			// Actualizar nombre duplicado en otro
			try {
				this.metricsService.metricsUpdate(metrics3);
				fail();
			} catch (ExceptionKiuPlus e) {
			}

			// Crear nombre duplicado
			Metrics metrics2 = new Metrics();
			metrics2.setFactor(factor);
			metrics2.setName(name);
			try {

				this.metricsService.metricsCreate(metrics2);
				fail();
			} catch (ExceptionKiuPlus e) {

			}

			// Actualizar metrica que no existe
			try {
				this.metricsService.metricsUpdate(metrics2);
				fail();
			} catch (ExceptionKiuPlus e) {

			}

			// Borrar metrica que no existe
			try {
				this.metricsService.metricsDelete(idDelete);
				fail();
			} catch (ExceptionKiuPlus e) {

			}
			this.methodT(metricCreate);
		} catch (ExceptionKiuPlus e) {
			throw e;
		}

	}

	public void methodT(Metrics metric) throws ExceptionKiuPlus {
		try {

			String name = "Method1";
			Method method = new Method(metric);

			method = this.methodCreate(method, name);
			this.methodsService.methodCreate(method);

			// Consultar
			List<Method> methods = this.methodsService.methodsGetAll(metric.getId());

			assertEquals(1, methods.size());

			// Actualizar
			Method methodCreate = this.methodsService.methodGetFull(method.getId());
			name = "methodTest2";
			methodCreate.setName(name);
			this.methodsService.methodUpdate(methodCreate);
			method = this.methodsService.methodGet(method.getId());

			assertEquals(name, method.getName());
			assertEquals(metric.getId(), method.getMetrics().getId());
			assertEquals(3, method.getOperation().getParameters().size());
			assertEquals(2, method.getOperation().getWebService().getOperations().size());
			assertEquals(3, method.getOperation().getWebService().getOperations().get(0).getParameters().size());
			assertEquals(3, method.getOperation().getWebService().getOperations().get(1).getParameters().size());
			assertEquals(1, metricsService.metricsGet(metric.getId()).getMethods().size());

			// Actualizar operacion de metodo
			method.getOperation().getParameters().remove(0);

			List<Parameter> parameters1 = new ArrayList<>();
			Parameter parameter1 = new Parameter();
			parameter1.setName("Parameter7");
			parameter1.setDataType(EnumDataType.DATETIME);
			parameter1.setParameterType(EnumParameterType.COLUMN_NAME);
			parameters1.add(parameter1);

			method.getOperation().getParameters().add(parameter1);

			this.methodsService.methodUpdate(method);
			method = this.methodsService.methodGet(method.getId());

			assertEquals(6, method.getOperation().getWebService().getOperations().get(0).getParameters().size()
					+ method.getOperation().getWebService().getOperations().get(1).getParameters().size());

			// Cambiar operacion del metodo

			Operation operation = (Operation) method.getOperation().getWebService().getOperations().stream()
					.filter(o -> o.getName().equals("Operation2")).toArray()[0];

			method.setOperation(operation);

			this.methodsService.methodUpdate(method);
			method = this.methodsService.methodGet(method.getId());

			assertEquals("Operation2", method.getOperation().getName());
			assertEquals(2, method.getOperation().getWebService().getOperations().size());

			// Cambiar web service
			method = this.methodCreate(method, "Method4");
			this.methodsService.methodUpdate(method);
			method = this.methodsService.methodGet(method.getId());

			assertEquals(2, method.getOperation().getWebService().getOperations().size());
			// Duplicados

			// Method method3 = new Method(metric);
			// method3.setName("method3");
			// this.methodsService.methodCreate(method3);
			//
			// method3.setName(name);

			// Actualizar nombre duplicado en otro
			// try {
			// this.methodsService.methodUpdate(method3);
			// fail();
			// } catch (ExceptionKiuPlus e) {
			// }

			// Crear nombre duplicado
			Method method2 = new Method();
			method2.setMetrics(metric);
			method2.setName(name);
			try {

				this.methodsService.methodCreate(method2);
				fail();
			} catch (ExceptionKiuPlus e) {

			}

			// Actualizar metrica que no existe
			try {
				this.methodsService.methodUpdate(method2);
				fail();
			} catch (ExceptionKiuPlus e) {

			}

			// Borrar metrica que no existe
			try {
				this.methodsService.methodDelete(idDelete);
				fail();
			} catch (ExceptionKiuPlus e) {

			}
		} catch (ExceptionKiuPlus e) {
			throw e;
		}

	}

	private Method methodCreate(Method method, String name) {

		method.setName(name);

		WebService webService = new WebService();
		webService.setUrl(
				"https://www.google.com.uy/search?q=SQLite&oq=SQLite&aqs=chrome..69i57j69i60l2j0l3.1629j0j7&sourceid=chrome&ie=UTF-8");

		Operation operation1 = new Operation();
		operation1.setName("Operation1");
		operation1.setWebService(webService);

		List<Operation> operations = new ArrayList<>();

		List<Parameter> parameters1 = new ArrayList<>();
		Parameter parameter1 = new Parameter();
		parameter1.setName("Parameter1");
		parameter1.setDataType(EnumDataType.BOOL);
		parameter1.setParameterType(EnumParameterType.ATTRIBUTE);
		parameters1.add(parameter1);

		Parameter parameter2 = new Parameter();
		parameter2.setName("Parameter2");
		parameter2.setDataType(EnumDataType.INTEGER);
		parameter2.setParameterType(EnumParameterType.COLUMN_NAME);
		parameters1.add(parameter2);

		Parameter parameter3 = new Parameter();
		parameter3.setName("Parameter3");
		parameter3.setDataType(EnumDataType.DATETIME);
		parameter3.setParameterType(EnumParameterType.CONNECT_TO_DB);
		parameters1.add(parameter3);

		operation1.setParameters(parameters1);

		operations.add(operation1);

		List<Parameter> parameters2 = new ArrayList<>();
		Parameter parameter4 = new Parameter();
		parameter4.setName("Parameter4");
		parameter4.setDataType(EnumDataType.DOUBLE);
		parameter4.setParameterType(EnumParameterType.COLUMN_NAME);
		parameters2.add(parameter4);

		Parameter parameter5 = new Parameter();
		parameter5.setName("Parameter5");
		parameter5.setDataType(EnumDataType.INTEGER);
		parameter5.setParameterType(EnumParameterType.REFERENCE);
		parameters2.add(parameter5);

		Parameter parameter6 = new Parameter();
		parameter6.setName("Parameter6");
		parameter6.setDataType(EnumDataType.STRING);
		parameter6.setParameterType(EnumParameterType.TABLE_NAME);
		parameters2.add(parameter6);

		Operation operation2 = new Operation();
		operation2.setName("Operation2");
		operation2.setParameters(parameters2);
		operation2.setWebService(webService);

		operations.add(operation2);
		webService.setOperations(operations);

		method.setOperation(operation1);

		return method;
	}

	public void dimensionsDeleteT() {
		try {
			// Eliminar dimension

			List<Dimension> dimensions = this.dimensionsService.dimensionsGetAll();

			try {
				for (Dimension d : dimensions)
					dimensionsService.dimensionDelete(d.getId());
				fail();
			} catch (ExceptionKiuPlus e) {
			}

			for (Dimension d : dimensions) {
				this.factorDeleteT(d);
				dimensionsService.dimensionDelete(d.getId());
			}

			assertEquals(0, dimensionsService.dimensionsGetAll().size());
		} catch (Exception e) {
			fail();
		}

	}

	private void factorDeleteT(Dimension d) {

		// Falla
		try {
			this.factorsService.factorDelete(idDelete);
			fail();
		} catch (ExceptionKiuPlus e) {

		}

		try {
			for (Factor f : d.getFactors()) {
				if (f.getMetrics().size() > 0) {
					this.factorsService.factorDelete(f.getId());
				}
			}
			fail();
		} catch (ExceptionKiuPlus e) {

		}

		for (Factor f : d.getFactors()) {
			this.metricsDeleteT(f);
		}

		// Exito
		try {
			for (Factor f : d.getFactors()) {
				this.factorsService.factorDelete(f.getId());
			}
			assertEquals(0, this.factorsService.factorsGetAll(d.getId()).size());
		} catch (ExceptionKiuPlus e) {
			fail();
		}

	}

	private void metricsDeleteT(Factor f) {
		// Falla
		try {
			this.metricsService.metricsDelete(idDelete);
			fail();
		} catch (ExceptionKiuPlus e) {

		}

		try {
			if (f.getMetrics().size() > 0) {
				for (Metrics m : f.getMetrics()) {
					if (m.getMethods().size() > 0) {
						this.metricsService.metricsDelete(m.getId());
					}
				}
				fail();
			}
		} catch (ExceptionKiuPlus e) {

		}
		for (Metrics m : f.getMetrics()) {
			this.methodDeleteT(m);
		}
		// Exito
		try {
			for (Metrics m : f.getMetrics()) {
				this.metricsService.metricsDelete(m.getId());
			}
			assertEquals(0, this.metricsService.metricsGetAll(f.getId()).size());
		} catch (ExceptionKiuPlus e) {
			fail();
		}
	}

	private void methodDeleteT(Metrics me) {
		// Falla
		try {
			this.methodsService.methodDelete(idDelete);
			fail();
		} catch (ExceptionKiuPlus e) {

		}

		// Exito
		try {
			for (Method m : me.getMethods()) {
				this.methodsService.methodDelete(m.getId());
			}
			assertEquals(0, this.methodsService.methodsGetAll(me.getId()).size());
		} catch (ExceptionKiuPlus e) {
			fail();
		}
	}

}
