package com.kiuplus.qualityModel.logic.domain;

import java.util.List;

import org.hibernate.Hibernate;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.query.Query;
import org.springframework.stereotype.Repository;

import com.kiuplus.logic.domain.HibernateUtils;
import com.kiuplus.logic.model.AttributeColumn;
import com.kiuplus.logic.model.AttributeTable;
import com.kiuplus.logic.model.DataModel;

/**
 * @author Beatriz
 *
 */

@Repository
public class DataModelDAOImpl implements DataModelDAO {

	private SessionFactory sessionFactory;

	public DataModelDAOImpl() {
		this.sessionFactory = (SessionFactory) HibernateUtils.getSessionFactory();
	}

	@SuppressWarnings("unchecked")
	public List<DataModel> dataModelsGet() {

		Session session = sessionFactory.openSession();
		try {
			Query<DataModel> query = session.createQuery("from " + DataModel.class.getName());
			List<DataModel> dataModels = query.getResultList();

			return dataModels;
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		} finally {
			session.close();
		}
	}

	public DataModel dataModelGet(int id) {
		Session session = sessionFactory.openSession();
		try {
			DataModel dataModel = (DataModel) session.get(DataModel.class, id);

			return dataModel;
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		} finally {
			session.close();
		}
	}

	public DataModel dataModelGetFull(int id) {
		Session session = sessionFactory.openSession();
		try {
			DataModel dataModel = (DataModel) session.get(DataModel.class, id);
			Hibernate.initialize(dataModel.getResultGroups());

			return dataModel;
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		} finally {
			session.close();
		}
	}

	public List<DataModel> dataModelFind(String name) {
		Session session = sessionFactory.openSession();
		try {
			Query<DataModel> query = session
					.createQuery("from " + DataModel.class.getSimpleName() + " where name= :name ");
			query.setParameter("name", name);
			List<DataModel> dataModels = query.getResultList();

			return dataModels;
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		} finally {
			session.close();
		}
	}

	public void dataModelAdd(DataModel dataModel) {
		Session session = sessionFactory.openSession();
		try {
			session.beginTransaction();

			dataModel.setId((int) session.save(dataModel));
			dataModel.getDatabaseMetadata()
					.setName(dataModel.getDatabaseMetadata().getName() + "_" + dataModel.getId());
			session.update(dataModel);

			session.getTransaction().commit();
		} catch (Exception e) {
			e.printStackTrace();
			session.getTransaction().rollback();
			throw e;
		} finally {
			session.close();
		}

	}

	public void dataModelUpdate(DataModel dataModel) {
		Session session = sessionFactory.openSession();
		try {
			session.beginTransaction();

			session.update(dataModel);

			session.getTransaction().commit();
		} catch (Exception e) {
			e.printStackTrace();
			session.getTransaction().rollback();
			throw e;
		} finally {
			session.close();
		}
	}

	public void dataModelDelete(DataModel dataModel) {

		Session session = sessionFactory.openSession();
		try {
			session.beginTransaction();

			session.delete(dataModel);

			session.getTransaction().commit();
		} catch (Exception e) {
			e.printStackTrace();
			session.getTransaction().rollback();
			throw e;
		} finally {
			session.close();
		}

	}

	public List<AttributeTable> tablesAttributeFind(String name) {
		Session session = sessionFactory.openSession();

		try {
			Query<AttributeTable> query = session
					.createQuery("from " + AttributeTable.class.getSimpleName() + " where name= :name ");
			query.setParameter("name", name);
			List<AttributeTable> attributesTables = query.getResultList();

			return attributesTables;
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		} finally {
			session.close();
		}

	}

	public List<AttributeColumn> columnsAttributeFind(String name) {
		Session session = sessionFactory.openSession();
		try {
			Query<AttributeColumn> query = session
					.createQuery("from " + AttributeColumn.class.getSimpleName() + " where name= :name ");
			query.setParameter("name", name);
			List<AttributeColumn> attributesColumns = query.getResultList();

			return attributesColumns;
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		} finally {
			session.close();
		}
	}
}
