package com.kiuplus.qualityModel.logic.domain;

import java.io.ByteArrayOutputStream;
import java.io.StringReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.MimeHeaders;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPConnection;
import javax.xml.soap.SOAPConnectionFactory;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPPart;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathFactory;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

import com.kiuplus.logic.dataAccess.*;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.AttributeColumn;
import com.kiuplus.logic.model.AttributeDatabase;
import com.kiuplus.logic.model.InstancedMetric;
import com.kiuplus.logic.model.Operation;
import com.kiuplus.logic.model.Parameter;
import com.kiuplus.logic.model.ParameterAttribute;
import com.kiuplus.logic.model.ParameterValue;
import com.kiuplus.logic.model.Reference;
import com.kiuplus.logic.model.ResultGroup;
import com.kiuplus.logic.model.Attribute.EnumAttributeType;
import com.kiuplus.logic.model.Parameter.EnumParameterType;
import com.kiuplus.logic.objects.GenericObject;
import com.kiuplus.logic.objects.Schema;
import com.kiuplus.qualityModel.logic.service.InstancedMetricsService;
import com.kiuplus.qualityModel.logic.service.ResultGroupsService;

@Repository

public class ExecuteServiceDAOImpl implements ExecuteServiceDAO {

	@Autowired
	private InstancedMetricsService instancedMetricsService;
	
	@Autowired
	private ResultGroupsService resultGroupService;
	
	public ExecuteServiceDAOImpl() {
	}
	
	public void executeMetrics (List<InstancedMetric> imList, boolean reloadData) throws Exception{
    	try
    	{
    		//Obtengo la info de las bases de datos (datos a evaluar + metadatos)
    		AttributeDatabase adbData = imList.get(0).getDataModel().getDatabaseSource();
    		AttributeDatabase adbMetaData = imList.get(0).getDataModel().getDatabaseMetadata();
    		
			DataAccess dataAccesslMetadata = new MySQLDataAccess(adbMetaData.getUrl() + "?allowMultiQueries=true", 
																	adbMetaData.getUser(), 
																	adbMetaData.getPassword());
  			dataAccesslMetadata.connect();
  			dataAccesslMetadata.setDataBase(adbMetaData.getName()); 
  			
  			//Obtengo parte del esquema que participa en las mtricas que van a ejecutarse
	    	Schema schemaToCopy = getSchemaToCopy(imList);
	    	schemaToCopy.setName(adbData.getName());
	    	
    		if (reloadData)
    		{
    			//Inicio el DataAcces que voy a utilizar para obtener los datos a evaluar
    	    	DataAccess dataAccess;
    			switch (adbData.getSystem()){
    			case MySQL:
    				dataAccess = new MySQLDataAccess(adbData.getUrl()+ "/" + adbData.getName(), adbData.getUser(), adbData.getPassword());
    				break;
    			case PostgreSQL:
    				dataAccess = new PostgreSQLDataAccess(adbData.getUrl()+ "/" + adbData.getName(), adbData.getUser(), adbData.getPassword());
    				break;
    			default:
    				dataAccess = new DataAccess(adbData.getUrl()+ "/" + adbData.getName(), adbData.getUser(), adbData.getPassword());
    				break;
    			}
				//Me conecto a la base de datos que voy a consultar los datos
				dataAccess.connect();
				
				//Copio las tablas
	  			schemaToCopy.copy(dataAccess, dataAccesslMetadata);
	  	
	  			//Me desconecto de la BD a evaluar
	  			dataAccess.disconnect();
    		} 

	    	//Ejecutar cada una de las metricas
  			for(InstancedMetric im : imList){
  				try{
  					//TO DO: Verificar si estn los datos que necesitamos para ejecutar im
  					
	  				String tableName = "";
	  				List<String> attributeStringList =  new ArrayList<String>();
	  				Map<String, String> attributeParameters = new HashMap<String, String>();
	  				
	  				//Obtengo los parametros que son referenciales
	  				List<Reference> referencedParameterList = im.getReferencedParameterList();
	  				List<GenericObject> referencedParameters = new ArrayList<GenericObject> ();
	  				for (Reference r : referencedParameterList){
	  					attributeParameters.put(r.getParameter().getName(), r.getParameter().getName());
	  					referencedParameters.add(new GenericObject(r.getParameter().getName(), r.getReferenceValue()));  					
	  				}
	  				//Obtengo los parametros que son de Base de datos
	  				List<ParameterAttribute> parameterAttributes = im.getParameterAttributeList();
	  				
	  				Date imTimeStamp = null;
	  				
	  				//Depende de la granularidad de la metrica
	  				switch (im.getMethod().getMetrics().getGranularity()){
	  				case Cell:
	  				case CellSet:
	  				case Tuple:
	  					//Creo la tabla donde se van a guardar los resultados de la im
		  				dataAccesslMetadata.createIMResultTableG1(im.getId(), im.getMethod().getOperation().getDataTypeReturn().getText());
		  				
	  					for(ParameterAttribute p : parameterAttributes) {
							//Nombre de la tabla
	  		    			if(p.getAttribute().getTypeAttribute().equals(EnumAttributeType.COLUMNS)) {
	  		    				AttributeColumn ac = (AttributeColumn) p.getAttribute();
	  		    				tableName = ac.getAttributeTable().getName();
	  		    				attributeStringList.add(p.getAttribute().getName());
	  		    				attributeParameters.put(p.getParameter().getName(), p.getAttribute().getName());
	  		    			}	
	  		    		}
	  					attributeStringList.add("id_metadata_" + tableName);
	  					final String innerTableName = tableName;
	  					List<GenericObject> toEvalute = dataAccesslMetadata.getValues(adbMetaData.getName(), tableName, attributeStringList);
	  					toEvalute.forEach(valueToEvalute -> {
	  						try {
		  						//Agrego los parametros por referencia
		  						valueToEvalute.addProperties(referencedParameters);
		  						//Invoco al WS
		  						Object result = invokeWesService(im.getMethod().getOperation(), valueToEvalute, attributeParameters);
								//Guardo el resultado en la BD
		  						dataAccesslMetadata.saveIMResultG1(im.getId(), Integer.parseInt(valueToEvalute.getProperty("id_metadata_" + innerTableName)), result);
							} catch (Exception e) {
								e.printStackTrace();
							}
	  					});
	  					imTimeStamp = Calendar.getInstance().getTime();
	  					break;
	  				case Column:
	  				case ColumnSet:
	  				case Table:
	  				case Database: //TO DO: No me queda claro su
	  					//Creo la tabla donde se van a guardar los resultados de la im
		  				dataAccesslMetadata.createIMResultTableG2(im.getId(), im.getMethod().getOperation().getDataTypeReturn().getText());
		  				
	  					GenericObject infoToEvalute = new GenericObject();
	  					for(ParameterAttribute p : parameterAttributes) {
		    				if(p.getParameter().getParameterType().equals(EnumParameterType.COLUMN_NAME)
			    					|| p.getParameter().getParameterType().equals(EnumParameterType.TABLE_NAME)){
		    					infoToEvalute.setProperty(p.getParameter().getName(), p.getAttribute().getName());
		    				}
		    				if(p.getParameter().getParameterType().equals(EnumParameterType.CONNECT_TO_DB)){
		    					String dburl = im.getDataModel().getDatabaseMetadata().getUrl() 
		    									+ "/" + p.getAttribute().getName() 
		    									+ "?user=" + im.getDataModel().getDatabaseMetadata().getUser()
		    									+ "&password=" + im.getDataModel().getDatabaseMetadata().getPassword();
		    					infoToEvalute.setProperty(p.getParameter().getName(), dburl);
		    				}
			    		}
	  					//Agrego los parametros por referencia
	  					infoToEvalute.addProperties(referencedParameters);
	  					//Invoco al WS
  						Object result = invokeWesService(im.getMethod().getOperation(), infoToEvalute, null);
						//Guardo el resultado en la BD
  						imTimeStamp = Calendar.getInstance().getTime();
  						dataAccesslMetadata.saveIMResultG2(im.getId(), result, imTimeStamp);
	  					break;
	  				default:
	  					break;
	  	    			
	  	    		}
	  				
	  				//Actualiza la im con la fecha de ejecucin y la referencia al resultado
	  				im.setMetadataTableResult("metadata_im_" + im.getId());
					im.setDateTimeResult(imTimeStamp);
					this.instancedMetricsService.instancedMetricEdit(im);
					
  				} catch(Exception e){
					//throw e;
  				}
  			}
  			
  			//Cierro la conexin con la base de metadatos
  			dataAccesslMetadata.disconnect();
    	} catch(Exception e){
    		throw e;
    	}
    }
	
	public Map<Integer, String> executeAggregations (List<ResultGroup> aggList, boolean deleteIMResults) throws Exception{
		Map<Integer, String> errorAggregationsMap = new HashMap<Integer, String>();
		try{
			//Me conecto a la base de datos donde voy a guardar los resultados
			AttributeDatabase adb = aggList.get(0).getDataModel().getDatabaseMetadata();
			DataAccess dataAccesslMetadata = new MySQLDataAccess(adb.getUrl() + "/" + adb.getName() + "?allowMultiQueries=true", 
																	adb.getUser(), 
																	adb.getPassword());
			dataAccesslMetadata.connect();
			Map<Integer, InstancedMetric> imMap = new HashMap<Integer, InstancedMetric>();
			Map<Integer, InstancedMetric> errorInstancedMetricsMap = new HashMap<Integer, InstancedMetric>();
			//Ejecuto las agregaciones seleccionadas
			for(ResultGroup agg : aggList){
				try{
					List<ParameterValue> pvList = agg.getParameterValues();
					GenericObject infoToEvalute = new GenericObject();
					for(ParameterValue pv : pvList) {
						switch (pv.getParameter().getParameterType()){
						case COLUMN_NAME:
							infoToEvalute.setProperty(pv.getParameter().getName(), "result");
							break;
						case TABLE_NAME:
							InstancedMetric im = pv.getInstancedMetric();
							infoToEvalute.setProperty(pv.getParameter().getName(), im.getMetadataTableResult());
							if (!imMap.containsKey(im.getId())){
								imMap.put(im.getId(), im);
							}
							break;
						case CONNECT_TO_DB:
							String dburl = agg.getDataModel().getDatabaseMetadata().getUrl() 
											+ "/" + agg.getDataModel().getDatabaseMetadata().getName()
											+ "?user=" + agg.getDataModel().getDatabaseMetadata().getUser()
											+ "&password=" + agg.getDataModel().getDatabaseMetadata().getPassword();
	    					infoToEvalute.setProperty(pv.getParameter().getName(), dburl);
							break;
						case REFERENCE:
							Reference r = (Reference) pv;
							infoToEvalute.setProperty(r.getParameter().getName(), r.getReferenceValue());  					
							break;
						default:
							break;
						}
		    		}
					//Invoco al WS
					Object result = invokeWesService(agg.getOperation(), infoToEvalute, null);
					
					dataAccesslMetadata.setAutoCommit(false);
					//Si no existe crea la tabla para guardar los resultados de la agregacin 
					dataAccesslMetadata.createAggregationResultTable(agg.getId(), agg.getOperation().getDataTypeReturn().getText());
					//Guardo el resultado en la BD
					dataAccesslMetadata.saveAggregationResult(agg.getId(), result, agg.getInstancedMetricsExecutedDateTime());
					dataAccesslMetadata.commit();
					
					//Actualiza la im con la fecha de ejecucin y la referencia al resultado
					agg.setMetadataTableResult("metadata_agg_" + agg.getId());
					agg.setDateTimeResult(Calendar.getInstance().getTime());
					this.resultGroupService.resultGroupEdit(agg);
					
				}catch (Exception ex){
					errorAggregationsMap.put(agg.getId(), ex.getMessage());
					errorInstancedMetricsMap.put(agg.getInstancedMetric().getId(), agg.getInstancedMetric());
					
				} finally{
					dataAccesslMetadata.setAutoCommit(true);
				}
			}
			
			//Elimina los resultados de las MI que se haya ejecutado todas las agregaciones correctamente
			if (deleteIMResults){
				for(InstancedMetric im : imMap.values()){
					if (!errorInstancedMetricsMap.containsKey(im.getId())){
						dataAccesslMetadata.dropTable(im.getMetadataTableResult());
						im.setMetadataTableResult(null);
						im.setDateTimeResult(null);
						this.instancedMetricsService.instancedMetricEdit(im);
					}
				}
			}
			dataAccesslMetadata.disconnect();
			return errorAggregationsMap;
		}catch (Exception ex){
			throw new ExceptionKiuPlus(ex.getMessage());
		}
	}
	
	public void executeComposedInstancedMetric(List<ResultGroup> composedIMs, boolean deleteIMResults) throws Exception{
		try{
			//Hay que ejecutar las MIs
				//Se guardan los resultados intermedios (de las MIs)
				//No se guardan
			//No hay que ejecutar las MIs (estn los resultados en la base de metadatos
		}
		catch (Exception ex){
			throw ex;
		}
	}
	
	public List<GenericObject> getMetricResult(InstancedMetric im) throws Exception{
		List<GenericObject> result = new ArrayList<GenericObject>();
		//Me conecto a la base de datos donde voy a guardar los resultados
		AttributeDatabase adb = im.getDataModel().getDatabaseMetadata();
		DataAccess dataAccesslMetadata = new MySQLDataAccess(adb.getUrl() + "/" + adb.getName() + "?allowMultiQueries=true", 
																adb.getUser(), 
																adb.getPassword());
		dataAccesslMetadata.connect();
		
		//Obtengo los parametros que son de Base de datos
		List<ParameterAttribute> parameterAttributes = im.getParameterValues().stream()
			.filter(p -> p.getClass().equals(ParameterAttribute.class))
			.map(m -> (ParameterAttribute) m)
			.collect(Collectors.toList());
		//Depende de la granularidad de la mtrica lo que hago 
		switch (im.getMethod().getMetrics().getGranularity()){
			case Cell:
			case CellSet:
			case Tuple:
				String tableDataName = null;
				List<String> attributeStringList = new ArrayList<String>();
				for(ParameterAttribute p : parameterAttributes) {
	    			if(p.getAttribute().getTypeAttribute().equals(EnumAttributeType.COLUMNS)) {
	    				AttributeColumn ac = (AttributeColumn) p.getAttribute();
	    				tableDataName = ac.getAttributeTable().getName();
	    				attributeStringList.add(tableDataName + "." + p.getAttribute().getName());
	    			}	
	    		}
				result = dataAccesslMetadata.getResults("metadata_im_" + im.getId(), tableDataName, attributeStringList);
				break;
			case Column:			
			case ColumnSet:
			case Table:
				result = dataAccesslMetadata.getColumnResults("metadata_im_" + im.getId());
				break;
			default:
				break;
		}
		dataAccesslMetadata.disconnect();
		return result;
	}
	
	public List<GenericObject> getAggregationResult(ResultGroup agg) throws ExceptionKiuPlus, Exception{
		List<GenericObject> result = new ArrayList<GenericObject>();
		//Me conecto a la base de datos donde voy a guardar los resultados
		AttributeDatabase adb = agg.getDataModel().getDatabaseMetadata();
		DataAccess dataAccesslMetadata = new MySQLDataAccess(adb.getUrl() + "/" + adb.getName() + "?allowMultiQueries=true", 
																adb.getUser(), 
																adb.getPassword());
		dataAccesslMetadata.connect();
		result = dataAccesslMetadata.getAggregationResults(agg.getMetadataTableResult());
		dataAccesslMetadata.disconnect();
		return result;
	}
	
	/**********************
	 * Funciones privadas 
	 **********************/
	private Schema getSchemaToCopy(List<InstancedMetric> imList) throws Exception{
    	try{
	    	Schema result = new Schema();
	    	for(InstancedMetric im : imList){
	    		List<ParameterAttribute> parameterAttributes = im.getParameterAttributeList();
	    		switch (im.getMethod().getMetrics().getGranularity()){
				case Cell:
				case CellSet:
				case Tuple: //Entiendo que es este caso
					for(ParameterAttribute p : parameterAttributes) {
		    			//Nombre de la tabla
		    			if(p.getAttribute().getTypeAttribute().equals(EnumAttributeType.COLUMNS)) {
		    				AttributeColumn ac = (AttributeColumn)p.getAttribute();
		    				result.add(ac.getAttributeTable().getName(), p.getAttribute().getName());
		    			}
		    		}
					break;
				case Column:
				case ColumnSet:
		    		for(ParameterAttribute p : parameterAttributes) {
	    				if(p.getParameter().getParameterType().equals(EnumParameterType.COLUMN_NAME)) {
	    					//Columna
	    					AttributeColumn ac = (AttributeColumn)p.getAttribute();
	    					//Agrego: NombreTabla, NombreClumna
	    					result.add(ac.getAttributeTable().getName(), p.getAttribute().getName());
	    				}
		    		}
					break;
				case Table:
		    		for(ParameterAttribute p : parameterAttributes) {
	    				if(p.getParameter().getParameterType().equals(EnumParameterType.TABLE_NAME)) {
	    					//Agrego: NombreTabla
	    					result.add(p.getAttribute().getName());
	    				}
		    		}
					break;
				case Database:
					result.setCopyDataBase(true);
					break;
				default:
					break;
	    			
	    		}
	    	}
			return result;
	    } catch (Exception e){
	    	throw e;
    	}
    }
	
	private Object invokeWesService(Operation operation, GenericObject valueToEvalute, Map<String, String> attributeParameters) throws Exception{
  		try{
  			//TODO: definir que protocolo de WS vamos a usar
  			//Parametros que necesito tener para crear e invocar el WS
  			String prefix = "web";
			String targetNamespace = operation.getWebService().getTargetNamespace();
			
  			// Creo la conexin SOAP
  	        SOAPConnectionFactory soapConnectionFactory = SOAPConnectionFactory.newInstance();
  	        SOAPConnection soapConnection = soapConnectionFactory.createConnection();
  	        
  	        //Armamos el xml para invocar el WS
  	        MessageFactory messageFactory = MessageFactory.newInstance();
  	        SOAPMessage soapMessage = messageFactory.createMessage();
  	        SOAPPart soapPart = soapMessage.getSOAPPart();
  	        
  	        MimeHeaders headers = soapMessage.getMimeHeaders();
  	        URL tempURL = new URL(operation.getWebService().getUrl());
  	        headers.addHeader("SOAPAction", tempURL.getProtocol() + "://" + tempURL.getAuthority() + "/" + operation.getName());
  	        
  	        // SOAP Envelope
  	        SOAPEnvelope envelope = (SOAPEnvelope) soapPart.getEnvelope();
  	        envelope.addNamespaceDeclaration(prefix, targetNamespace);

  	        SOAPBody body = soapMessage.getSOAPBody();
  	        QName bodyName = new QName(null, operation.getName(), prefix);
  	        SOAPBodyElement bodyElement = body.addBodyElement(bodyName);
  	        
  	        //Agrego los parametros
  	        for (Parameter parameter : operation.getParameters()){
  	        //operation.getParameters().forEach((parameter)->{
  	        	SOAPElement symbol;
  				try {
  					
  					if (attributeParameters != null){
  						String toAdd = valueToEvalute.getProperty(attributeParameters.get(parameter.getName())); //Valor a evaluar
  						if (toAdd != null){
  							symbol = bodyElement.addChildElement(parameter.getName()); //Nombre del parametro , prefix
  							symbol.addTextNode(toAdd);
  						}
  					} else{
  						String toAdd = valueToEvalute.getProperty(parameter.getName()); //Valor a evaluar
  						if (toAdd != null){
  							symbol = bodyElement.addChildElement(parameter.getName()); //Nombre del parametro , prefix
  							symbol.addTextNode(toAdd);
  						}
  					}
  				} catch (SOAPException e) {
  					// TODO Auto-generated catch block
  					e.printStackTrace();
  				}
  		        
  	        //});
  	        }
  	        soapMessage.saveChanges();
  	        //PARA PRUEBA
  	        ByteArrayOutputStream out = new ByteArrayOutputStream();
  	        soapMessage.writeTo(out);
  	        String strMsg = new String(out.toByteArray());
  	        System.out.println(strMsg);
  	        
  	        //Realizo la llamada al WS con el xml armado
  	        SOAPMessage soapResponse = soapConnection.call(soapMessage, tempURL);
  	        return getResult(soapResponse, operation);
  	        
  		} catch (Exception e){
  			throw new ExceptionKiuPlus("The WebService was unavailable or there was a problem calling it.");
  		}
  	}
	
	private Object getResult(SOAPMessage sm, Operation operation) throws Exception {
  		try{
	  		ByteArrayOutputStream out = new ByteArrayOutputStream();
	  		sm.writeTo(out);
	  		String xml = new String(out.toByteArray());
	  		InputSource source = new InputSource(new StringReader(xml));
	  		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
	  		DocumentBuilder db = dbf.newDocumentBuilder();
	  		Document document = db.parse(source);
	  		XPathFactory xpathFactory = XPathFactory.newInstance();
	  		XPath xpath = xpathFactory.newXPath();
	  		String resultString = xpath.evaluate("/Envelope/Body/" + operation.getName() + "Response/return", document);
	  		Object result = null;
	  		switch (operation.getDataTypeReturn()){
	  		case BOOL:
	  			result = new Boolean(resultString);
	  			break;
	  		case DATETIME:
	  			break;
	  		case DOUBLE:
	  			result = new Double(resultString);
	  			break;
	  		case INTEGER:
	  			result = new Integer(resultString);
	  			break;
	  		default:
	  			result = resultString;
	  			break;
	  		}
	  		return result;
  		} catch (Exception e){
  			throw e;
  		}
  		
  	}


}
