package com.kiuplus.qualityModel.logic.domain;

import java.util.List;

import org.hibernate.Hibernate;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.query.Query;
import org.springframework.stereotype.Repository;

import com.kiuplus.logic.domain.HibernateUtils;
import com.kiuplus.logic.model.ResultGroup;
import com.kiuplus.logic.model.InstancedMetric;
import com.kiuplus.logic.model.ParameterValue;

@Repository
public class ResultGroupsDAOImpl implements ResultGroupsDAO {

	private SessionFactory sessionFactory;

	public ResultGroupsDAOImpl() {
		this.sessionFactory = (SessionFactory) HibernateUtils.getSessionFactory();
	}

	public ResultGroup resultGroupGet(int id) {
		Session session = sessionFactory.openSession();
		try {
			ResultGroup resultGroup = (ResultGroup) session.get(ResultGroup.class, id);
			if(resultGroup != null)
				Hibernate.initialize(resultGroup.getOperation());
			return resultGroup;
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		} finally {
			session.close();
		}
	}

	public List<ResultGroup> resultGroupsGet(List<Integer> resultGroupsIds) {
		Session session = sessionFactory.openSession();
		try {
			Query<ResultGroup> query = session
					.createQuery("from " + ResultGroup.class.getName() + " WHERE id IN :idsList");
			query.setParameter("idsList", resultGroupsIds);
			List<ResultGroup> resultGroups = query.getResultList();
			return resultGroups;
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		} finally {
			session.close();
		}
	}

	public List<ResultGroup> resultGroupsFind(String name) {
		Session session = sessionFactory.openSession();
		try {
			Query<ResultGroup> query = session
					.createQuery("from " + ResultGroup.class.getSimpleName() + " where name= :name ");
			query.setParameter("name", name);
			List<ResultGroup> resultGroups = query.getResultList();

			return resultGroups;
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		} finally {
			session.close();
		}
	}

	public void resultGroupCreate(ResultGroup resultGroup) {
		Session session = sessionFactory.openSession();
		try {
			session.beginTransaction();
			// Operation operation = resultGroup.getOperation();
			// operation.setId((int) session.save(operation));
			resultGroup.setId((int) session.save(resultGroup));
			session.getTransaction().commit();
		} catch (Exception e) {
			e.printStackTrace();
			session.getTransaction().rollback();
			throw e;
		} finally {
			session.close();
		}

	}

	public void resultGroupEdit(ResultGroup resultGroup) {
		Session session = sessionFactory.openSession();
		try {
			session.beginTransaction();

			session.merge(resultGroup);

			session.getTransaction().commit();
		} catch (Exception e) {
			e.printStackTrace();
			session.getTransaction().rollback();
			throw e;
		} finally {
			session.close();
		}

	}

	public void resultGroupDelete(ResultGroup resultGroup) {

		Session session = sessionFactory.openSession();
		try {
			session.beginTransaction();

			session.delete(resultGroup);

			session.getTransaction().commit();
		} catch (Exception e) {
			e.printStackTrace();
			session.getTransaction().rollback();
			throw e;
		} finally {
			session.close();
		}

	}

	public List<ParameterValue> parameterValueInstancedMetricFind(int idInstancedMetric) {
		Session session = sessionFactory.openSession();
		try {
			Query<ParameterValue> query = session.createQuery("from " + ParameterValue.class.getSimpleName()
					+ " where idInstancedMetric= :idInstancedMetric AND idResultGroup IS NOT NULL");
			query.setParameter("idInstancedMetric", idInstancedMetric);
			List<ParameterValue> parameterValue = query.getResultList();

			return parameterValue;
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		} finally {
			session.close();
		}
	}

}
