package com.kiuplus.qualityModel.logic.service;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.List;

import com.itextpdf.text.DocumentException;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.AttributeColumn;
import com.kiuplus.logic.model.AttributeTable;
import com.kiuplus.logic.model.DataModel;

/** Accede a la base de datos source
 * @author Beatriz
 *
 */
public interface DataModelsService {

	/**
	 * @return los modelos de datos de la base de datos
	 * @throws ExceptionKiuPlus Excepcion si no se puede desencriptar la contrasea
	 */
	public List<DataModel> dataModelsGet();

	/** Obtiene el modelo de datos con los resultados de grupos
	 * @param id
	 * @return
	 */
	public DataModel dataModelGetFull(int id);
	/**
	 * @param id
	 * @return DataModel identificado con el id
	 * @throws ExceptionKiuPlus Excepcion si no se puede desencriptar la contrasea
	 */
	public DataModel dataModelGet(int id);

	/**
	 * Crea el modelo de datos.
	 * 
	 * @param dataModel
	 * @exception Lanza
	 *                excepcion si existe otro modelo de datos con el mismo nombre
	 */
	public void dataModelCreate(DataModel dataModel) throws ExceptionKiuPlus;

	/**
	 * Modifica el modelo de datos.
	 * 
	 * @param dataModel
	 * @exception Lanza
	 *                excepcion si existe otro modelo de datos con el mismo nombre
	 */
	public void dataModelEdit(DataModel dataModel) throws ExceptionKiuPlus;

	/**
	 * Elimina el data model
	 * 
	 * @param id
	 * @throws ExceptionKiuPlus
	 *             No existe el dataModel a eliminar
	 *             Tiene metricas instanciadas asociadas.
	 */				
	public void dataModelDelete(int id) throws ExceptionKiuPlus;

	/**
	 * Obtiene de la bd de configuracion las tablas con el nombre
	 * 
	 * @param name
	 * @return
	 */
	public List<AttributeTable> tableAttributeFind(String name);

	/**
	 * Obtiene de la bd de configuracion las columnas con el nombre
	 * 
	 * @param name
	 * @return
	 */
	public List<AttributeColumn> columnsAttributeFind(String name);

	
	/** Exporta el modelo de datos con sus metricas instanciadas y resultados de grupos a un archivo pdf
	 * @param dataModel
	 * @return
	 * @throws IOException
	 * @throws DocumentException
	 */
	public ByteArrayOutputStream  dataQualityModelExportToPDF(DataModel dataModel, String path) throws IOException, DocumentException;

	
	
	
}
