package com.kiuplus.qualityModel.logic.service;

import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.itextpdf.text.BaseColor;
import com.itextpdf.text.Chunk;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Element;
import com.itextpdf.text.Font;
import com.itextpdf.text.FontFactory;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.Phrase;
import com.itextpdf.text.TabSettings;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;
import com.itextpdf.text.Image;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.Attribute.EnumAttributeType;
import com.kiuplus.logic.model.AttributeColumn;
import com.kiuplus.logic.model.AttributeTable;
import com.kiuplus.logic.model.DataModel;
import com.kiuplus.logic.model.InstancedMetric;
import com.kiuplus.logic.model.ParameterAttribute;
import com.kiuplus.logic.model.ParameterValue;
import com.kiuplus.logic.model.Reference;
import com.kiuplus.logic.model.ResultGroup;
import com.kiuplus.logic.model.ResultGroup.EnumResultGroup;
import com.kiuplus.qualityModel.logic.domain.DataModelDAO;
import com.kiuplus.qualityModel.logic.domain.DataModelDAOImpl;

/**
 * @author Beatriz
 *
 */
@Service
public class DataModelsServiceImpl implements DataModelsService {

	@Autowired
	private DataModelDAO dao;

	// Fuentes para exportar a PDF
	private Font fontTitle1 = FontFactory.getFont(FontFactory.HELVETICA_BOLD, 18, new BaseColor(84, 117, 62));
	private Font fontTitle2 = FontFactory.getFont(FontFactory.HELVETICA_BOLD, 14, new BaseColor(96, 134, 70));
	private Font fontTitle3 = FontFactory.getFont(FontFactory.HELVETICA, 13, new BaseColor(108, 150, 79));
//	private Font fontTitle4 = FontFactory.getFont(FontFactory.HELVETICA, 12, BaseColor.BLACK);

	private Font fontTextTitle = FontFactory.getFont(FontFactory.HELVETICA_BOLD, 10, BaseColor.BLACK);
	private Font fontText = FontFactory.getFont(FontFactory.HELVETICA, 10, BaseColor.BLACK);

	public DataModelsServiceImpl() {
		this.dao = new DataModelDAOImpl();
	}

	public List<DataModel> dataModelsGet() {
		List<DataModel> dataModels = dao.dataModelsGet();
		// for (DataModel dataModel : dataModels) {
		// // Desencripta la contrasea
		// try {
		// dataModel.getDatabaseSource()
		// .setPassword(this.decryptedPassword(dataModel.getDatabaseSource().getPasswordEncrypted()));
		// } catch (ExceptionKiuPlus e) {
		// // TODO: handle exception
		// }
		// }
		return dataModels;
	}

	public DataModel dataModelGet(int id) {

		DataModel dataModel = dao.dataModelGet(id);

		return dataModel;
	}

	public DataModel dataModelGetFull(int id) {
		DataModel dataModel = dao.dataModelGetFull(id);

		return dataModel;
	}

	public void dataModelCreate(DataModel dataModel) throws ExceptionKiuPlus {

		List<DataModel> dataModelDao = dao.dataModelFind(dataModel.getName());

		// Verifica que no exista un dataModel con el mismo nombre

		if (dataModelDao.size() > 0) {

			throw new ExceptionKiuPlus("There is a data model with the same name.");
		}

		dao.dataModelAdd(dataModel);
	}

	public void dataModelEdit(DataModel dataModel) throws ExceptionKiuPlus {

		List<DataModel> dataModelDao = dao.dataModelFind(dataModel.getName());

		// Verifica que no exista un dataModel con el mismo nombre

		for (DataModel dm : dataModelDao) {
			// hay otro DataModel con el mimso nombre y distinto id
			if (dm.getId() != dataModel.getId())
				throw new ExceptionKiuPlus("There is a data model with the same name.");
		}
		dao.dataModelUpdate(dataModel);
	}

	public void dataModelDelete(int id) throws ExceptionKiuPlus {
		DataModel dataModel = this.dataModelGet(id);
		if (dataModel == null) {
			throw new ExceptionKiuPlus("There is no data model");
		}

		if (dataModel.getInstancedMetrics().size() > 0) {
			throw new ExceptionKiuPlus("The data model has instanced metrics.");
		}

		this.dao.dataModelDelete(dataModel);
	}

	public List<AttributeColumn> columnsAttributeFind(String name) {
		return this.dao.columnsAttributeFind(name);
	}

	public List<AttributeTable> tableAttributeFind(String name) {
		return this.dao.tablesAttributeFind(name);
	}

	// Auxiliares

	/**
	 * Exporta el modelo de datos a PDF: informacion del modelo de datos, metricas instanciadas,
	 * resultados de grupos y resultados
	 * 
	 * @param idDataModel
	 * @return
	 * @throws FileNotFoundException
	 * @throws DocumentException
	 * @throws IOException
	 */
	public ByteArrayOutputStream dataQualityModelExportToPDF(DataModel dataModel, String pathImage)
			throws FileNotFoundException, DocumentException {

		ByteArrayOutputStream baos = new ByteArrayOutputStream();

		try {
			Document document = new Document();
			PdfWriter.getInstance(document, baos);

			document.open();

			try {

				Image image = Image.getInstance(pathImage + "/logo_kiuplus.png");

				image.setAlignment(Element.ALIGN_RIGHT);
				image.scaleToFit(128, 31);

				document.add(image);

			} catch (Exception e) {
				e.printStackTrace();
			}

			Paragraph p = new Paragraph();
			p.setAlignment(Paragraph.ALIGN_CENTER);
			p.add(new Chunk(dataModel.getName(), fontTitle1));
			p.add(new Chunk("\n"));

			document.add(p);
			document.add(Chunk.NEWLINE);

			// Database
			p = new Paragraph();
			p.setAlignment(Paragraph.ALIGN_CENTER);
			p.add(new Chunk("Database", fontTitle2));
			document.add(p);
			document.add(Chunk.NEWLINE);

			p = new Paragraph();
			p.setTabSettings(new TabSettings(58f));
			p.setIndentationLeft(20f);
			// Name
			p.add(new Chunk("Name:", fontTextTitle));
			p.add(Chunk.TABBING);
			p.add(new Chunk(dataModel.getDatabaseSource().getName(), fontText));
			p.add(Chunk.NEWLINE);

			// Url
			p.add(new Chunk("Url:", fontTextTitle));
			p.add(Chunk.TABBING);
			p.add(new Chunk(dataModel.getDatabaseSource().getUrl(), fontText));
			p.add(Chunk.NEWLINE);

			// System
			p.add(new Chunk("System:", fontTextTitle));
			p.add(Chunk.TABBING);
			p.add(new Chunk(dataModel.getDatabaseSource().getSystem().toString(), fontText));

			document.add(p);
			document.add(Chunk.NEWLINE);

			// Instanced metric
			p = new Paragraph();
			p.add(new Chunk("Instanced metrics", fontTitle2));
			p.setAlignment(Paragraph.ALIGN_CENTER);
			document.add(p);
			document.add(new Paragraph(" "));

			for (InstancedMetric im : dataModel.getInstancedMetrics()) {
				p = new Paragraph();
				p.setIndentationLeft(20f);
				// Name
				p.add(new Chunk(im.getName(), fontTitle3));
				document.add(p);
				document.add(Chunk.NEWLINE);

				p = new Paragraph();

				p.setIndentationLeft(40f);
				// Dimension
				p.add(new Chunk("Dimension:", fontTextTitle));
				p.add(Chunk.TABBING);
				p.add(new Chunk(im.getMethod().getMetrics().getFactor().getDimension().getName(), fontText));
				p.add(Chunk.NEWLINE);

				// Factor
				p.add(new Chunk("Factor: ", fontTextTitle));
				p.add(Chunk.TABBING);
				p.add(new Chunk(im.getMethod().getMetrics().getFactor().getName(), fontText));
				p.add(Chunk.NEWLINE);

				// Metrics
				p.add(new Chunk("Metrics:", fontTextTitle));
				p.add(Chunk.TABBING);
				p.add(new Chunk(im.getMethod().getMetrics().getName(), fontText));
				p.add(Chunk.NEWLINE);

				p.add(new Chunk("   Granularity:", fontTextTitle));
				p.add(Chunk.TABBING);
				p.add(new Chunk(im.getMethod().getMetrics().getGranularity().getText(), fontText));
				p.add(Chunk.NEWLINE);

				// Method
				p.add(new Chunk("Method:", fontTextTitle));
				p.add(Chunk.TABBING);
				p.add(new Chunk(im.getMethod().getName(), fontText));
				p.add(Chunk.NEWLINE);

				// Operation
				p.add(new Chunk("Operation:", fontTextTitle));
				p.add(Chunk.NEWLINE);

				// Url:
				p.add(Chunk.TABBING);
				p.add(new Chunk("Url:", fontTextTitle));
				p.add(Chunk.TABBING);
				p.add(new Chunk(im.getMethod().getOperation().getWebService().getUrl(), fontText));
				p.add(Chunk.NEWLINE);

				// Operation
				p.add(Chunk.TABBING);
				p.add(new Chunk("Name:", fontTextTitle));
				p.add(Chunk.TABBING);
				p.add(new Chunk(im.getMethod().getOperation().getName(), fontText));
				p.add(Chunk.NEWLINE);

				// Operation parameters
				p.add(Chunk.TABBING);
				p.add(new Chunk("Parameters:", fontTextTitle));
				p.add(Chunk.NEWLINE);
				document.add(p);
				document.add(new Paragraph(" "));
				// Parameters
				PdfPTable table = new PdfPTable(5);

				table.addCell(new Phrase("Name", fontTextTitle));
				table.addCell(new Phrase("Type", fontTextTitle));
				table.addCell(new Phrase("Table", fontTextTitle));
				table.addCell(new Phrase("Column", fontTextTitle));
				table.addCell(new Phrase("Value", fontTextTitle));

				for (ParameterValue parameterValue : im.getParameterValues()) {

					table.addCell(new Phrase(parameterValue.getParameter().getName() + " ("
							+ parameterValue.getParameter().getDataTypeText() + ")", fontText));
					table.addCell(new Phrase(parameterValue.getParameter().getParameterType().getText(), fontText));
					switch (parameterValue.getParameter().getParameterType()) {
					case ATTRIBUTE:

						ParameterAttribute parameterAttribute = (ParameterAttribute) parameterValue;
						if (parameterAttribute.getAttribute().getTypeAttribute().equals(EnumAttributeType.COLUMNS)) {
							AttributeColumn ac = (AttributeColumn) parameterAttribute.getAttribute();
							table.addCell(new Phrase(ac.getAttributeTable().getName(), fontText));
							table.addCell(new Phrase(ac.getName(), fontText));
						}

						if (parameterAttribute.getAttribute().getTypeAttribute().equals(EnumAttributeType.TABLES)) {
							AttributeTable at = (AttributeTable) parameterAttribute.getAttribute();
							table.addCell("");
							table.addCell(new Phrase(at.getName(), fontText));

						}
						table.addCell("");
						break;
					case REFERENCE:
						Reference reference = (Reference) parameterValue;
						table.addCell("");
						table.addCell("");
						table.addCell(new Phrase(reference.getReferenceValue(), fontText));

						break;
					case COLUMN_NAME:

						ParameterAttribute parameterAttribute2 = (ParameterAttribute) parameterValue;
						if (parameterAttribute2.getAttribute().getTypeAttribute().equals(EnumAttributeType.COLUMNS)) {
							AttributeColumn ac = (AttributeColumn) parameterAttribute2.getAttribute();
							table.addCell(new Phrase(ac.getAttributeTable().getName(), fontText));
							table.addCell(new Phrase(ac.getName(), fontText));
							table.addCell("");
						}
						break;
					case TABLE_NAME:
						ParameterAttribute parameterAttributeT = (ParameterAttribute) parameterValue;
						if (parameterAttributeT.getAttribute().getTypeAttribute().equals(EnumAttributeType.TABLES)) {
							AttributeTable at = (AttributeTable) parameterAttributeT.getAttribute();
							table.addCell(new Phrase(at.getName(), fontText));
						}
						table.addCell("");
						table.addCell("");
						break;
					default:
						table.addCell("");
						table.addCell("");
						table.addCell("");
					}

				}
				document.add(table);
				document.add(Chunk.NEWLINE);
			}

			// Aggregations

			p = new Paragraph();
			p.add(new Chunk("Aggregations", fontTitle2));
			p.setAlignment(Paragraph.ALIGN_CENTER);
			document.add(p);
			document.add(new Paragraph(" "));

			for (ResultGroup resultGroups : dataModel.getResultGroups()) {
				if (resultGroups.getType().equals(EnumResultGroup.Aggregation)) {
					dataQualityModelExportToPDFAggregation(document, resultGroups);
				}
			}

			// Composed Instanced Metrics
			p = new Paragraph();
			p.add(new Chunk("Composed Instanced Metrics", fontTitle2));
			p.setAlignment(Paragraph.ALIGN_CENTER);
			document.add(p);
			document.add(new Paragraph(" "));

			for (ResultGroup resultGroups : dataModel.getResultGroups()) {
				if (resultGroups.getType().equals(EnumResultGroup.ComposedInstancedMetric)) {
					dataQualityModelExportToPDFIMComposed(document, resultGroups);
				}
			}

			document.close();
			return baos;
		} catch (DocumentException e) {
			e.printStackTrace();
			throw e;
		}
	}

	private void dataQualityModelExportToPDFAggregation(Document document, ResultGroup resultGroups)
			throws DocumentException {

		Paragraph p = new Paragraph();
		p.setIndentationLeft(20f);
		// Name
		p.add(new Chunk(resultGroups.getName(), fontTitle3));
		document.add(p);
		document.add(Chunk.NEWLINE);

		p = new Paragraph();

		// Instanced Metric
		p.add(Chunk.TABBING);
		p.add(new Chunk("Instanced Metric:", fontTextTitle));
		p.add(Chunk.TABBING);
		p.add(new Chunk(resultGroups.getInstancedMetric().getName(), fontText));
		p.add(Chunk.NEWLINE);

		// End Granularity
		p.add(Chunk.TABBING);
		p.add(new Chunk("End Granularity:", fontTextTitle));
		p.add(Chunk.TABBING);
		p.add(new Chunk(resultGroups.getGranularity().getText(), fontText));
		p.add(Chunk.NEWLINE);

		// Url Web service
		p.add(Chunk.TABBING);
		p.add(new Chunk("Url:", fontTextTitle));
		p.add(Chunk.TABBING);
		p.add(Chunk.TABBING);
		p.add(Chunk.TABBING);
		p.add(new Chunk(resultGroups.getOperation().getWebService().getUrl(), fontText));
		p.add(Chunk.NEWLINE);

		// Operation
		p.add(Chunk.TABBING);
		p.add(new Chunk("Operation:", fontTextTitle));
		p.add(Chunk.TABBING);
		p.add(Chunk.TABBING);
		p.add(new Chunk(resultGroups.getOperation().getName(), fontText));
		p.add(Chunk.NEWLINE);

		// Operation parameters
		p.add(Chunk.TABBING);
		p.add(Chunk.TABBING);
		p.add(new Chunk("Parameters:", fontTextTitle));
		p.add(Chunk.NEWLINE);
		document.add(p);
		document.add(new Paragraph(" "));

		// Parameters
		PdfPTable table;

		table = new PdfPTable(3);

		table.addCell(new Phrase("Name", fontTextTitle));
		table.addCell(new Phrase("Type", fontTextTitle));
		table.addCell(new Phrase("Value", fontTextTitle));

		for (ParameterValue parameterValue : resultGroups.getParameterValues()) {

			table.addCell(new Phrase(parameterValue.getParameter().getName() + " ("
					+ parameterValue.getParameter().getDataType().getText() + ")", fontText));

			switch (parameterValue.getParameter().getParameterType()) {

			case COLUMN_NAME:
				table.addCell(new Phrase("Name of the results column", fontText));
				table.addCell(" ");
				break;
			case TABLE_NAME:
				table.addCell(new Phrase("Name of the results table", fontText));
				table.addCell(" ");
				break;
			case REFERENCE:
				table.addCell(new Phrase("Reference", fontText));
				Reference reference = (Reference) parameterValue;
				table.addCell(new Phrase(reference.getReferenceValue(), fontText));
				break;
			case CONNECT_TO_DB:
				table.addCell(new Phrase("Connect to DB", fontText));
				table.addCell(" ");
				break;
			default:
				table.addCell(" ");

			}
		}
		document.add(table);
		document.add(Chunk.NEWLINE);
	}

	private void dataQualityModelExportToPDFIMComposed(Document document, ResultGroup resultGroups)
			throws DocumentException {

		Paragraph p = new Paragraph();
		p.setIndentationLeft(20f);
		// Name
		p.add(new Chunk(resultGroups.getName(), fontTitle3));
		document.add(p);
		document.add(Chunk.NEWLINE);

		p = new Paragraph();

		// Instanced Metric
		p.add(Chunk.TABBING);
		p.add(new Chunk("Instanced Metric:", fontTextTitle));
		p.add(Chunk.TABBING);
		p.add(new Chunk(resultGroups.getInstancedMetric().getName(), fontText));
		p.add(Chunk.NEWLINE);

		// Instanced Metric Granularity
		p.add(Chunk.TABBING);
		p.add(Chunk.TABBING);
		p.add(new Chunk("IM Granularity:", fontTextTitle));
		p.add(Chunk.TABBING);
		p.add(new Chunk(resultGroups.getInstancedMetric().getMethod().getMetrics().getGranularity().getText(),
				fontText));
		p.add(Chunk.NEWLINE);

		// End Granularity
		p.add(Chunk.TABBING);
		p.add(new Chunk("End Granularity:", fontTextTitle));
		p.add(Chunk.TABBING);
		p.add(new Chunk(resultGroups.getGranularity().getText(), fontText));
		p.add(Chunk.NEWLINE);

		// Url Web service
		p.add(Chunk.TABBING);
		p.add(new Chunk("Url:", fontTextTitle));
		p.add(Chunk.TABBING);
		p.add(Chunk.TABBING);
		p.add(Chunk.TABBING);
		p.add(new Chunk(resultGroups.getOperation().getWebService().getUrl(), fontText));
		p.add(Chunk.NEWLINE);

		// Operation
		p.add(Chunk.TABBING);
		p.add(new Chunk("Operation:", fontTextTitle));
		p.add(Chunk.TABBING);
		p.add(Chunk.TABBING);
		p.add(new Chunk(resultGroups.getOperation().getName(), fontText));
		p.add(Chunk.NEWLINE);

		// Operation parameters
		p.add(Chunk.TABBING);
		p.add(Chunk.TABBING);
		p.add(new Chunk("Parameters:", fontTextTitle));
		p.add(Chunk.NEWLINE);
		document.add(p);
		document.add(new Paragraph(" "));

		// Parameters
		PdfPTable table;

		table = new PdfPTable(3);

		table.addCell(new Phrase("Name", fontTextTitle));
		table.addCell(new Phrase("Type", fontTextTitle));
		table.addCell(new Phrase("Value", fontTextTitle));

		for (ParameterValue parameterValue : resultGroups.getParameterValues()) {

			table.addCell(new Phrase(parameterValue.getParameter().getName() + " ("
					+ parameterValue.getParameter().getDataType().getText() + ")", fontText));

			switch (parameterValue.getParameter().getParameterType()) {

			case ATTRIBUTE:
				table.addCell(new Phrase("Result of", fontText));
				table.addCell(new Phrase(parameterValue.getInstancedMetric().getFullName(), fontText));
				break;
			case REFERENCE:
				table.addCell(new Phrase("Reference", fontText));
				Reference reference = (Reference) parameterValue;
				table.addCell(new Phrase(reference.getReferenceValue(), fontText));
				break;
			default:
				table.addCell(" ");
			}
		}
		document.add(table);
		document.add(Chunk.NEWLINE);
	}

}
