package com.kiuplus.qualityModel.logic.service;

import java.util.List;

import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.AttributeColumn;
import com.kiuplus.logic.model.AttributeDatabase;
import com.kiuplus.logic.model.AttributeTable;

public interface DatabaseService {

	/**
	 * Retorna las columnas de la tabla Las columnas tienen asociadas la tabla a la cual pertenecen
	 * 
	 * @param database
	 * @param attributeTable
	 * @return
	 * @throws ExceptionKiuPlus
	 *             Indica problemas al conectarse a a la base o al obtener el esquema
	 */
	public List<AttributeColumn> columnsGet(AttributeDatabase database, AttributeTable attributeTable)
			throws ExceptionKiuPlus;

	/**
	 * Retorna las tablas de la base de datos sin las columnas
	 * 
	 * @param database
	 * @return
	 * @throws ExceptionKiuPlus
	 *             Indica problemas al conectarse a a la base o al obtener el esquema
	 */
	public List<AttributeTable> tablesFind(AttributeDatabase database) throws ExceptionKiuPlus;

	/**
	 * Obtiene todas las tablas y sus columnas de la base de datos
	 * 
	 * @param database
	 * @return
	 * @throws Indica
	 *             problemas al conectarse a a la base o al obtener el esquema
	 */
	public List<AttributeTable> tablesColumnsGet(AttributeDatabase database) throws ExceptionKiuPlus;

	/**
	 * Retorna la base de datos con sus tablas y las columnas.
	 * 
	 * @param databaseSource
	 * @return
	 * @throws ExceptionKiuPlus
	 */
	public AttributeDatabase databaseGet(AttributeDatabase databaseSource) throws ExceptionKiuPlus;

	/**
	 * Intenta conectarse a la base de datos
	 * 
	 * @param dataBase
	 *            Datos de la base a la cual se testea la conexion
	 * @return La conexion a la base fue exitosa
	 * @throws ExceptionKiuPlus
	 *             Error indicando la causa de la falla de la conexion
	 */
	public boolean dataBaseTestConnection(AttributeDatabase dataBase) throws ExceptionKiuPlus;

	/**
	 * @param columnName
	 * @param tableName
	 * @param database
	 * @return columna identificada con el nombre de la tabla especificada
	 * @throws ExceptionKiuPlus
	 *             Problemas al conectarse a a la base o al obtener el esquema
	 */
	public AttributeColumn columnGet(String columnName, String tableName, AttributeDatabase database) throws ExceptionKiuPlus;
	
	

}
