package com.kiuplus.qualityModel.logic.service;

import java.util.ArrayList;
import java.util.List;

import org.springframework.stereotype.Service;

import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.AttributeColumn;
import com.kiuplus.logic.model.AttributeDatabase;
import com.kiuplus.logic.model.AttributeTable;
import com.kiuplus.logic.dataAccess.*;

@Service
public class DatabaseServiceImpl implements DatabaseService {

	private DataAccess getDataAccess(AttributeDatabase database) throws ExceptionKiuPlus{
		DataAccess dataAccess;
		switch (database.getSystem()){
		case MySQL:
			dataAccess = new MySQLDataAccess(database.getUrl()+ "/" + database.getName(), database.getUser(), database.getPassword());
			break;
		case PostgreSQL:
			dataAccess = new PostgreSQLDataAccess(database.getUrl()+ "/" + database.getName(), database.getUser(), database.getPassword());
			break;
		default:
			dataAccess = new DataAccess(database.getUrl()+ "/" + database.getName(), database.getUser(), database.getPassword());
			break;
		}
		return dataAccess;
	}

	/**
	 * Retorna los nombre de objetos del tipo key (TABLE, COLUMNS, DB) del esquema.
	 * 
	 * @param database
	 *            Base de datos de la que se va a obtener la informacion
	 * @param query
	 *            Consulta para obtener la informacion del esquema
	 * @param key
	 *            Tipo de objetos para obtener la informacion.
	 * @return
	 * @throws ExceptionKiuPlus
	 */
	public List<AttributeColumn> columnsGet(AttributeDatabase database, AttributeTable attributeTable)
			throws ExceptionKiuPlus {
		try {
			DataAccess dataAccess = getDataAccess(database);
			dataAccess.connect();
			List<AttributeColumn> columns = dataAccess.getColumnsNamesTable(database, attributeTable);
			dataAccess.disconnect();
			return columns;
		} catch (Exception e) {
			throw new ExceptionKiuPlus(e.getMessage());
		}
	}

	public List<AttributeTable> tablesFind(AttributeDatabase database) throws ExceptionKiuPlus {
		try {
			//Inicio el DataAcces que voy a utilizar para obtener los datos a evaluar
	    	DataAccess dataAccess = getDataAccess(database);
	    	// Ejecuta la conulta en BD
	    	dataAccess.connect();
			List<String> objectsExtractor = dataAccess.getTablesName(database.getName());
			dataAccess.disconnect();
			
			// Procesa las propiedades de las tablas y sus columnas
			List<AttributeTable> tables = new ArrayList<>();
			for (String object : objectsExtractor){
				AttributeTable attribute = new AttributeTable();
				attribute.setName(object);
				tables.add(attribute);
			}
			return tables;
		} catch (Exception e) {
			e.printStackTrace();
			throw new ExceptionKiuPlus("It is not possible to obtain the tables" + e.getMessage());
		}
	}

	public List<AttributeTable> tablesColumnsGet(AttributeDatabase database) throws ExceptionKiuPlus {
		try {
			DataAccess dataAccess = getDataAccess(database);
	    	// Ejecuta la conulta en BD
	    	dataAccess.connect();
			List<String> tableNames = dataAccess.getTablesName(database.getName());
			dataAccess.disconnect();
			// Procesa las propiedades de las tablas y sus columnas
			List<AttributeTable> tables = new ArrayList<>();

			for (String tableName : tableNames) {
				AttributeTable attribute = new AttributeTable();
				attribute.setName(tableName);
				attribute.setAttributesColumns(this.columnsGet(database, attribute));
				attribute.setAttributeDatabase(database);
				tables.add(attribute);
			}

			return tables;
		} catch (Exception e) {
			throw new ExceptionKiuPlus(e.getMessage());
		}

	}

	public AttributeDatabase databaseGet(AttributeDatabase database) throws ExceptionKiuPlus {
		AttributeDatabase d = new AttributeDatabase(database.getId(), database.getName(), database.getUrl(),
				database.getUser(), database.getPassword(), database.getSchema(), database.getSystem());

		List<AttributeTable> tables = this.tablesColumnsGet(database);

		d.setAttributesTable(tables);

		return d;
	}

	public boolean dataBaseTestConnection(AttributeDatabase dataBase) throws ExceptionKiuPlus {
		try {
			DataAccess dataAccess = getDataAccess(dataBase);
			dataAccess.connect();
			dataAccess.disconnect();
		} catch (Exception e) {
			throw new ExceptionKiuPlus(e.getMessage());
		}
		return true;
	}

	public AttributeColumn columnGet(String columnName, String tableName, AttributeDatabase database) throws ExceptionKiuPlus {
		try {
			//getColumnType
			DataAccess dataAccess = getDataAccess(database);
			dataAccess.connect();
			AttributeColumn attribute = dataAccess.getColumnTypeParser(database.getName(), tableName, columnName);
			dataAccess.disconnect();
			return attribute;
		} catch (Exception e) {
			throw new ExceptionKiuPlus(e.getMessage());
		}
	}

	

}
