package com.kiuplus.qualityModel.logic.service;

import java.io.ByteArrayOutputStream;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Future;

import com.itextpdf.text.DocumentException;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.AttributeDatabase;
import com.kiuplus.logic.model.DataModel;
import com.kiuplus.logic.model.InstancedMetric;
import com.kiuplus.logic.model.ResultGroup;
import com.kiuplus.logic.objects.*;


public interface ExecuteService {

	/** Ejecuta las metricas instanciadas y agregaciones y actualiza en base de datos donde se alamacenan los resultados
	 * @param instancedMetrics
	 * @param aggregations
	 * @param reloadData Indica si los datos a analizar se deben cargar de la base de datos fuente
	 * @return Retorna true si se ejecuto correctamente las metricas instanciadas
	 * @throws InterruptedException
	 * @throws ExceptionKiuPlus 
	 */
	public Future<Boolean> executeDataQaulityModel(List<InstancedMetric> instancedMetrics, List<ResultGroup> resultGroups, boolean reloadData) throws InterruptedException, ExceptionKiuPlus;
	
	/** Ejecuta las metricas instanciadas y actualiza en base de datos donde se alamacenan los resultados
	 * @param instancedMetrics
	 * @param reloadData Indica si los datos a analizar se deben cargar de la base de datos fuente
	 * @return Retorna true si se ejecuto correctamente las metricas instanciadas
	 * @throws InterruptedException
	 * @throws ExceptionKiuPlus 
	 */
	public Future<Boolean> executeInstancedMetrics(List<InstancedMetric> instancedMetrics,  boolean reloadData) throws InterruptedException, ExceptionKiuPlus;
	
	/** Ejecuta las agregaciones y actualiza en base de datos donde se alamacenan los resultados
	 * @param aggregations
	 * @param deleteIMResults
	 * @return Retorna true si se ejecuto correctamente las agregaciones
	 * @throws InterruptedException
	 * @throws ExceptionKiuPlus 
	 */
	public Future<Map<Integer, String>> executeAggregations(List<ResultGroup> aggregations, boolean deleteIMResults) throws InterruptedException, ExceptionKiuPlus;
	
	
	/** Ejecuta las composedIMs y actualiza en base de datos donde se alamacenan los resultados
	 * @param composedIMs
	 * @param deleteIMResults
	 * @return Retorna true si se ejecuto correctamente las composedIMs
	 * @throws InterruptedException
	 * @throws ExceptionKiuPlus 
	 */
	public Future<Map<Integer, String>> executeComposedIMs(List<ResultGroup> composedIMs, boolean deleteIMResults) throws InterruptedException, ExceptionKiuPlus;
	
	
	/** Elimina la tabla de resultados de la ejecucion de la metrica instanciada
	 * @param instancedMetric
	 */
	public void deleteResultExecutionInstancedMetric(InstancedMetric instancedMetric) throws Exception;
	
	/** Exporta el resultado de la ejecucion del modelo de datos
	 * @param dataModel
	 * @param instencedMetricsResult resultado de las metricas instanciadas ejecutadas
	 * @param aggregationsResult resultado de las agregaciones ejecutadas
	 * @return
	 * @throws DocumentException
	 */
	public ByteArrayOutputStream qualityModelDataModelResultExportPDF(DataModel dataModel,
			Map<InstancedMetric, List<GenericObject>> instencedMetricsResult,
			Map<ResultGroup, List<GenericObject>> resultGroupsResult, String pathImage)  throws DocumentException;
	
	
	/** Obtiene los resultados de la ejecucion
	 * @param database
	 * @param tableName
	 * @return
	 * @throws ExceptionKiuPlus
	 */
//	public List<GenericObject> resultExecutionQuery(AttributeDatabase database, String tableName) throws ExceptionKiuPlus;
	
	/** Obtiene los resultados de la ejecucion
	 * @param database
	 * @param tableName
	 * @return
	 * @throws ExceptionKiuPlus
	 */
	public List<GenericObject> resultExecutionQuery(InstancedMetric im) throws ExceptionKiuPlus;
	
	
	public List<GenericObject> resultExecutionQuery(ResultGroup resultGroup) throws ExceptionKiuPlus;
}
