package com.kiuplus.qualityModel.logic.service;

import java.io.ByteArrayOutputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.Future;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Service;

import com.itextpdf.text.BaseColor;
import com.itextpdf.text.Chunk;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Element;
import com.itextpdf.text.Font;
import com.itextpdf.text.FontFactory;
import com.itextpdf.text.Image;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.Phrase;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.extractors.Extractor;
import com.kiuplus.logic.extractors.MySQL;
import com.kiuplus.logic.model.AttributeDatabase;
import com.kiuplus.logic.model.DataModel;
import com.kiuplus.logic.model.InstancedMetric;
import com.kiuplus.logic.model.Parameter.EnumParameterType;
import com.kiuplus.logic.model.ParameterValue;
import com.kiuplus.logic.model.ResultGroup;
import com.kiuplus.logic.objects.*;

import com.kiuplus.qualityModel.logic.domain.ExecuteServiceDAO;

@Service
public class ExecuteServiceImpl implements ExecuteService {

	@Autowired
	private ExecuteServiceDAO executeServiceDAO;

	@Async
	public Future<Boolean> executeDataQaulityModel(List<InstancedMetric> instancedMetrics,
			List<ResultGroup> resultGroups, boolean reloadData) throws InterruptedException, ExceptionKiuPlus {
		try {
			this.executeServiceDAO.executeMetrics(instancedMetrics, reloadData);

		} catch (Exception e) {

		}
		// Fin de la ejecucion
		return new AsyncResult<Boolean>(true);
	}

	@Async
	public Future<Boolean> executeInstancedMetrics(List<InstancedMetric> instancedMetrics, boolean reloadData)
			throws InterruptedException, ExceptionKiuPlus {
		try {
			this.executeServiceDAO.executeMetrics(instancedMetrics, reloadData);

		} catch (Exception e) {

		}
		// Fin de la ejecucion
		return new AsyncResult<Boolean>(true);
	}

	@Async
	public Future<Map<Integer, String>> executeAggregations(List<ResultGroup> aggregations, boolean deleteIMResults)
			throws InterruptedException, ExceptionKiuPlus {
		Map<Integer, String> result = new HashMap<Integer, String>();
		try {
			// Ejecutar agregaciones
			result = this.executeServiceDAO.executeAggregations(aggregations, deleteIMResults);
		} catch (Exception e) {
			// throw new ExceptionKiuPlus(e.getMessage());
		}
		// Fin de la ejecucion
		return new AsyncResult<Map<Integer, String>>(result);
	}

	public Future<Map<Integer, String>> executeComposedIMs(List<ResultGroup> composedIMs, boolean deleteIMResults)
			throws InterruptedException, ExceptionKiuPlus {

		Map<Integer, String> result = new HashMap<Integer, String>();
		try {
			// Ejecutar composed IM
			// result = this.executeServiceDAO.executeComposedIM(composedIMs, deleteIMResults);
		} catch (Exception e) {
			throw new ExceptionKiuPlus(e.getMessage());
		}
		// Fin de la ejecucion
		return new AsyncResult<Map<Integer, String>>(result);
	}

	public void deleteResultExecutionInstancedMetric(InstancedMetric instancedMetric) throws Exception {
		AttributeDatabase database = instancedMetric.getDataModel().getDatabaseMetadata();
		Extractor extractor = new MySQL(database.getServer(), database.getUser(), database.getPassword());
		extractor.Connect();
		String query = "DROP TABLE IF EXISTS " + instancedMetric.getMetadataTableResult();
		extractor.ExecuteUpdate(query);
		extractor.Disconnect();
	}

	public ByteArrayOutputStream qualityModelDataModelResultExportPDF(DataModel dataModel,
			Map<InstancedMetric, List<GenericObject>> instencedMetricsResult,
			Map<ResultGroup, List<GenericObject>> resultGroupsResult, String pathImage) throws DocumentException {

		DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");

		ByteArrayOutputStream baos = new ByteArrayOutputStream();

		try {
			Document document = new Document();
			PdfWriter.getInstance(document, baos);

			document.open();

			Font fontTitle1 = FontFactory.getFont(FontFactory.HELVETICA_BOLD, 18, new BaseColor(84, 117, 62));
			Font fontTitle2 = FontFactory.getFont(FontFactory.HELVETICA_BOLD, 14, new BaseColor(96, 134, 70));
			Font fontTitle3 = FontFactory.getFont(FontFactory.HELVETICA, 13, new BaseColor(108, 150, 79));
			Font fontTitle4 = FontFactory.getFont(FontFactory.HELVETICA, 12, BaseColor.BLACK);

			Font fontTextTitle = FontFactory.getFont(FontFactory.HELVETICA_BOLD, 10, BaseColor.BLACK);
			Font fontText = FontFactory.getFont(FontFactory.HELVETICA, 10, BaseColor.BLACK);

			try {

				Image image = Image.getInstance(pathImage + "/logo_kiuplus.png");

				image.setAlignment(Element.ALIGN_RIGHT);
				image.scaleToFit(128, 31);
				document.add(image);

			} catch (Exception e) {
				e.printStackTrace();
			}

			Paragraph p = new Paragraph();
			p.setAlignment(Paragraph.ALIGN_CENTER);
			p.add(new Chunk("Result of Execut " + dataModel.getName(), fontTitle1));
			document.add(p);
			document.add(Chunk.NEWLINE);

			// Resultado de las metricas instanciadas

			p = new Paragraph();
			p.setAlignment(Paragraph.ALIGN_CENTER);
			p.add(new Chunk("Instanced Metrics", fontTitle2));
			document.add(p);
			document.add(Chunk.NEWLINE);

			for (InstancedMetric im : instencedMetricsResult.keySet()) {
				List<GenericObject> genericObjects = instencedMetricsResult.get(im);
				p = new Paragraph();
				p.add(new Chunk(im.getName(), fontTitle3));
				p.add(Chunk.NEWLINE);
				p.add(Chunk.TABBING);

				if (!genericObjects.isEmpty()) {
					p.add(new Chunk("Date of execution: ", fontTextTitle));
					p.add(new Chunk(df.format(im.getDateTimeResult()), fontText));
				}
				document.add(p);

				genericObjectToPDF(genericObjects, document, fontTextTitle, fontText);

			}

			// Resultado de las aggregaciones
			p = new Paragraph();
			p.setAlignment(Paragraph.ALIGN_CENTER);
			p.add(new Chunk("Aggregations", fontTitle2));
			document.add(p);
			document.add(Chunk.NEWLINE);

			for (ResultGroup a : resultGroupsResult.keySet()) {

				List<GenericObject> genericObjects = resultGroupsResult.get(a);

				p = new Paragraph();
				p.add(new Chunk(a.getName(), fontTitle3));
				p.add(Chunk.TABBING);
				document.add(p);

				if (!genericObjects.isEmpty()) {
					p = new Paragraph();
					
					p.add(Chunk.TABBING);
					p.add(new Chunk("End granularity: ", fontTextTitle));
					p.add(Chunk.TABBING);
					p.add(new Chunk(a.getGranularity().getText(), fontText));
					p.add(Chunk.NEWLINE);
					p.add(Chunk.TABBING);
					p.add(new Chunk("Instanced metrics: ", fontTextTitle));
					p.add(Chunk.TABBING);

					for (ParameterValue pv : a.getParameterValues()) {
						if (pv.getParameter().getParameterType().equals(EnumParameterType.TABLE_NAME)) {
													
							p.add(new Chunk(pv.getInstancedMetric().getName(), fontText));							
							p.add(Chunk.NEWLINE);
							p.add(Chunk.TABBING);
							p.add(Chunk.TABBING);
							p.add(Chunk.TABBING);
							p.add(Chunk.TABBING);
						}
					}
					document.add(p);
				}

				genericObjectToPDF(genericObjects, document, fontTextTitle, fontText);
			}

			document.close();
			return baos;
		} catch (DocumentException e) {
			e.printStackTrace();
			throw e;
		}

	}

	private void genericObjectToPDF(List<GenericObject> genericObjects, Document document, Font fontTextTitle,
			Font fontText) throws DocumentException {
		Set<String> listKeys;

		Font fontTextNull = FontFactory.getFont(FontFactory.HELVETICA, 10, Font.ITALIC, BaseColor.GRAY);

		if (!genericObjects.isEmpty()) {
			document.add(Chunk.NEWLINE);
			listKeys = genericObjects.get(0).getProperties().keySet();

			PdfPTable table = new PdfPTable(listKeys.size());

			for (String columnName : listKeys) {
				table.addCell(new Phrase(columnName, fontTextTitle));
			}

			for (GenericObject genericObject : genericObjects) {

				for (String value : genericObject.getProperties().values()) {
					if (value == null) {
						table.addCell(new Phrase("null", fontTextNull));
					} else {
						table.addCell(new Phrase(value, fontText));
					}
				}

			}
			document.add(table);

		} else {
			Paragraph p = new Paragraph();
			p.add(Chunk.TABBING);
			p.add(new Chunk("Has not results", fontText));
			document.add(p);
		}
		document.add(Chunk.NEWLINE);
	}

	// public List<GenericObject> resultExecutionQuery(AttributeDatabase database, String tableName)
	// throws ExceptionKiuPlus {
	// try {
	// // Ejecuta la conulta en BD
	// Extractor extractor = new MySQL(database.getServer(), database.getUser(),
	// database.getPassword());
	// extractor.Connect();
	// String query = "SELECT * FROM " + tableName;
	// List<GenericObject> objectsExtractor = extractor.ExecuteQuery(query);
	//
	// // Coloca el resultado en la utlima fila
	// for (GenericObject go : objectsExtractor) {
	// String result = go.getProperties().get("Result");
	// go.getProperties().remove("Result");
	// go.getProperties().put("Result", result);
	//
	// }
	//
	// extractor.Disconnect();
	// return objectsExtractor;
	// } catch (Exception e) {
	// throw new ExceptionKiuPlus(e.getMessage());
	// }
	// }

	public List<GenericObject> resultExecutionQuery(InstancedMetric im) throws ExceptionKiuPlus {
		try {
			return this.executeServiceDAO.getMetricResult(im);
		} catch (Exception e) {
			throw new ExceptionKiuPlus(e.getMessage());
		}
	}

	public List<GenericObject> resultExecutionQuery(ResultGroup resultGroup) throws ExceptionKiuPlus {
		try {
			return this.executeServiceDAO.getAggregationResult(resultGroup);
		} catch (Exception e) {
			throw new ExceptionKiuPlus(e.getMessage());
		}
	}

}
