package com.kiuplus.qualityModel.logic.service;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.InstancedMetric;
import com.kiuplus.logic.model.ParameterAttribute;
import com.kiuplus.qualityModel.logic.domain.InstancedMetricDAO;
import com.kiuplus.qualityModel.logic.domain.InstancedMetricDAOImpl;

/**
 * @author Beatriz
 *
 */
@Service
public class InstancedMetricsServiceImpl implements InstancedMetricsService {
	// SELECT * FROM INFORMATION_SCHEMA.COLUMNS

	@Autowired
	private InstancedMetricDAO dao;

	@Autowired
	private ResultGroupsService resultGroupsService;
	

	public InstancedMetricsServiceImpl() {
		this.dao = new InstancedMetricDAOImpl();
	}


	public List<InstancedMetric> instancedMetricsGet(int idDataModel) {
		
		return dao.instancedMetricsGet(idDataModel);
	}
	
	// ****************************************************//
	// Instanced Metrics ABM
	// ****************************************************//

	public InstancedMetric instancedMetricGet(int id) {
		return this.dao.instancedMetricGet(id);
	}
	

	public List<InstancedMetric> instancedMetricsGet(List<Integer> ids){
		return this.dao.instancedMetricsGet(ids);
	}

	
	public InstancedMetric instancedMetricFind(String name) {
		List<InstancedMetric> instancedMetrics = this.dao.instancedMetricFind(name);
		return instancedMetrics.isEmpty() ? new InstancedMetric() : instancedMetrics.get(0);
	}
	
	public void instancedMetricCreate(InstancedMetric instancedMetric) throws ExceptionKiuPlus {
		List<InstancedMetric> instancedMetricDao = dao.instancedMetricFind(instancedMetric.getName());

		// Verifica que no exista una metrica instanciada con el mismo nombre

		if (instancedMetricDao.size() > 0) {

			throw new ExceptionKiuPlus("There is a instanced metric with the same name.");
		}

		dao.instancedMetricAdd(instancedMetric);

	}

	
	public void instancedMetricEdit(InstancedMetric instancedMetric) throws ExceptionKiuPlus {
		List<InstancedMetric> instancedMetricDao = dao.instancedMetricFind(instancedMetric.getName());

		// Verifica que no exista otra metrica instanciada con el mismo nombre

		for (InstancedMetric imd : instancedMetricDao) {
			// hay otra metrica instanciada con el mimso nombre y distinto id
			if (instancedMetric.getId() != imd.getId())
				throw new ExceptionKiuPlus("There is a instanced metric with the same name.");
		}

		dao.instancedMetricUpdate(instancedMetric);

	}

	public void instancedMetricDelete(int id) throws ExceptionKiuPlus {
		InstancedMetric instancedMetric = this.instancedMetricGet(id);

		
		if(this.resultGroupsService.instancedMetricHasResultGroups(id)) {
			throw new ExceptionKiuPlus("Can not delete the metric instantiated: is used in aggregations.");
		}
		if (instancedMetric == null) {
			throw new ExceptionKiuPlus("There is no instanced metric");
		}

		this.dao.instancedMetricDelete(instancedMetric);

	}

	// ****************************************************//
	// ParameterAttribute
	// ****************************************************//

	public ParameterAttribute parameterAttributeGet(int id) {
		return this.dao.parameterAttributeGet(id);
	}



	

}
