package com.kiuplus.qualityModel.logic.service;

import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.util.List;

import com.itextpdf.text.DocumentException;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.ResultGroup;
import com.kiuplus.logic.objects.*;

public interface ResultGroupsService {

	
	/** 
	 * @param id
	 * @return grupo de resultado identificada con el id
	 */
	public ResultGroup resultGroupGet(int id);
	
	
	/** Obtiene los grupos de resultados identificadas con los ids
	 * @param resultGroupsIds
	 * @return
	 */
	public List<ResultGroup> resultGroupsGet(List<Integer> resultGroupsIds);
	
	/** Crea el grupo de resultado
	 * @param resultGroupGranularity 
	 * @throws ExceptionKiuPlus Si existe otro grupo de resultado con el mismo nombre
	 */
	public void resultGroupCreate(ResultGroup resultGroup) throws ExceptionKiuPlus;

	/** Elimina el grupo de resultado identificado con el id
	 * @param id
	 * @throws ExceptionKiuPlus 
	 */
	public void resultGroupDelete(int id) throws ExceptionKiuPlus;

	/** Actualiza el grupo de resultado identificado con el id
 	 * @param resultGroup
	 * @throws ExceptionKiuPlus 
	 */
	public void resultGroupEdit(ResultGroup resultGroup) throws ExceptionKiuPlus;

	/** 
	 * @param idInstancedMetric identificador de la metrica instanciada
	 * @return Existen grupo de resultado que tienen asociadas la metrica instanciada
	 */
	public boolean instancedMetricHasResultGroups(int idInstancedMetric);

	/** Exporta a PDF el resultado de la ejecucion
	 * @param resultGroup
	 * @return
	 */
	public ByteArrayOutputStream resultGroupResultExportToPDF(ResultGroup resultGroup, List<GenericObject> genericObjects) throws FileNotFoundException, DocumentException;

	

}