package com.kiuplus.qualityModel.logic.service;

import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.util.List;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.itextpdf.text.BaseColor;
import com.itextpdf.text.Chunk;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Font;
import com.itextpdf.text.FontFactory;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.Phrase;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.ResultGroup;
import com.kiuplus.logic.objects.*;
import com.kiuplus.qualityModel.logic.domain.ResultGroupsDAO;

@Service
public class ResultGroupsServiceImpl implements ResultGroupsService {

	@Autowired
	private ResultGroupsDAO dao;

	@Override
	public ResultGroup resultGroupGet(int id) {
		return this.dao.resultGroupGet(id);
	}

	public List<ResultGroup> resultGroupsGet(List<Integer> resultGroupsIds){
		return this.dao.resultGroupsGet(resultGroupsIds);
	}
	
	public void resultGroupCreate(ResultGroup resultGroup) throws ExceptionKiuPlus {
		List<ResultGroup> resultGroups = dao.resultGroupsFind(resultGroup.getName());

		// Verifica que no exista un resultado de grupo con el mismo nombre

		if (resultGroups.size() > 0) {

			throw new ExceptionKiuPlus("There is a aggregation or composed instanced metric with the same name.");
		}

		dao.resultGroupCreate(resultGroup);

	}

	public void resultGroupDelete(int id) throws ExceptionKiuPlus {
		ResultGroup resultGroup = this.resultGroupGet(id);

		if (resultGroup == null) {
			throw new ExceptionKiuPlus("There is no instanced metric");
		}

		this.dao.resultGroupDelete(resultGroup);

	}

	public void resultGroupEdit(ResultGroup resultGroup) throws ExceptionKiuPlus {
		List<ResultGroup> resultGroups = dao.resultGroupsFind(resultGroup.getName());

		for (ResultGroup a : resultGroups) {
			// hay otra metrica instanciada con el mimso nombre y distinto id
			if (resultGroup.getId() != a.getId())
				throw new ExceptionKiuPlus("There is a instanced metric with the same name.");
		}

		this.dao.resultGroupEdit(resultGroup);

	}

	public boolean instancedMetricHasResultGroups(int idInstancedMetric) {
		return this.dao.parameterValueInstancedMetricFind(idInstancedMetric).size() > 0;
	}

	public ByteArrayOutputStream resultGroupResultExportToPDF(ResultGroup resultGroup,
			List<GenericObject> genericObjects) throws FileNotFoundException, DocumentException {

		ByteArrayOutputStream baos = new ByteArrayOutputStream();

		try {
			Document document = new Document();
			PdfWriter.getInstance(document, baos);

			document.open();

			Font fontTitle = FontFactory.getFont(FontFactory.HELVETICA_BOLD, 16, new BaseColor(46, 116, 181));
			Font fontSubTitle = FontFactory.getFont(FontFactory.HELVETICA_BOLD, 14, new BaseColor(46, 116, 181));
			Font fontSubTitle2 = FontFactory.getFont(FontFactory.HELVETICA, 12, new BaseColor(46, 116, 181));

			Font fontTextTitle = FontFactory.getFont(FontFactory.HELVETICA_BOLD, 10, BaseColor.BLACK);
			Font fontText = FontFactory.getFont(FontFactory.HELVETICA, 10, BaseColor.BLACK);

			Paragraph p = new Paragraph();
			p.setAlignment(Paragraph.ALIGN_CENTER);
			p.add(new Chunk(resultGroup.getName(), fontTitle));
			p.add(new Chunk("\n"));
			document.add(p);
			document.add(Chunk.NEWLINE);

			p = new Paragraph();
			p.add(new Chunk("Result of execution", fontSubTitle));
			document.add(p);
			document.add(Chunk.NEWLINE);

			Set<String> listKeys;

			if (!genericObjects.isEmpty()) {
				listKeys = genericObjects.get(0).getProperties().keySet();

				PdfPTable table = new PdfPTable(listKeys.size());

				for (String columnName : listKeys) {
					table.addCell(new Phrase(columnName, fontTextTitle));
				}

				for (GenericObject genericObject : genericObjects) {

					for (String value : genericObject.getProperties().values()) {
						table.addCell(new Phrase(value, fontText));
					}

				}
				document.add(table);
				document.add(Chunk.NEWLINE);
			}

			document.close();
			return baos;
		} catch (DocumentException e) {
			e.printStackTrace();
			throw e;
		}
	}

}
