package com.kiupluswebapp.controller;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.validation.Valid;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;
import com.kiupluswebapp.model.Factor;
import com.kiupluswebapp.model.Method;
import com.kiupluswebapp.model.Metrics;
import com.kiupluswebapp.model.Operation;

@Controller
public class ConfigurationMethod {

	// private MethodManager methodManager;

	Map<Integer, String> dimensionsList;
	Map<Integer, String> factorsList;
	Map<Integer, String> metricsList;
	Map<Integer, String> wsOperationsList;
	
	@RequestMapping(value = "/configurationMethod", method = RequestMethod.GET)
	public ModelAndView initForm() {
		ModelAndView mav = new ModelAndView("configurationMethod");

		Method method = new Method();
		mav.addObject(method);

		this.initData();
		mav = this.loadData(mav);
		
		return mav;
	}

	
	private ModelAndView loadData(ModelAndView mav){
		
	
		mav.addObject("dimensionsList", this.dimensionsList);
		
		mav.addObject("factorsList", this.factorsList);
		
		mav.addObject("metricsList", this.metricsList);	

		mav.addObject("wsOperationsList", this.wsOperationsList);

		return mav;
	} 
	
	private void initData(){
		
		this.dimensionsList = this.getDimensions();
		this.factorsList = new LinkedHashMap<Integer, String>();	
		this.metricsList = new LinkedHashMap<Integer, String>();
		this.wsOperationsList = new LinkedHashMap<Integer, String>();	
	}
	
	@RequestMapping(value = "/addMethod", method = RequestMethod.POST)
	public ModelAndView submit(@ModelAttribute("method") @Valid Method method, BindingResult result, Model model) {

		
		
		
		if (result.hasErrors()) {
			ModelAndView mav = new ModelAndView("configurationMethod");
			mav.addObject(method);
			mav = this.loadData(mav);

			return mav;
		}

		return new ModelAndView("configurationMethodList");
	}

	private Map<Integer, String> getDimensions() {
		Map<Integer, String> dimensions = new LinkedHashMap<Integer, String>();

		dimensions.put(0, "Accuracy");
		dimensions.put(1, "Completeness");
		dimensions.put(2, "Freshness");
		dimensions.put(3, "Consistency");
		dimensions.put(4, "Uniqueness");
		return dimensions;

	}

	// private void initModelList(Model model) {
	//
	// Map<Integer, String> factor = new LinkedHashMap<Integer, String>();
	// factor.put(0,"Semantic correctness");
	// factor.put(1,"Syntactic correctness");
	// factor.put(2,"Precision");
	//
	// model.addAttribute("factorList", factor);
	//
	// }

	@RequestMapping(value = "/getFactors", method = RequestMethod.GET, produces = "application/json")
	public @ResponseBody List<Factor> getFactors(@RequestParam(value = "dimension", required = true) String dimension,
			Model model) {

		List<Factor> factors = new ArrayList<Factor>();

		// Accuracy
		if (dimension.equals("0")) {
			factors.add(new Factor(0, "Semantic correctness"));
			factors.add(new Factor(1, "Syntactic correctness"));
			factors.add(new Factor(2, "Precision"));

		}

		// Completeness
		if (dimension.equals("1")) {

			factors.add(new Factor(3, "Density"));
			factors.add(new Factor(4, "Coverage"));

		}

		// Freshness
		if (dimension.equals("2")) {

			factors.add(new Factor(5, "Currency"));
			factors.add(new Factor(6, "Timeliness"));
			factors.add(new Factor(7, "Volatility"));

		}

		// Consistency
		if (dimension.equals("3")) {

			factors.add(new Factor(8, "Domain integrity"));
			factors.add(new Factor(9, "Relation integrity"));
			factors.add(new Factor(10, "Inter Relation integrity"));

		}

		// Uniqueness
		if (dimension.equals("4")) {
			factors.add(new Factor(11, "Duplicatefree"));
			factors.add(new Factor(12, "Contradictionfree"));
		}

		
		this.factorsList = new LinkedHashMap<Integer, String>();
		for(int i = 0; i<factors.size(); i++){
				this.factorsList.put(factors.get(i).getId(), factors.get(i).getName());
		}
		
		return factors;

	}

	@RequestMapping(value = "/getMetrics", method = RequestMethod.GET, produces = "application/json")
	public @ResponseBody List<Metrics> getMetrics(@RequestParam(value = "factor", required = true) String factor,
			Model model) {

		List<Metrics> metrics = new ArrayList<Metrics>();

		if (factor.equals("0")) {
			metrics.add(new Metrics(0, "Booleano de correctitud semntica"));
			metrics.add(new Metrics(1, "Desviacin de correctitud semntica"));
			metrics.add(new Metrics(2, "Grado de correctitud semntica"));
		}

		if (factor.equals("1")) {
			metrics.add(new Metrics(3, "Booleano de correctitud sintctia"));
			metrics.add(new Metrics(4, "Desviacin de correctitud sintctica"));

		}

		if (factor.equals("2")) {
			metrics.add(new Metrics(5, "Escala"));
			metrics.add(new Metrics(6, "Error estndard"));
			metrics.add(new Metrics(7, "Granularidad"));
		}
		this.metricsList = new LinkedHashMap<Integer, String>();
		// model.addAttribute("metrics", metrics);
		for(int i = 0; i<metrics.size(); i++){
				this.metricsList.put(metrics.get(i).getId(), metrics.get(i).getName());
		}
		return metrics;

	}



	@RequestMapping(value = "/getWsOperations", method = RequestMethod.GET, produces = "application/json")
	public @ResponseBody List<Operation> getWsOperations(@RequestParam(value = "url", required = true) String url,
			Model model) {

		List<Operation> operations = new ArrayList<Operation>();
		
		// Accuracy
		if (url.contains("www")) {
			

			operations.add(new Operation("GetCountryWeather"));
			operations.add(new Operation("GetCityWeather"));
			
			this.wsOperationsList = new LinkedHashMap<Integer, String>();
			for(int i = 0; i<operations.size(); i++){
					this.wsOperationsList.put(operations.get(i).getId(), operations.get(i).getName());
			}
		}
		
		
		return operations;

	}
}

// @RequestMapping("/configurationMethod")
// public ModelAndView configurationMethod() {
//
//// this.methodManager = new SimpleMethodManager();
//// Map<String, Object> myModel = new HashMap<String, Object>();
//// myModel.put("command", this.methodManager.getMethods());
//// ModelAndView model = new ModelAndView("configurationMethod");
//// model.addObject("model", myModel);
//// return model;
//
// return new ModelAndView("configurationMethod","methodForm", new
// Method());
// }
//
// @RequestMapping(value = "/save", method = RequestMethod.POST)
// public ModelAndView save(@ModelAttribute("methodForm")Method method,
// ModelMap model) {
// return new ModelAndView("redirect:/configurationMethodList");
// }
//
//
// @RequestMapping("/configurationMethodList")
// public ModelAndView viewemp(){
// //write the code to get all employees from DAO
// //here, we are writing manual code of list for easy understanding
// List<Method> list=new ArrayList<Method>();
// list.add(new Method(1,"rahul"));
// list.add(new Method(2,"aditya"));
// list.add(new Method(3,"sachin"));
//
// return new ModelAndView("configurationMethodList","list",list);
// }
