package com.project.kiuplus.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import com.kiuplus.configuration.logic.service.DimensionsService;
import com.kiuplus.configuration.logic.service.FactorsService;
import com.kiuplus.configuration.logic.service.MethodsService;
import com.kiuplus.configuration.logic.service.MetricsService;
import com.kiuplus.logic.model.Dimension;
import com.kiuplus.logic.model.Method;
import com.kiuplus.logic.model.Metrics;
import com.project.kiuplus.exceptions.ExceptionMethod;

@Controller
public class ConfigurationController {

	@Autowired
	private DimensionsService dimensionsService;

	@Autowired
	private FactorsService factorsService;

	@Autowired
	private MetricsService metricsService;

	@Autowired
	private MethodsService methodsService;

	@RequestMapping("/configuration")
	public ModelAndView configurationMethod() {
		ModelAndView mv = new ModelAndView("configuration/configuration");

		// Dimensiones para mostrar en el menu lateral
		mv.addObject("dimensions", this.dimensionsService.dimensionsGetAll());

		mv.addObject("dimensionId", -1);
		mv.addObject("factorId", -1);
		mv.addObject("metricsId", -1);
		mv.addObject("methodId", -1);
		return mv;
	}

	@RequestMapping("/configurationMenuSide")
	public ModelAndView menuSideConfiguration(@RequestParam String type, @RequestParam int id) {
		ModelAndView mv = new ModelAndView("configuration/configurationMenuSide");
		try {

			// Dimensiones para mostrar en el menu lateral
			List<Dimension> dimensions = this.dimensionsService.dimensionsGetAll();

			mv.addObject("dimensions", dimensions);
			mv.addObject("type", type);

			// Inicializo las variables
			int dimensionId = -1;
			int factorId = -1;
			int metricsId = -1;
			int methodId = -1;

			switch (type) {
			case "dimension":
				dimensionId = id;
				break;
			case "factor":
				dimensionId = this.factorsService.factorGet(id).getDimension().getId();
				factorId = id;
				break;
			case "metrics":
				Metrics metrics = this.metricsService.metricsGet(id);
				dimensionId = metrics.getFactor().getDimension().getId();
				factorId = metrics.getFactor().getId();
				metricsId = id;
				break;
			case "method":
				Method method = this.methodsService.methodGet(id);
				dimensionId = method.getMetrics().getFactor().getDimension().getId();
				factorId = method.getMetrics().getFactor().getId();
				metricsId = method.getMetrics().getId();
				methodId = method.getId();
				break;
			}

			mv.addObject("dimensionId", dimensionId);
			mv.addObject("factorId", factorId);
			mv.addObject("metricsId", metricsId);
			mv.addObject("methodId", methodId);

		} catch (Exception e) {
			
		}
		return mv;
	}

	/**
	 * Obtiene las dimensiones para mostrarlas en el menu lateral
	 * 
	 * @param model
	 * @return Lista de dimensiones, donde los nombres de dimensiones, factores, metricas y metodos
	 *         tienen largo maximo 20.
	 * 
	 */
	@RequestMapping(value = "/dimensionsMenuGet", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public @ResponseBody List<Dimension> dimensionsMenuGet(Model model) throws Exception {

		try {

			return this.dimensionsService.dimensionsGetAll();

		} catch (Exception e) {
			throw new ExceptionMethod("It is not possible to obtain the dimensions.");
		}
	}

	// Captura las excepciones del tipo ExceptionMethod para enviarlas a la
	// llamada ajax
	@ExceptionHandler({ ExceptionMethod.class })
	public ResponseEntity<String> manageMethodException(Exception e) {
		return new ResponseEntity<String>(e.getMessage(), HttpStatus.NOT_FOUND);
	}

}
