package com.project.kiuplus.controller;

import java.util.ArrayList;
import java.util.List;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.kiuplus.configuration.logic.service.DimensionsService;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.Dimension;

/**
 * @author Beatriz
 *
 */

@Controller
@SessionAttributes("dimension")
public class DimensionsConfigurationController {

	@Autowired
	private DimensionsService dimensionService;

	/**
	 * @return Pagina para listar las dimensiones
	 */
	@RequestMapping(value = "/configurationDimensionsListPage", method = RequestMethod.GET)
	public ModelAndView listDimensionsPage() {

		ModelAndView mav = new ModelAndView("configuration/configurationDimensionsList");

		List<Dimension> listDimensions = new ArrayList<Dimension>();
		listDimensions = dimensionService.dimensionsGetAll();
		mav.addObject("listDimensions", listDimensions);

		return mav;
	}

	/**
	 * @param id
	 *            Dimenson a mostrar
	 * @return Pagina para mostrar la dimension
	 */
	@RequestMapping(value = "/configurationDimensionShowPage", method = RequestMethod.GET)
	public ModelAndView showDimensionPage(@RequestParam int id) {
		ModelAndView mav = new ModelAndView("configuration/configurationDimensionShow");

		Dimension dimension = dimensionService.dimensionFindFullDById(id);
		mav.addObject(dimension);
		mav.addObject("action", "create");
		return mav;
	}

	/**
	 * @return Pagina para crear una dimesion
	 */
	@RequestMapping(value = "/configurationDimensionCreatePage", method = RequestMethod.GET)
	public ModelAndView createDimensionPage() {
		ModelAndView mav = new ModelAndView("configuration/configurationDimensionCreateOrEdit");

		Dimension dimension = new Dimension();
		mav.addObject(dimension);
		mav.addObject("action", "create");
		return mav;
	}

	/**
	 * @param id
	 * @param model
	 * @return Pagina para editar las dimensiones.
	 */
	@RequestMapping(value = "/configurationEditDimensionPage", method = RequestMethod.GET)
	public ModelAndView editDimensionPage(@RequestParam int id, Model model) {
		Dimension dimension = dimensionService.dimensionFindFullDById(id);
		ModelAndView mv = new ModelAndView("configuration/configurationDimensionCreateOrEdit");
		mv.addObject(dimension);
		mv.addObject("action", "edit");
		return mv;
	}

	/**
	 * Crea una nueva entidad dimension o edita la existente dependiendo del id.
	 * 
	 * @param dimension
	 * @param result
	 * @param model
	 * @return Redirecciona a la pagina que lista las dimensiones
	 */
	@RequestMapping(value = "/configurationCreateOrEditDimension", method = RequestMethod.POST)
	public String createOrEditDimension(@ModelAttribute("dimension") @Valid Dimension dimension, BindingResult result,
			Model model, RedirectAttributes redirectAttributes) {

		try {
			if (result.hasErrors()) {
				return "configuration/configurationDimensionCreateOrEdit";
			}

			if (dimension.getId() == 0) {
				/// Agregar dimension
				dimensionService.dimensionCreate(dimension);
				redirectAttributes.addFlashAttribute("message", "Dimension added successfully.");

			} else {
				/// Editar dimension
				dimensionService.dimensionUpdate(dimension);
				redirectAttributes.addFlashAttribute("message", "Dimension successfully modified.");
			}
			return "redirect:configurationDimensionShowPage?id=" + dimension.getId();

		} catch (ExceptionKiuPlus ex) {
			model.addAttribute("error", ex.getMessage());
			return "configuration/configurationDimensionCreateOrEdit";
		}
	}

	/**
	 * Elimina la dimension
	 * 
	 * @param id
	 * @param redirectAttributes
	 * @return Pagina para listar las dimensiones
	 */
	@RequestMapping(value = "/configurationDeleteDimension", method = RequestMethod.GET)
	public String deleteDimension(@RequestParam int id, Model model, RedirectAttributes redirectAttributes) {
		try {

			// TODO: deberia saltar un pop ap al eliminar la dimension de confirmacion
			dimensionService.dimensionDelete(id);
			redirectAttributes.addFlashAttribute("message", "Dimension deleted successfully.");

			return "redirect:configurationDimensionsListPage";

		} catch (ExceptionKiuPlus ex) {
			ex.printStackTrace();
			redirectAttributes.addFlashAttribute("error", "Can not delete the dimension: " + ex.getMessage());

			return "redirect:configurationDimensionShowPage?id=" + id;
		}
	}
}
