package com.project.kiuplus.controller;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.kiuplus.configuration.logic.service.DimensionsService;
import com.kiuplus.configuration.logic.service.FactorsService;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.Dimension;
import com.kiuplus.logic.model.Factor;

/**
 * @author Beatriz
 *
 */

@Controller
@SessionAttributes("factor")
public class FactorsConfigurationController {

	@Autowired
	private FactorsService serviceFactors;

	@Autowired
	private DimensionsService serviceDimensions;

	/**
	 * @param id
	 *            factor a mostrar
	 * @return Pagina para mostrar el factor
	 */
	@RequestMapping(value = "/configurationFactorShowPage", method = RequestMethod.GET)
	public ModelAndView factorShowPage(@RequestParam int id) {
		ModelAndView mav = new ModelAndView("configuration/configurationFactorShow");

		Factor factor = serviceFactors.factorGetFull(id);
		mav.addObject(factor);
		mav.addObject("action", "create");
		return mav;
	}

	/**
	 * @return Pagina para crear un factor
	 */
	@RequestMapping(value = "/configurationFactorCreatePage", method = RequestMethod.GET)
	public ModelAndView factorCreatePage(@RequestParam int idDimension) {
		ModelAndView mav = new ModelAndView("configuration/configurationFactorCreateOrEdit");

		Dimension dimension = this.serviceDimensions.dimensionGet(idDimension);
		Factor factor = new Factor(dimension);
		mav.addObject(factor);
		mav.addObject("action", "create");
		return mav;
	}

	/**
	 * @param id
	 * @param model
	 * @return Pagina para editar los factores.
	 */
	@RequestMapping(value = "/configurationFactorEditPage", method = RequestMethod.GET)
	public ModelAndView factorEditPage(@RequestParam int id, Model model) {
		Factor factor = serviceFactors.factorGet(id);
		ModelAndView mv = new ModelAndView("configuration/configurationFactorCreateOrEdit");
		mv.addObject(factor);
		mv.addObject("action", "edit");
		return mv;
	}

	/**
	 * Crea una nueva entidad factor o edita la existente dependiendo del id.
	 * 
	 * @param factor
	 * @param result
	 * @param model
	 * @return Redirecciona a la pagina que lista los factores
	 */
	@RequestMapping(value = "/configurationFactorCreateOrEdit", method = RequestMethod.POST)
	public String factorCreateOrEdit(@ModelAttribute("factor") @Valid Factor factor, BindingResult result, Model model,
			RedirectAttributes redirectAttributes) {

		try {
			if (result.hasErrors()) {

				return "configuration/configurationFactorCreateOrEdit";

			}

			if (factor.getId() == 0) {
				/// Agregar factor
				serviceFactors.factorCreate(factor);
				redirectAttributes.addFlashAttribute("message", "Factor added successfully.");

			} else {
				/// Editar factor
				serviceFactors.factorUpdate(factor);

				redirectAttributes.addFlashAttribute("message", "Factor successfully modified.");
			}
			return "redirect:configurationFactorShowPage?id=" + factor.getId();

		} catch (ExceptionKiuPlus ex) {
			model.addAttribute("error", ex.getMessage());
			return "configuration/configurationFactorCreateOrEdit";
		}
	}

	/**
	 * Elimina el factor
	 * 
	 * @param id
	 * @param redirectAttributes
	 * @return Pagina para listar los factores
	 */
	@RequestMapping(value = "/configurationFactorDelete", method = RequestMethod.GET)
	public String factorDelete(@RequestParam int id, Model model, RedirectAttributes redirectAttributes) {

		try {

			// TODO: deberia saltar un pop ap al eliminar la factor de
			// confirmacion
			Factor factor = serviceFactors.factorDelete(id);
			redirectAttributes.addFlashAttribute("message", "Factor deleted successfully.");

			return "redirect:configurationDimensionShowPage?id=" + factor.getDimension().getId();

		} catch (ExceptionKiuPlus ex) {
			ex.printStackTrace();
			redirectAttributes.addFlashAttribute("error", "Can not delete the factor: " + ex.getMessage());
			String retorno = "redirect:configurationFactorShowPage?id=" + id;
			return retorno;
		}
	}
}
