package com.project.kiuplus.controller;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpSession;

import org.json.simple.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.kiuplus.configuration.logic.service.MethodsService;
import com.kiuplus.logic.model.Method;
import com.kiuplus.logic.model.Metrics;
import com.kiuplus.logic.model.Operation;
import com.kiuplus.webservices.dataquality.WebServiceService;
import com.project.kiuplus.exceptions.ExceptionMethod;

/**
 * Contiene los metodos que se utilizan en diferentes controladores
 * 
 * @author Beatriz
 *
 */
@Controller
public class MainController {

	@Autowired
	private HttpSession session;

	@Autowired
	private WebServiceService webServiceService;

	@Autowired
	private MethodsService methodsService;

	/**
	 * Valida el web service, obtiene las operaciones y parametros de los web services Se generan
	 * ids para identificar en el controllador cada operacion y parametros.
	 * 
	 * @param url
	 *            del web service
	 * @param model
	 * @return
	 * @throws Exception
	 *             Controla: la url no sea vacia Exista
	 * 
	 */
	@RequestMapping(value = "/urlWebServiceValidate", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public @ResponseBody List<Operation> urlWebServiceValidate(@RequestParam(value = "url", required = true) String url,
			Model model) throws Exception {

		if (url == "") {
			session.setAttribute("listOperations", new ArrayList<Operation>());
			throw new ExceptionMethod("The url of wsdl can not be empty");
		}
		try {

			List<Operation> operations = webServiceService.operationsGet(url);
			session.setAttribute("listOperations", operations);
			return operations;

		} catch (Exception e) {
			String message = e.getMessage() == null || e.getMessage().isEmpty() ? "Unable to process web service"
					: e.getMessage();
			throw new ExceptionMethod(message);
		}
	}

	/**
	 * Obtiene las tablas de la base de datos source y los parametros de la operacion asociada al
	 * metodo
	 * 
	 * @param idDataModel
	 * @param model
	 * @param idMethod
	 * @return Retorna el metodo, la metrica y el web service al cual pertenecen
	 * @throws Exception
	 */
	@SuppressWarnings("unchecked")
	@RequestMapping(value = "/methodGet", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public @ResponseBody JSONObject methodGetAll(@RequestParam(value = "idMethod", required = true) int idMethod, Model model)
			throws Exception {

		try {
			Method method = this.methodsService.methodGet(idMethod);
			
			JSONObject json = new JSONObject();
			json.put("method", method);
			Metrics metrics = method.getMetrics();
			metrics.setMethods(null);
			json.put("metrics", method.getMetrics());
			json.put("webService", method.getOperation().getWebService());
			return json;

		} catch (Exception e) {
			throw new ExceptionMethod("It is not possible to obtain the tables.");
		}
	}

	/**
	 * Obtiene las operaciones de la session y parametros de los web services
	 * 
	 * @param model
	 * @return
	 * 
	 */
	@SuppressWarnings("unchecked")
	@RequestMapping(value = "/operationsWebServiceGet", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public @ResponseBody List<Operation> operationsWebServiceGet(Model model) throws Exception {

		try {
			List<Operation> operations = (ArrayList<Operation>) session.getAttribute("listOperations");
			return operations;
		} catch (Exception e) {
			throw new ExceptionMethod("It is not possible to obtain the parameters.");
		}
	}

	// Captura las excepciones del tipo ExceptionMethod para enviarlas a la
	// llamada ajax
	@ExceptionHandler({ ExceptionMethod.class })
	public ResponseEntity<String> manageMethodException(Exception e) {
		return new ResponseEntity<String>(e.getMessage(), HttpStatus.NOT_FOUND);
	}

}
