package com.project.kiuplus.controller;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.kiuplus.configuration.logic.service.MethodsService;
import com.kiuplus.configuration.logic.service.MetricsService;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.Method;
import com.kiuplus.logic.model.Metrics;
import com.kiuplus.logic.model.Metrics.EnumGranularity;
import com.kiuplus.logic.model.Operation;
import com.kiuplus.logic.model.Parameter;
import com.kiuplus.logic.model.Parameter.EnumDataType;
import com.kiuplus.logic.model.Parameter.EnumParameterType;
import com.kiuplus.logic.model.WebService;
import com.project.kiuplus.exceptions.ExceptionMethod;

/**
 * @author Beatriz
 *
 */

@Controller
@SessionAttributes({ "method" })
public class MethodsConfigurationController {

	@Autowired
	private MethodsService serviceMethods;

	@Autowired
	private MetricsService serviceMetrics;

	@Autowired
	private HttpSession session;

	// Introduce enumParameterTypeList en cada vista
	// No acceder a base de datos desde aqui, ya que enlencen significativamente
	// la aplicacionj
	@ModelAttribute("enumParameterTypeList")
	public EnumParameterType[] enumParameterTypeListLoad() {
		EnumParameterType[] enumParameterTypeList = EnumParameterType.values();
		return enumParameterTypeList;
	}

	/**
	 * @param id
	 *            del metodo a mostrar
	 * @return Pagina para mostrar el metodo
	 */
	@RequestMapping(value = "/configurationMethodShowPage", method = RequestMethod.GET)
	public ModelAndView methodShowPage(@RequestParam int id) {
		ModelAndView mav = new ModelAndView("configuration/configurationMethodShow");

		Method method = this.serviceMethods.methodGet(id);
		mav.addObject(method);
		// mav.addObject("enumParameterType", EnumParameterType.values());
		mav.addObject("webService", method.getOperation().getWebService());
		return mav;
	}

	/**
	 * @return Pagina para crear un metodo
	 */
	@RequestMapping(value = "/configurationMethodCreatePage", method = RequestMethod.GET)
	public ModelAndView methodCreatePage(@RequestParam int idMetric) {
		ModelAndView mav = new ModelAndView("configuration/configurationMethodCreateOrEdit");

		Metrics metrics = this.serviceMetrics.metricsGet(idMetric);
		Method method = new Method(metrics);
		mav.addObject(method);

		mav.addObject("hasMethods", false);
//		mav.addObject("operationCleanWS", new ArrayList<Operation>());
		
		session.setAttribute("listOperations", new ArrayList<Operation>());
	
		return mav;
	}

	/**
	 * @param id
	 * @param model
	 * @return Pagina para editar los metodos.
	 * @throws Exception
	 */
	@ModelAttribute("method")
	@RequestMapping(value = "/configurationMethodEditPage", method = RequestMethod.GET)
	public ModelAndView methodEditPage(@RequestParam int id, Model model) throws Exception {
		Method method = this.serviceMethods.methodGetFull(id);
		ModelAndView mav = new ModelAndView("configuration/configurationMethodCreateOrEdit");
		mav.addObject(method);

		mav.addObject("hasMI", !method.getInstancedMetrics().isEmpty());
		List<Operation> operations = method.getOperation().getWebService().getOperations();

		List<Operation> operationsCopy = this.operationsCopy(operations);
		session.setAttribute("listOperations", operationsCopy);

//		List<Operation> operationCleanWS = operations.stream().filter(o -> o.getParameters().size() == 0).collect(Collectors.toList());;
//		
//		mav.addObject("operationCleanWS", operationCleanWS);
		
		return mav;
	}

	private List<Operation> operationsCopy(List<Operation> operations) {

		List<Operation> operationsCopy = new ArrayList<Operation>();
		if (operations != null) {
			for (Operation op : operations) {
				Operation oCopy = new Operation(op.getId(), op.getName(), op.getDataTypeReturn(), op.getWebService());

				for (Parameter p : op.getParameters()) {
					Parameter pCopy = new Parameter(p.getId(), p.getName(), p.getParameterType(), p.getDataType());
					oCopy.getParameters().add(pCopy);
				}
				operationsCopy.add(oCopy);
			}
		}

		return operationsCopy;
	}

	/**
	 * Crea una nueva entidad metodo o edita la existente dependiendo del id.
	 * 
	 * @param method
	 * @param result
	 * @param model
	 * @return Redirecciona a la pagina que lista los metodos
	 */
	@RequestMapping(value = "/configurationMethodCreateOrEdit", method = RequestMethod.POST)
	public String methodCreateOrEdit(@ModelAttribute("method") @Valid Method method, BindingResult result, Model model,
			RedirectAttributes redirectAttributes) {

		// Creo las operaciones del metodo.
		@SuppressWarnings("unchecked")
		List<Operation> operationsList = ((ArrayList<Operation>) session.getAttribute("listOperations"));

		try {

			if (method.getOperation().getWebService().getUrl() == "") {
				result.rejectValue("operation.webService.url", "blanckValue",
						new Object[] { "'operation.webService.url'" }, "Can not be empty");
			}

			if (method.getOperation().getName() == null || method.getOperation().getName().equals("")) {
				result.rejectValue("operation.name", "blanckValue", new Object[] { "'operation.name'" },
						"Select an operation");

			}

			boolean error = this.parametersValidate(method.getOperation().getParameters(),
					method.getMetrics().getConnectToDB(), result, method, model);

			if (result.hasErrors() || error) {

				this.methodLoadParameters(operationsList, method);
				

//				List<Operation> operationCleanWS = operationsList.stream().filter(o -> o.getParameters().size() == 0).collect(Collectors.toList());;
//				model.addAttribute("operationCleanWS", operationCleanWS);
				
				model.addAttribute("hasMI", !method.getInstancedMetrics().isEmpty());
				return "configuration/configurationMethodCreateOrEdit";

			}

			// if (method.getId() == 0) {
			// /// Actualizar el metodo con lo que selecciono el usuario
			method = this.methodCreate(method, operationsList);
			// } else {
			// /// Editar metodo
			// method = this.methodCreate(method, operationsList);
			// }

			// controla que los tipos de datos de los parametros sean string en caso que sea
			// TABLE_NAME, COLUMN_NAME o CONNECT_TO_DB

			for (Parameter parameter : method.getOperation().getParameters()) {
				if ((parameter.getParameterType().equals(EnumParameterType.COLUMN_NAME)
						|| parameter.getParameterType().equals(EnumParameterType.TABLE_NAME)
						|| parameter.getParameterType().equals(EnumParameterType.CONNECT_TO_DB))
						&& !parameter.getDataType().equals(EnumDataType.STRING)) {
					result.rejectValue("operation.parameters", "blanckValue", new Object[] { "'operation.parameters'" },
							"The column name, table name and connect to DB parameters must be strings");
					this.methodLoadParameters(operationsList, method);
					model.addAttribute("hasMI", !method.getInstancedMetrics().isEmpty());
					return "configuration/configurationMethodCreateOrEdit";
				}
			}

			// Operation clean WS
//			if(!method.getOperationCleanWS().getName().equals("")) {
//				for(Operation o : method.getOperation().getWebService().getOperations()) {
//					if(o.getName().equals(method.getOperationCleanWS().getName())) {
//							method.setOperationCleanWS(o);
//							break;
//					}
//				}
//			}else {
//				method.setOperationCleanWS(null);
//			}
			
			if (method.getId() == 0) {
				this.serviceMethods.methodCreate(method);
				redirectAttributes.addFlashAttribute("message", "Method added successfully.");

			} else {
				this.serviceMethods.methodUpdate(method);
				redirectAttributes.addFlashAttribute("message", "Method successfully modified.");
			}

			return "redirect:configurationMethodShowPage?id=" + method.getId();

		} catch (ExceptionKiuPlus ex) {
			this.methodLoadParameters(operationsList, method);
			
//			List<Operation> operationCleanWS = operationsList.stream().filter(o -> o.getParameters().size() == 0).collect(Collectors.toList());;
//			model.addAttribute("operationCleanWS", operationCleanWS);
			
			model.addAttribute("error", ex.getMessage());
			
			return "configuration/configurationMethodCreateOrEdit";
		}
	}

	private void methodLoadParameters(List<Operation> operations, Method method) {
		List<Operation> operationsList = operationsCopy(operations);
		if (operationsList != null) {
			// Establezco las operaciones del web service que eligio el
			// usuario ya que el metodo solo tien el id.

			for (Operation o : operationsList) {
				if (o.getName().equals(method.getOperation().getName())) {
					// Seteo los tipos de parametros que selecciono el usuario
					for (Parameter p : o.getParameters()) {
						Parameter parameterUser = method.getOperation().getParameter(p.getName());
						p.setParameterType(parameterUser.getParameterType());
					}

					method.setOperation(o);
				}

			}

			method.getOperation().getWebService().setOperations(operationsCopy(operationsList));
		}
	}

	// Crea el metodo con el tipo de parametro que selecciono el usuario en caso
	// que pertenezcan a la operacion del metodo o
	// si no es la que eligio el ususario se crea una operacion con parametros
	// sin tipo
	// private Method methodEdit(Method method, List<Operation> operationsList) throws
	// ExceptionKiuPlus {
	//
	// Method methodOriginal = this.serviceMethods.methodGetFull(method.getId());
	//
	// methodOriginal.setName(method.getName());
	//
	// // Cuando un metodo tiene MI asociadas solo se permite editar el nombre
	// if (methodOriginal.getInstancedMetrics().isEmpty()) {
	// WebService webService = methodOriginal.getOperation().getWebService();
	// webService.setUrl(method.getOperation().getWebService().getUrl());
	//
	// webService.getOperations().removeIf(o ->
	// !o.getName().equals(methodOriginal.getOperation().getName()));
	//
	// for (Operation o : operationsList) {
	//
	// // Si es la operacion que eligio el usuario, le agrego los
	// // parametros
	// if (o.getName().equals(method.getOperation().getName())) {
	//
	// methodOriginal.getOperation().setName(o.getName());
	// methodOriginal.getOperation().setDataTypeReturn(o.getDataTypeReturn());
	//
	// methodOriginal.getOperation().getParameters()
	// .removeAll(methodOriginal.getOperation().getParameters());
	//
	// for (Parameter parameterOriginal : o.getParameters()) {
	// for (Parameter p : method.getOperation().getParameters()) {
	//
	// if (p.getName().equals(parameterOriginal.getName())) {
	//
	// Parameter parameter = new Parameter(0, parameterOriginal.getName(),
	// p.getParameterType(), parameterOriginal.getDataType());
	//
	// methodOriginal.getOperation().getParameters().add(parameter);
	// break;
	// }
	// }
	// }
	// } else {
	//
	// Operation operation = new Operation();
	// operation.setName(o.getName());
	// operation.setDataTypeReturn(o.getDataTypeReturn());
	//
	// webService.getOperations().add(operation);
	// operation.setWebService(webService);
	// for (Parameter parameterOriginal : o.getParameters()) {
	//
	// Parameter parameter = new Parameter(0, parameterOriginal.getName(),
	// parameterOriginal.getParameterType(), parameterOriginal.getDataType());
	//
	// operation.getParameters().add(parameter);
	//
	// }
	// }
	// }
	// }
	// return methodOriginal;
	//
	// }

	// Crea el metodo con el tipo de parametro que selecciono el usuario en caso
	// que pertenezcan a la operacion del metodo o
	// si no es la que eligio el ususario se crea una operacion con parametros
	// sin tipo
	private Method methodCreate(Method method, List<Operation> operationsList) throws ExceptionKiuPlus {

		method.getOperation().getWebService().getOperations().clear();
		WebService webService = new WebService(0, method.getOperation().getWebService().getUrl(), method.getOperation().getWebService().getTargetNamespace());

		for (Operation o : operationsList) {

			Operation operation = new Operation(0, o.getName(), o.getDataTypeReturn());
			// operation.setWebService(method.getOperation().getWebS ervice());
			// Si es la operacion que eligio el usuario, le agrego los
			// parametros
			for (Parameter parameterOriginal : o.getParameters()) {
				if (o.getName().equals(method.getOperation().getName())) {
					for (Parameter p : method.getOperation().getParameters()) {
						if (p.getName().equals(parameterOriginal.getName())) {

							Parameter parameter = new Parameter(0, parameterOriginal.getName(), p.getParameterType(),
									parameterOriginal.getDataType());

							operation.getParameters().add(parameter);
						}
					}

				} else {
					Parameter parameter = new Parameter(0, parameterOriginal.getName(),
							parameterOriginal.getParameterType(), parameterOriginal.getDataType());

					operation.getParameters().add(parameter);

				}
			}

			if (o.getName().equals(method.getOperation().getName())) {
				method.setOperation(operation);

			}
			operation.setWebService(webService);
			webService.getOperations().add(operation);

		}

		return method;

	}

	/**
	 * Valida los parametros
	 * 
	 * @param parameters
	 * @param connectToDB
	 * @param result
	 *            Se retornan los errores especifico de los parametros
	 * @param model
	 * @return Hay error general
	 */
	private boolean parametersValidate(List<Parameter> parameters, boolean connectToDB, BindingResult result,
			Method method, Model model) {
		try {
			for (Parameter parameter : parameters) {
				if (parameter.getParameterType() == null) {
					result.rejectValue("operation.parameters", "nullValue", new Object[] { "'operation.parameters'" },
							"Select the type of all the parameters");
					return true;
				}
			}

			// Verifica que si la granularidad es columna, tabla o BD entonces tiene que haber 1
			// parametro del tipo CONNECT_TO_DB en caso contrario no tiene que haber ninguno de este
			// tipo.

			int connectionParameterCount = (int) parameters.stream()
					.filter(p -> p.getParameterType().equals(EnumParameterType.CONNECT_TO_DB)).count();
			if (connectToDB) {
				if (connectionParameterCount != 1) {
					result.rejectValue("operation.parameters", "nullValue", new Object[] { "'operation.parameters'" },
							"Select a connection parameter");
				}
			} else {
				if (connectionParameterCount != 0) {
					result.rejectValue("operation.parameters", "nullValue", new Object[] { "'operation.parameters'" },
							"Not select parameters of type CONNECT TO DB");
				}
			}

			// Valida que si es del tipo columna, columnSet o table seleccione los parametros de
			// table name y column name
			int paramColumnCount = parameters.stream()
					.filter(p -> p.getParameterType().equals(EnumParameterType.COLUMN_NAME)).toArray().length;
			if (method.getMetrics().getGranularity().equals(EnumGranularity.Column) && paramColumnCount != 1) {
				result.rejectValue("operation.parameters", "nullValue", new Object[] { "'operation.parameters'" },
						"Select a parameter of type column name");
			}

			if (method.getMetrics().getGranularity().equals(EnumGranularity.ColumnSet) && paramColumnCount == 0) {
				result.rejectValue("operation.parameters", "nullValue", new Object[] { "'operation.parameters'" },
						"Select the parameters of type column name");
			}

			int paramTableCount = parameters.stream()
					.filter(p -> p.getParameterType().equals(EnumParameterType.TABLE_NAME)).toArray().length;

			if ((method.getMetrics().getGranularity().equals(EnumGranularity.Table)
					|| method.getMetrics().getGranularity().equals(EnumGranularity.Column)) && paramTableCount != 1) {
				result.rejectValue("operation.parameters", "nullValue", new Object[] { "'operation.parameters'" },
						"Select a parameter of type table name");
			}

			if (method.getMetrics().getGranularity().equals(EnumGranularity.ColumnSet) && paramTableCount == 0) {
				result.rejectValue("operation.parameters", "nullValue", new Object[] { "'operation.parameters'" },
						"Select the parameters of type table name");
			}

			return false;
		} catch (Exception e) {
			e.printStackTrace();
			model.addAttribute("error", e.getMessage());
			return true;
		}
	}

	/**
	 * Elimina el metodo
	 * 
	 * @param id
	 * @param redirectAttributes
	 * @return Pagina para listar los metodos
	 */
	@RequestMapping(value = "/configurationMethodDelete", method = RequestMethod.GET)
	public String methodDelete(@RequestParam int id, Model model, RedirectAttributes redirectAttributes) {

		try {
			Method method = this.serviceMethods.methodDelete(id);
			redirectAttributes.addFlashAttribute("message", "Method deleted successfully.");

			return "redirect:configurationMetricsShowPage?id=" + method.getMetrics().getId();

		} catch (ExceptionKiuPlus ex) {
			ex.printStackTrace();
			redirectAttributes.addFlashAttribute("error", "Can not delete the method: " + ex.getMessage());
			return "redirect:configurationMethodShowPage?id=" + id;

		} catch (Exception e) {
			e.printStackTrace();
			redirectAttributes.addFlashAttribute("error", "Can not delete the method: " + e.getMessage());
			return "redirect:configurationMethodShowPage?id=" + id;
		}
	}

	/**
	 * Obtiene las operaciones de la session y parametros de los web services
	 * 
	 * @param model
	 * @return
	 * 
	 */
	@SuppressWarnings("unchecked")
	@RequestMapping(value = "/operationGet", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public @ResponseBody Operation operationGet(
			@RequestParam(value = "operationName", required = true) String operationName, Model model,
			HttpServletRequest request) throws Exception {

		try {
			List<Operation> operations = (ArrayList<Operation>) session.getAttribute("listOperations");
			Operation operation = (Operation) (operations.stream().filter(o -> o.getName().equals(operationName))
					.toArray())[0];

			Method method = (Method) request.getSession().getAttribute("method");

			method.setOperation(operation);

			request.getSession().setAttribute("method", method);

			return operation;
		} catch (Exception e) {
			throw new ExceptionMethod("It is not possible to obtain the parameters.");
		}
	}

	// Captura las excepciones del tipo ExceptionMethod para enviarlas a la
	// llamada ajax
	@ExceptionHandler({ ExceptionMethod.class })
	public ResponseEntity<String> manageMethodException(Exception e) {
		return new ResponseEntity<String>(e.getMessage(), HttpStatus.NOT_FOUND);
	}

}
