package com.project.kiuplus.controller;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.kiuplus.configuration.logic.service.FactorsService;
import com.kiuplus.configuration.logic.service.MetricsService;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.Factor;
import com.kiuplus.logic.model.Metrics;
import com.kiuplus.logic.model.Metrics.EnumGranularity;

/**
 * @author Beatriz
 *
 */

@Controller
@SessionAttributes("metrics")
public class MetricsConfigurationController {

	@Autowired
	private MetricsService serviceMetrics;

	@Autowired
	private FactorsService serviceFactors;

	/**
	 * @param id
	 *            de metrica a mostrar
	 * @return Pagina para mostrar la metrica
	 */
	@RequestMapping(value = "/configurationMetricsShowPage", method = RequestMethod.GET)
	public ModelAndView metricsShowPage(@RequestParam int id) {
		ModelAndView mav = new ModelAndView("configuration/configurationMetricsShow");

		Metrics metrics = serviceMetrics.metricsGetFull(id);

		mav.addObject(metrics);
		mav.addObject("action", "create");
		return mav;
	}

	/**
	 * @return Pagina para crear una metrica
	 */
	@RequestMapping(value = "/configurationMetricsCreatePage", method = RequestMethod.GET)
	public ModelAndView metricsCreatePage(@RequestParam int idFactor) {
		ModelAndView mav = new ModelAndView("configuration/configurationMetricsCreateOrEdit");

		Factor factor = this.serviceFactors.factorGet(idFactor);
		Metrics metrics = new Metrics(factor);
		mav.addObject(metrics);

		mav.addObject("granularityList", EnumGranularity.values());

		mav.addObject("action", "create");
		return mav;
	}

	/**
	 * @param id
	 * @param model
	 * @return Pagina para editar las metricas.
	 */
	@RequestMapping(value = "/configurationMetricsEditPage", method = RequestMethod.GET)
	public ModelAndView metricsEditPage(@RequestParam int id, Model model) {
		Metrics metrics = serviceMetrics.metricsGetFull(id);

		ModelAndView mv = new ModelAndView("configuration/configurationMetricsCreateOrEdit");

		// Si la metrica tiene metodos no se puede editar la granularidad
		mv.addObject("hasMethods", !metrics.getMethods().isEmpty());
		mv.addObject(metrics);
		mv.addObject("action", "edit");
		mv.addObject("granularityList", EnumGranularity.values());
		return mv;
	}

	/**
	 * Crea una nueva entidad metrica o edita la existente dependiendo del id.
	 * 
	 * @param metrics
	 * @param result
	 * @param model
	 * @return Redirecciona a la pagina que lista los metricses
	 */
	@RequestMapping(value = "/configurationMetricsCreateOrEdit", method = RequestMethod.POST)
	public String metricsCreateOrEdit(@ModelAttribute("metrics") @Valid Metrics metrics, BindingResult result,
			Model model, RedirectAttributes redirectAttributes) {

		try {

			if (metrics.getGranularity().equals(EnumGranularity.Undefined)) {
				result.rejectValue("granularity", "blanckValue", new Object[] { "'granularity'" },
						"Select a granularity");
			}

			if (result.hasErrors()) {
				model.addAttribute("hasMethods", !metrics.getMethods().isEmpty());
				return "configuration/configurationMetricsCreateOrEdit";

			}

			if (metrics.getId() == 0) {
				/// Agregar metrics
				serviceMetrics.metricsCreate(metrics);
				redirectAttributes.addFlashAttribute("message", "Metrics added successfully.");

			} else {
				/// Editar metrics
				serviceMetrics.metricsUpdate(metrics);

				redirectAttributes.addFlashAttribute("message", "Metrics successfully modified.");
			}
			return "redirect:configurationMetricsShowPage?id=" + metrics.getId();

		} catch (ExceptionKiuPlus ex) {

			model.addAttribute("error", ex.getMessage());
			return "configuration/configurationMetricsCreateOrEdit";
		}
	}

	/**
	 * Elimina la metrica
	 * 
	 * @param id
	 * @param redirectAttributes
	 * @return Pagina para listar las metricas
	 */
	@RequestMapping(value = "/configurationMetricsDelete", method = RequestMethod.GET)
	public String metricsDelete(@RequestParam int id, Model model, RedirectAttributes redirectAttributes) {

		try {
			Metrics metrics = serviceMetrics.metricsDelete(id);
			redirectAttributes.addFlashAttribute("message", "Metrics deleted successfully.");

			return "redirect:configurationFactorShowPage?id=" + metrics.getFactor().getId();

		} catch (ExceptionKiuPlus ex) {
			ex.printStackTrace();
			redirectAttributes.addFlashAttribute("error", "Can not delete the metrics: " + ex.getMessage());
			return "redirect:configurationMetricsShowPage?id=" + id;

		}
	}
}
