package com.project.kiuplus.controller.execute;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Future;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import com.eviware.soapui.config.TestOnDemandLocationsRequestDocumentConfig.TestOnDemandLocationsRequest.Request;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.kiuplus.configuration.logic.service.DimensionsService;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.Attribute;
import com.kiuplus.logic.model.AttributeColumn;
import com.kiuplus.logic.model.AttributeTable;
import com.kiuplus.logic.model.DataModel;
import com.kiuplus.logic.model.Dimension;
import com.kiuplus.logic.model.InstancedMetric;
import com.kiuplus.logic.model.ParameterAttribute;
import com.kiuplus.logic.model.ParameterValue;
import com.kiuplus.logic.model.ResultGroup;
import com.kiuplus.logic.model.Attribute.EnumAttributeType;
import com.kiuplus.logic.model.Metrics.EnumGranularity;
import com.kiuplus.logic.model.ResultGroup.EnumResultGroup;
import com.kiuplus.logic.objects.GenericObject;
import com.kiuplus.qualityModel.logic.service.DataModelsService;
import com.kiuplus.qualityModel.logic.service.DatabaseService;
import com.kiuplus.qualityModel.logic.service.ExecuteService;
import com.kiuplus.qualityModel.logic.service.InstancedMetricsService;
import com.kiuplus.qualityModel.logic.service.ResultGroupsService;

@Controller
public class ExecuteAggregationController {

	@Autowired
	private DataModelsService dataModelsService;

	@Autowired
	private ExecuteService executeService;

	@Autowired
	private InstancedMetricsService instancedMetricsService;

	@Autowired
	private ResultGroupsService resultGroupsService;

	@Autowired
	private DimensionsService dimensionsService;

	@Autowired
	private Environment enviroment;

	/**
	 * Retorna la pagina para seleccionar el modelo de calidad de datos
	 * 
	 * @param model
	 * @return
	 */
	@RequestMapping(value = "/executeAggregations", method = RequestMethod.GET)
	public ModelAndView executeAggregations(Model model) {

		ModelAndView mv = new ModelAndView("execute/executeAggregations");
		List<DataModel> dataQualityModels = this.dataModelsService.dataModelsGet();
		mv.addObject("dataQualityModels", dataQualityModels);

		// Listas para guardar los ids de las agregaciones seleccionadas para ejecutar
		mv.addObject("aggregationsIds", new ArrayList<Integer>());

		List<ResultGroup> aggregations = dataQualityModels.get(0).getResultGroups().stream()
				.filter(a -> a.getType().equals(EnumResultGroup.Aggregation)).collect(Collectors.toList());

		mv.addObject("aggregations", aggregations);

		return mv;
	}

	/**
	 * @return Pagina para seleccionar las agregaciones a ejecutar
	 */
	@RequestMapping(value = "/executeAggregationsSelectPage", method = RequestMethod.GET)
	public ModelAndView executeAggregationsSelectPage(@RequestParam int idDataQualityModel, Model model) {

		ModelAndView mv = new ModelAndView("execute/executeAggregationsSelect");
		DataModel dataModel = this.dataModelsService.dataModelGetFull(idDataQualityModel);

		// Listas para guardar los ids de las agregaciones seleccionadas para ejecutar
		mv.addObject("aggregationsIds", new ArrayList<Integer>());

		List<ResultGroup> aggregations = dataModel.getResultGroups().stream()
				.filter(a -> a.getType().equals(EnumResultGroup.Aggregation)).collect(Collectors.toList());

		mv.addObject("aggregations", aggregations);
		mv.addObject("dataModel", dataModel);

		return mv;
	}

	/**
	 * Verifica si hay una ejecucion previa de alguna de las agregaciones y si alguno de los datos
	 * involucrados en la ejecucion ya estan cargados en el sistema
	 * 
	 * @param instancedMetricsIds
	 * @return Json con previusExecution indicando si hay una ejecucion previa dataLoad si los datos
	 *         estan cargados en nuestro sistema
	 * @throws Exception
	 */
//	@RequestMapping(value = "/executeAggregationsValidateAjax", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
//	public @ResponseBody ObjectNode executeAggregationsValidateAjax(
//			@RequestParam(value = "aggregationsIds[]", required = false) Integer[] aggregationsIds) throws Exception {
//		try {
//			ObjectMapper mapper = new ObjectMapper();
//			ObjectNode objectNode1 = mapper.createObjectNode();
//
//			List<ResultGroup> aggregations = new ArrayList<>();
//			if (aggregationsIds != null) {
//				aggregations = this.resultGroupsService.resultGroupsGet(Arrays.asList(aggregationsIds));
//			}
//
//			// Verifica si hay una ejecucion previa de alguna agregacion
//			boolean previusExecution = false;
//
//			for (ResultGroup a : aggregations) {
//				if (a.getDateTimeResult() != null) {
//					previusExecution = true;
//					break;
//				}
//			}
//
//			objectNode1.put("previusExecution", previusExecution);
//
//			return objectNode1;
//		} catch (Exception e) {
//			e.printStackTrace();
//			throw e;
//		}
//	}

	/**
	 * Ejecuta las agregaciones
	 * 
	 * @param aggregationsIds
	 * @param instancedMetricsIds
	 * @param reloadData
	 *            Los datos se cargan de la base de datos fuentes
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/executeAggregationsAjax", method = RequestMethod.GET)
	public ModelAndView executeAggregationsAjax(
			@RequestParam(value = "aggregationsIds[]", required = false) Integer[] aggregationsIds,
			@RequestParam(value = "idDataQualityModel", required = true) int idDataQualityModel,
			@RequestParam(value = "deleteIMResults", required = true) boolean deleteIMResults) {
		
		ModelAndView mv = new ModelAndView("execute/executeAggregationsResultShow");
		try {

			
			if (aggregationsIds == null || aggregationsIds.length == 0) {
				throw new ExceptionKiuPlus("Select the aggregations to be evaluated");
			}

			// Obtiene las agregaciones identificadas con el id
			List<ResultGroup> aggregations = new ArrayList<>();

			if (aggregationsIds != null) {
				aggregations = this.resultGroupsService.resultGroupsGet(Arrays.asList(aggregationsIds));
			}

			// Ejecuta las agregaciones
			Future<Map<Integer, String>> resultExecute = this.executeService.executeAggregations(aggregations, deleteIMResults);
			
			// Espera que termine de ejecutarse
			while (true) {
				if (resultExecute.isDone()) {
					break;
				}
			}

			// Obtiene los resultados de la ejecucion
			Map<ResultGroup, List<GenericObject>> resultAggExecute = new HashMap<ResultGroup, List<GenericObject>>();
			Map<Integer, String> errorAggMap = resultExecute.get();
			List<String> errorAggList = new ArrayList<String>();
			if (aggregations != null) {
				for (ResultGroup agg : aggregations) {
					if (!errorAggMap.containsKey(agg.getId())){
						resultAggExecute.put(agg, this.executeService.resultExecutionQuery(agg));
					}
					else{
						errorAggList.add("Can not execute the aggregation <strong>" + agg.getName() + "</strong>: " + errorAggMap.get(agg.getId()));
					}
				}
			}
			mv.addObject("resultAggExecute", resultAggExecute);
			mv.addObject("dataModel", this.dataModelsService.dataModelGet(idDataQualityModel));
			mv.addObject("errorAggList", errorAggList);
			return mv;
		} catch (Exception e) {
			mv.addObject("error", e.getMessage());
			return mv;
		}
	}

}
