package com.project.kiuplus.controller.execute;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Future;
import java.util.stream.Collectors;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;


import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.DataModel;
import com.kiuplus.logic.model.ResultGroup;
import com.kiuplus.logic.model.ResultGroup.EnumResultGroup;
import com.kiuplus.logic.objects.GenericObject;
import com.kiuplus.qualityModel.logic.service.DataModelsService;
import com.kiuplus.qualityModel.logic.service.ExecuteService;
import com.kiuplus.qualityModel.logic.service.ResultGroupsService;

@Controller
public class ExecuteComposedIMController {

	@Autowired
	private DataModelsService dataModelsService;

	@Autowired
	private ExecuteService executeService;

	@Autowired
	private ResultGroupsService resultGroupsService;


	/**
	 * Retorna la pagina para seleccionar el modelo de calidad de datos
	 * 
	 * @param model
	 * @return
	 */
	@RequestMapping(value = "/executeComposedIM", method = RequestMethod.GET)
	public ModelAndView executeComposedIM(Model model) {

		ModelAndView mv = new ModelAndView("execute/executeComposedIM");
		List<DataModel> dataQualityModels = this.dataModelsService.dataModelsGet();
		mv.addObject("dataQualityModels", dataQualityModels);

		return mv;
	}

	/**
	 * @return Pagina para seleccionar las composed IM a ejecutar
	 */
	@RequestMapping(value = "/executeComposedIMSelectPage", method = RequestMethod.GET)
	public ModelAndView executeComposedIMSelectPage(@RequestParam int idDataQualityModel, Model model) {

		ModelAndView mv = new ModelAndView("execute/executeComposedIMSelect");
		DataModel dataModel = this.dataModelsService.dataModelGetFull(idDataQualityModel);

		// Listas para guardar los ids de las Composed IM seleccionadas para ejecutar
		mv.addObject("composedIMIds", new ArrayList<Integer>());

		List<ResultGroup> composedIMs = dataModel.getResultGroups().stream()
				.filter(a -> a.getType().equals(EnumResultGroup.ComposedInstancedMetric)).collect(Collectors.toList());

		mv.addObject("composedIMs", composedIMs);
		mv.addObject("dataModel", dataModel);

		return mv;
	}

	/**
	 * Verifica si hay una ejecucion previa de alguna de las agregaciones y si alguno de los datos
	 * involucrados en la ejecucion ya estan cargados en el sistema
	 * 
	 * @param instancedMetricsIds
	 * @return Json con previusExecution indicando si hay una ejecucion previa dataLoad si los datos
	 *         estan cargados en nuestro sistema
	 * @throws Exception
	 */
//	@RequestMapping(value = "/executeAggregationsValidateAjax", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
//	public @ResponseBody ObjectNode executeAggregationsValidateAjax(
//			@RequestParam(value = "aggregationsIds[]", required = false) Integer[] aggregationsIds) throws Exception {
//		try {
//			ObjectMapper mapper = new ObjectMapper();
//			ObjectNode objectNode1 = mapper.createObjectNode();
//
//			List<ResultGroup> aggregations = new ArrayList<>();
//			if (aggregationsIds != null) {
//				aggregations = this.resultGroupsService.resultGroupsGet(Arrays.asList(aggregationsIds));
//			}
//
//			// Verifica si hay una ejecucion previa de alguna agregacion
//			boolean previusExecution = false;
//
//			for (ResultGroup a : aggregations) {
//				if (a.getDateTimeResult() != null) {
//					previusExecution = true;
//					break;
//				}
//			}
//
//			objectNode1.put("previusExecution", previusExecution);
//
//			return objectNode1;
//		} catch (Exception e) {
//			e.printStackTrace();
//			throw e;
//		}
//	}

	/**
	 * Ejecuta las composed IM
	 * 
	 * @param composedIMIds
	 * @param idDataQualityModel
	 * @param deleteIMResults
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/executeComposedIMAjax", method = RequestMethod.GET)
	public ModelAndView executeComposedIMAjax(
			@RequestParam(value = "composedIMIds[]", required = false) Integer[] composedIMIds,
			@RequestParam(value = "idDataQualityModel", required = true) int idDataQualityModel,
			@RequestParam(value = "deleteIMResults", required = true) boolean deleteIMResults) {
		
		ModelAndView mv = new ModelAndView("execute/executeComposedIMResultShow");
		try {			
			if (composedIMIds == null || composedIMIds.length == 0) {
				throw new ExceptionKiuPlus("Select the composed instanced metrics to be evaluated");
			}

			// Obtiene los composed IM identificadas con el id
			List<ResultGroup> composedIMs = new ArrayList<>();

			if (composedIMIds != null) {
				composedIMs = this.resultGroupsService.resultGroupsGet(Arrays.asList(composedIMIds));
			}

			// Ejecuta las composed IM
			Future<Map<Integer, String>> resultExecute = this.executeService.executeComposedIMs(composedIMs, deleteIMResults);
			
			// Espera que termine de ejecutarse
			while (true) {
				if (resultExecute.isDone()) {
					break;
				}
			}

			// Obtiene los resultados de la ejecucion
			Map<ResultGroup, List<GenericObject>> resultComposedIMExecute = new HashMap<ResultGroup, List<GenericObject>>();
			Map<Integer, String> errorComposedIMMap = resultExecute.get();
			List<String> errorComposedIMList = new ArrayList<String>();
			if (composedIMs != null) {
				for (ResultGroup composedIM : composedIMs) {
					if (!errorComposedIMMap.containsKey(composedIM.getId())){
						//TODO: resultComposedIMExecute.put(composedIM, this.executeService.resultExecutionQuery(composedIM));
						resultComposedIMExecute.put(composedIM, new ArrayList());
					}
					else{
						errorComposedIMList.add("Can not execute the composed instanced metrics <strong>" + composedIM.getName() + "</strong>: " + errorComposedIMList.get(composedIM.getId()));
					}
				}
			}
			mv.addObject("resultComposedIMExecute", resultComposedIMExecute);
			mv.addObject("dataModel", this.dataModelsService.dataModelGet(idDataQualityModel));
			mv.addObject("errorAggList", errorComposedIMList);
			return mv;
		} catch (Exception e) {
			mv.addObject("error", e.getMessage());
			return mv;
		}
	}

}
