package com.project.kiuplus.controller.execute;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.kiuplus.configuration.logic.service.DimensionsService;
import com.kiuplus.logic.model.DataModel;
import com.kiuplus.logic.model.Dimension;
import com.kiuplus.logic.model.InstancedMetric;
import com.kiuplus.logic.model.ParameterAttribute;
import com.kiuplus.logic.model.ParameterValue;
import com.kiuplus.logic.model.ResultGroup;
import com.kiuplus.logic.model.ResultGroup.EnumResultGroup;
//import com.kiuplus.logic.model.Aggregation.EnumAggregationType;
import com.kiuplus.logic.model.Attribute;
import com.kiuplus.logic.model.Attribute.EnumAttributeType;
import com.kiuplus.logic.model.Metrics.EnumGranularity;
import com.kiuplus.logic.model.AttributeColumn;
import com.kiuplus.logic.model.AttributeTable;
import com.kiuplus.logic.objects.*;
import com.kiuplus.qualityModel.logic.service.ResultGroupsService;
import com.kiuplus.qualityModel.logic.service.DataModelsService;
import com.kiuplus.qualityModel.logic.service.DatabaseService;
import com.kiuplus.qualityModel.logic.service.ExecuteService;
import com.kiuplus.qualityModel.logic.service.InstancedMetricsService;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@Controller
public class ExecuteController {

	@Autowired
	private DataModelsService dataModelsService;

	@Autowired
	private ExecuteService executeService;

	@Autowired
	private DatabaseService databaseService;

	@Autowired
	private InstancedMetricsService instancedMetricsService;

	@Autowired
	private ResultGroupsService resultGroupsService;

	@Autowired
	private DimensionsService dimensionsService;

	@Autowired
	private Environment enviroment;

	/**
	 * Retorna la pagina para seleccionar el modelo de calidad de datos y el tipo de ejecucion
	 * 
	 * @param model
	 * @return
	 */
	@RequestMapping(value = "/dataQualityModelExecutePpalPage", method = RequestMethod.GET)
	public ModelAndView dataQualityModelExecutePpalPage(Model model) {

		ModelAndView mv = new ModelAndView("execute/executeDataQualityModel");
		List<DataModel> dataQualityModels = this.dataModelsService.dataModelsGet();
		mv.addObject("dataQualityModels", dataQualityModels);
		return mv;
	}

	/**
	 * @return Pagina para seleccionar las metricas instanciadas y las agregaciones a ejecutar
	 */
	@RequestMapping(value = "/dataQualityModelExecutePage", method = RequestMethod.GET)
	public ModelAndView dataQualityModelExecutePage(@RequestParam int idDataQualityModel, @RequestParam String type,
			Model model) {

		ModelAndView mv = new ModelAndView("execute/executeDataQualityModelSelectElemets");
		DataModel dataModel = this.dataModelsService.dataModelGetFull(idDataQualityModel);

		// Listas para guardar los ids de las metricas instanciadas y agregaciones seleccionadas
		// para ejecutar
		mv.addObject("aggregationsIds", new ArrayList<Integer>());
		mv.addObject("instancedMetricsIds", new ArrayList<Integer>());

		List<ResultGroup> aggregations = new ArrayList<>();
		if (type.equals("IMPartialAgreg")) {
			// Lista de metricas instanciadas y agregaciones para que el usuario las seleccione
			mv.addObject("instancedMetrics", dataModel.getInstancedMetrics());

			aggregations = dataModel.getResultGroups().stream()
					.filter(a -> a.getType().equals(EnumResultGroup.Aggregation)).collect(Collectors.toList());
		} else {
			mv.addObject("instancedMetrics", new ArrayList<InstancedMetric>());

			aggregations = dataModel.getResultGroups().stream()
					.filter(a -> !a.getType().equals(EnumResultGroup.ComposedInstancedMetric))
					.collect(Collectors.toList());
		}

		mv.addObject("aggregations", aggregations);
		mv.addObject("dataModel", dataModel);
		mv.addObject("type", type);

		ObjectMapper mapper = new ObjectMapper();
		try {
			List<Dimension> dimensions = this.dimensionsService.dimensionsDataQualityModelGet(dataModel.getId());
			ObjectNode objectNode1 = mapper.createObjectNode();
			String jsonInString = mapper.writeValueAsString(dimensions);
			objectNode1.put("dimensions", jsonInString);

			mv.addObject("dimensions", objectNode1);
		} catch (JsonProcessingException e) {
			e.printStackTrace();
			mv.addObject("error", e.getMessage());

			return mv;
		}
		return mv;
	}

	/**
	 * 
	 * 
	 * 
	 * /** Verifica si hay una ejecucion previa de alguna de las metricas instanciadas o de las
	 * agrgegaciones y si alguno de los datos involucrados en la ejecucion ya estan cargados en el
	 * sistema
	 * 
	 * @param aggregationsIds
	 * @param instancedMetricsIds
	 * @return Json con previusExecution indicando si hay una ejecucion previa dataLoad si los datos
	 *         estan cargados en nuestro sistema
	 * @throws Exception
	 */
	@RequestMapping(value = "/qualityModelDataModelValidateExecuteAjax", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public @ResponseBody ObjectNode qualityModelDataModelValidateExecuteAjax(
			@RequestParam(value = "aggregationsIds[]", required = false) Integer[] aggregationsIds,
			@RequestParam(value = "instancedMetricsIds[]", required = false) Integer[] instancedMetricsIds)
			throws Exception {
		try {
			ObjectMapper mapper = new ObjectMapper();
			ObjectNode objectNode1 = mapper.createObjectNode();

			// Obtiene las agregaciones y metricas instanciadas
			List<ResultGroup> aggregations = new ArrayList<>();
			if (aggregationsIds != null) {
				aggregations = this.resultGroupsService.resultGroupsGet(Arrays.asList(aggregationsIds));
			}

			List<InstancedMetric> instancedMetrics = new ArrayList<>();
			if (instancedMetricsIds != null) {
				instancedMetrics = this.instancedMetricsService.instancedMetricsGet(Arrays.asList(instancedMetricsIds));
			}

			// Verifica si hay una ejecucion previa de alguna metrica instanciada o agregacion
			boolean previusExecution = false;

			for (ResultGroup a : aggregations) {
				if (a.getDateTimeResult() != null) {
					previusExecution = true;
					break;
				}
			}

			for (InstancedMetric im : instancedMetrics) {
				if (im.getDateTimeResult() != null) {
					previusExecution = true;
					break;
				}
			}

			objectNode1.put("previusExecution", previusExecution);
			// Verifica si hay datos cargados que involucren alguna de las metricas instanciadas del
			// modelo de datos.
			objectNode1.put("dataLoad", instancedMetricsAttributeDataLoaded(instancedMetrics).size() > 0);

			return objectNode1;
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	/**
	 * Verifica que otra metrica instanciada ejecutada del mismo modelo de datos no utilice los
	 * datos de las metricas instanciadas a ejecutar. Es decir, al recargar los datos se modifiquen
	 * los datos utilizados por las metrias instanciadas ejecutadas.
	 * 
	 * @param instancedMetrics
	 *            Metricas instanciadas que utilizan los datos de las que se van a ejecutar
	 * @return
	 */
	private List<InstancedMetric> instancedMetricsAttributeDataLoaded(List<InstancedMetric> instancedMetrics) {

		List<InstancedMetric> instancedMetricsExecuted = this.instancedMetricsService
				.instancedMetricsGet(instancedMetrics.get(0).getDataModel().getId());

		// Elimino las no ejecutadas
		instancedMetricsExecuted.removeIf(i -> i.getDateTimeResult() == null);

		// Elimino las que se estan por ejecutar, ya que estas ejecuciones se eliminaran
		// obligatoriamente
		for (InstancedMetric im : instancedMetrics)
			instancedMetricsExecuted.removeIf(i -> i.getId() == im.getId());

		// Lista de metricas instanciadas que tienen los datos cargados
		List<InstancedMetric> instancedMetricsDataLoaded = new ArrayList<InstancedMetric>();

		// Recorro las metricas instanciadas ejecutadas
		for (InstancedMetric imExecuted : instancedMetricsExecuted) {
			for (ParameterValue pvExecuted : imExecuted.getParameterValues()) {
				if (pvExecuted.getClass().equals(ParameterAttribute.class)) {
					Attribute aExecuted = ((ParameterAttribute) pvExecuted).getAttribute();

					// Si la granularidad de alguna de las MI ejecutadas es base de datos entonces
					// se va a eliminar la ejecucion
					if (imExecuted.getMethod().getMetrics().getGranularity().equals(EnumGranularity.Database)) {
						if (!instancedMetricsDataLoaded.contains(imExecuted)) {
							instancedMetricsDataLoaded.add(imExecuted);
						}
					} else {
						// Recorro las metricas instanciadas a ejecutar
						for (InstancedMetric imToExecute : instancedMetrics) {
							for (ParameterValue pvToE : imToExecute.getParameterValues()) {
								if (pvToE.getClass().equals(ParameterAttribute.class)) {
									Attribute aToE = ((ParameterAttribute) pvToE).getAttribute();
									switch (aToE.getTypeAttribute()) {
									case DATABASE:
										// MI a ejecutar es base de datos, se elimina la ejecucion
										// de todas las MI ejecutadas
										if (imToExecute.getMethod().getMetrics().getGranularity()
												.equals(EnumGranularity.Database)) {
											for (InstancedMetric im : instancedMetricsExecuted) {
												if (!instancedMetricsDataLoaded.contains(im))
													instancedMetricsDataLoaded.add(im);
											}
											return instancedMetricsDataLoaded;
										}
										break;
									case TABLES:
										AttributeTable aToETable = (AttributeTable) aToE;

										// La metrica a ejecutar utiliza la misma tabla que la
										// ejecutada
										if (aExecuted.getTypeAttribute().equals(EnumAttributeType.TABLES)) {
											AttributeTable aExecutedTable = (AttributeTable) aExecuted;
											if (aToETable.getId().equals(aExecutedTable.getId())) {
												if (!instancedMetricsDataLoaded.contains(imExecuted)) {
													instancedMetricsDataLoaded.add(imExecuted);
												}
											}
										}

										// La metrica instanciada a ejecutar utiliza la tabla de la
										// columna de una MI ejecutada
										if (aExecuted.getTypeAttribute().equals(EnumAttributeType.COLUMNS)) {
											AttributeColumn aExecutedColumn = (AttributeColumn) aExecuted;
											if (aToETable.getId().equals(aExecutedColumn.getAttributeTable().getId())) {
												if (!instancedMetricsDataLoaded.contains(imExecuted)) {
													instancedMetricsDataLoaded.add(imExecuted);
												}
											}
										}
										break;
									case COLUMNS:
										AttributeColumn aToEColumn = (AttributeColumn) aToE;

										// La metrica a ejecutar utiliza una columna de la tabla de
										// la ejecutada
										if (aExecuted.getTypeAttribute().equals(EnumAttributeType.TABLES)) {
											AttributeTable aExecutedTable = (AttributeTable) aExecuted;
											if (aToEColumn.getAttributeTable().getId().equals(aExecutedTable.getId())) {
												if (!instancedMetricsDataLoaded.contains(imExecuted)) {
													instancedMetricsDataLoaded.add(imExecuted);
												}
											}
										}

										// La metrica instanciada a ejecutar utiliza una columna de
										// la tabla a la cual pertenece la columna de la mi
										// ejecutada
										if (aExecuted.getTypeAttribute().equals(EnumAttributeType.COLUMNS)) {
											AttributeColumn aExecutedColumn = (AttributeColumn) aExecuted;
											if (aToEColumn.getAttributeTable().getId()
													.equals(aExecutedColumn.getAttributeTable().getId())) {
												if (!instancedMetricsDataLoaded.contains(imExecuted)) {
													instancedMetricsDataLoaded.add(imExecuted);
												}
											}
										}
										break;
									case UNDEFINED:
										break;

									}
								}
							}
						}
					}
				}
			}
		}

		return instancedMetricsDataLoaded;
	}

	/**
	 * Ejecuta las agregaciones y metricas instanciadas indicadas
	 * 
	 * @param aggregationsIds
	 * @param instancedMetricsIds
	 * @param reloadData
	 *            Los datos se cargan de la base de datos fuentes
	 * @return
	 * @throws Exception
	 */
	// @RequestMapping(value = "/qualityModelDataModelExecuteAjax", method = RequestMethod.GET)
	// public ModelAndView qualityModelDataModelExecuteAjax(
	// @RequestParam(value = "aggregationsIds[]", required = false) Integer[] aggregationsIds,
	// @RequestParam(value = "instancedMetricsIds[]", required = false) Integer[]
	// instancedMetricsIds,
	// @RequestParam(value = "reloadData", required = true) boolean reloadData) throws Exception {
	// try {
	//
	// ModelAndView mv = new ModelAndView("execute/executeDataQualityModelResultShow");
	// if ((aggregationsIds == null || aggregationsIds.length == 0)
	// && (instancedMetricsIds == null || instancedMetricsIds.length == 0)) {
	// throw new ExceptionKiuPlus("Select the instantiated metrics and aggregations to be
	// evaluated");
	// }
	//
	// // Obtiene las metricas instanciadas identificadas con el id
	// List<InstancedMetric> instancedMetrics = new ArrayList<>();
	//
	// if (instancedMetricsIds != null) {
	// instancedMetrics =
	// this.instancedMetricsService.instancedMetricsGet(Arrays.asList(instancedMetricsIds));
	// }
	// // Obtiene las agregaciones identificadas con el id
	// List<ResultGroup> aggregations = new ArrayList<>();
	// if (aggregationsIds != null) {
	// this.resultGroupsService.resultGroupsGet(Arrays.asList(aggregationsIds));
	// }
	//
	// if (reloadData) {
	// // Eliminar ejecuciones de las metricas instanciadas en la base de metadatos
	// List<InstancedMetric> instancedMetricsToRemove = this
	// .instancedMetricsAttributeDataLoaded(instancedMetrics);
	// for (InstancedMetric im : instancedMetricsToRemove) {
	// // im.setDateTimeResult(null);
	// // im.setMetadataTableResult(null);
	// // this.instancedMetricsService.instancedMetricEdit(im);
	// }
	// }
	//
	// // Ejecuta las metricas intanciadas y las agregaciones
	// Future<Boolean> resultExecute = this.executeService.executeDataQaulityModel(instancedMetrics,
	// aggregations,
	// reloadData);
	//
	// // Espera que termine de ejecutarse
	// while (true) {
	// if (resultExecute.isDone()) {
	// break;
	// }
	//
	// // Si aun no esta el resultado se duerme 1 segundo
	// Thread.sleep(1000);
	//
	// }
	//
	// // Cargo los resultados
	//
	// // Obtiene los resultados de la ejecucion
	// Map<InstancedMetric, List<GenericObject>> resultIMExecute = new HashMap<InstancedMetric,
	// List<GenericObject>>();
	// if (instancedMetrics != null) {
	// for (InstancedMetric im : instancedMetrics) {
	//
	//// resultIMExecute.put(im,
	// /* this.executeService.resultExecutionQuery(im) */
	//// this.executeService.resultExecutionQuery(im.getDataModel().getDatabaseMetadata(),
	//// im.getMetadataTableResult()));
	// resultIMExecute.put(im, this.executeService.resultExecutionQuery(im));
	// }
	// }
	//
	// Map<ResultGroup, List<GenericObject>> resultAggExecute = new HashMap<ResultGroup,
	// List<GenericObject>>();
	// if (aggregations != null) {
	// for (ResultGroup agg : aggregations) {
	//
	// // TODO:
	// // resultAggExecute.put(agg, this.databaseService.resultExecutionQuery(
	// // agg.getDataModel().getDatabaseMetadata(), agg.getMetadataTableResult()));
	// }
	// }
	// mv.addObject("resultIMExecute", resultIMExecute);
	// mv.addObject("resultAggExecute", resultAggExecute);
	//
	// return mv;
	// } catch (Exception e) {
	// e.printStackTrace();
	// throw new ExceptionKiuPlus(e.getMessage());
	// }
	// }

	/**
	 * Retorna la pagina para seleccionar el modelo de calidad de datos para mostrar el resultado
	 * 
	 * @param model
	 * @return
	 */
	@RequestMapping(value = "/executeDataQualityModelResultPage", method = RequestMethod.GET)
	public ModelAndView executeDataQualityModelResultPage(Model model) {

		ModelAndView mv = new ModelAndView("execute/executeDataQualityModelResult");
		List<DataModel> dataQualityModels = this.dataModelsService.dataModelsGet();
		mv.addObject("dataQualityModels", dataQualityModels);
		return mv;
	}

	/**
	 * Obtiene el resultado de todas las ejecuciones de modelo de datos
	 * 
	 * @param idDataModel
	 *            identificador del modelo de datos
	 * @param model
	 * @return
	 */
	@RequestMapping(value = "/executeDataQulityModelResultAjax", method = RequestMethod.GET)
	public ModelAndView executeDataQulityModelResultAjax(@RequestParam int idDataModel, Model model) {
		ModelAndView mv = new ModelAndView("execute/executeDataQualityModelResultShow");
		try {
			DataModel dataModel = this.dataModelsService.dataModelGetFull(idDataModel);

			// resultado
			Map<InstancedMetric, List<GenericObject>> resultIM = new HashMap<InstancedMetric, List<GenericObject>>();

			for (InstancedMetric instancedMetric : dataModel.getInstancedMetrics()) {
				resultIM.put(instancedMetric, this.executeService.resultExecutionQuery(instancedMetric));
			}

			Map<ResultGroup, List<GenericObject>> resultAgg = new HashMap<ResultGroup, List<GenericObject>>();
			for (ResultGroup aggregation : dataModel.getResultGroups()) {
				if (aggregation.getType().equals(EnumResultGroup.Aggregation)) {
					resultAgg.put(aggregation, this.executeService.resultExecutionQuery(aggregation));
				}
			}

			Map<ResultGroup, List<GenericObject>> composedIM = new HashMap<ResultGroup, List<GenericObject>>();
			for (ResultGroup composed : dataModel.getResultGroups()) {
				if (composed.getType().equals(EnumResultGroup.ComposedInstancedMetric)) {
					composedIM.put(composed, this.executeService.resultExecutionQuery(composed));
				}
			}

			model.addAttribute("resultIMExecute", resultIM);
			model.addAttribute("resultAggExecute", resultAgg);
			model.addAttribute("resultComposedIMExecute", composedIM);

			model.addAttribute("dataModel", dataModel);

		} catch (Exception e) {
			e.printStackTrace();
			model.addAttribute("error", e.getMessage());
		}
		return mv;
	}

	// *******************************************//
	// ** Exportar a PDF
	// *******************************************//
	/**
	 * Exporta el resultado de la ejecucion de los modelos de datos a PDF
	 * 
	 * @param id
	 * @param download
	 * @param request
	 * @param response
	 */
	@RequestMapping(value = "/qualityModelDataModelResultExportPDF", method = RequestMethod.GET)
	public void qualityModelDataModelResultExportPDF(@RequestParam int id, @RequestParam boolean download,
			HttpServletRequest request, HttpServletResponse response) {

		try {
			DataModel dataModel = this.dataModelsService.dataModelGetFull(id);

			Map<InstancedMetric, List<GenericObject>> instencedMetricsResult = new HashMap<InstancedMetric, List<GenericObject>>();
			for (InstancedMetric instancedMetric : dataModel.getInstancedMetrics()) {
				instencedMetricsResult.put(instancedMetric, this.executeService.resultExecutionQuery(instancedMetric));

			}

			Map<ResultGroup, List<GenericObject>> aggregationsResult = new HashMap<ResultGroup, List<GenericObject>>();
			for (ResultGroup aggregation : dataModel.getResultGroups()) {
				aggregationsResult.put(aggregation, this.executeService.resultExecutionQuery(aggregation));

			}
			String path = enviroment.getProperty("images.source");

			ByteArrayOutputStream baos = this.executeService.qualityModelDataModelResultExportPDF(dataModel,
					instencedMetricsResult, aggregationsResult, path);

			// DocFlavor docFormat = DocFlavor.BYTE_ARRAY.AUTOSENSE;
			// Doc document = new SimpleDoc(baos.toByteArray(), docFormat, null);

			// PrintRequestAttributeSet attributeSet = new HashPrintRequestAttributeSet();
			//
			// PrintService defaultPrintService = PrintServiceLookup.lookupPrintServices();
			//
			//
			// if (defaultPrintService != null) {
			// DocPrintJob printJob = defaultPrintService.createPrintJob();
			// try {
			// printJob.print(document, attributeSet);
			//
			// } catch (Exception ex) {
			// ex.printStackTrace();
			// }
			// } else {
			// System.err.println("No existen impresoras instaladas");
			// }

			// Establece los cabezales de la respuesta
			response.setHeader("Expires", "0");
			response.setHeader("Cache-Control", "must-revalidate, post-check=0, pre-check=0");
			String fileName = "Data model " + dataModel.getName() + " result of execute.pdf";
			response.setHeader("Pragma", "public");

			if (download) {
				response.setHeader("Content-disposition", "attachment; filename=" + fileName);
			} else {
				response.setHeader("Content-disposition", "inline; filename=" + fileName);
			}

			response.setContentType("application/pdf");

			response.setContentLength(((ByteArrayOutputStream) baos).size());

			OutputStream os = response.getOutputStream();

			baos.writeTo(os);
			os.flush();
			os.close();

		} catch (Exception e) {
			e.printStackTrace();

		}
	}

	/*
	 * @return Pagina para seleccionar las agregaciones de resultado y de granularidad a ejecutar
	 */
	// @SuppressWarnings("unchecked")
	// @RequestMapping(value = "/qualityModelDataModelExecuteGranulartyMeasurementAgregPage", method
	// = RequestMethod.GET)
	// public ModelAndView qualityModelDataModelExecuteGranulartyMeasurementAgregPage(@RequestParam
	// int id, Model model) {
	//
	// ModelAndView mv = new ModelAndView("qualityModel/qualityModelDataModelExecute");
	//
	// DataModel dataModel = this.dataModelsService.dataModelGetFull(id);
	//
	// // Listas para guardar los ids de las metricas instanciadas y agregaciones seleccionadas
	// // para ejecutar
	// mv.addObject("aggregationsIds", new ArrayList<Integer>());
	// mv.addObject("instancedMetricsIds", new ArrayList<Integer>());
	//
	// // Lista de metricas instanciadas y agregaciones para que el usuario las seleccione
	// mv.addObject("instancedMetrics", new ArrayList<InstancedMetric>());
	//
	// List<Aggregation> aggregations = dataModel.getAggregations().stream()
	// .filter(a -> a.getType().equals(EnumAggregationType.Measurement)
	// || a.getType().equals(EnumAggregationType.Granularity))
	// .collect(Collectors.toList());
	//
	// mv.addObject("aggregations", aggregations);
	// mv.addObject("dataModel", dataModel);
	//
	// mv.addObject("type", "GranulartyMeasurementAgreg");
	//
	// return mv;
	// }

	// @RequestMapping(value = "/qualityModelDataModelExecute", method = RequestMethod.POST)
	// public String qualityModelDataModelExecute(@ModelAttribute("dataModel") DataModel dataModel,
	// @RequestParam(value = "aggregationsIds", required = false) List<Integer> aggregationsIds,
	// @RequestParam(value = "instancedMetricsIds", required = false) List<Integer>
	// instancedMetricsIds,
	// BindingResult result, Model model, RedirectAttributes redirectAttributes) {
	//
	// try {
	// if ((aggregationsIds == null || aggregationsIds.size() == 0)
	// && (instancedMetricsIds == null || instancedMetricsIds.size() == 0)) {
	// throw new ExceptionKiuPlus("Select the instantiated metrics and aggregations to be
	// evaluated");
	// }
	//
	// // Obtiene las metricas instanciadas identificadas con el id
	// List<InstancedMetric> instancedMetrics = new ArrayList<>();
	// if (instancedMetricsIds != null) {
	// for (Integer imId : instancedMetricsIds) {
	// instancedMetrics.add(this.instancedMetricsService.instancedMetricGet(imId));
	// }
	// }
	//
	//
	// List<Aggregation> aggregations = new ArrayList<>();
	//
	// if(aggregationsIds != null) {
	// for(Integer idA : aggregationsIds) {
	// aggregations.add(this.aggregationsService.aggregationGet(idA));
	// }
	// }
	//
	// Future<Map<String, List<GenericObject>>> resultExecute =
	// this.dataModelsService.executeDataModel(instancedMetrics, aggregations);
	//
	// Map<InstancedMetric, List<GenericObject>> instencedMetricsResult = new
	// HashMap<InstancedMetric, List<GenericObject>>();
	// if (instancedMetrics != null) {
	// for (InstancedMetric im : instancedMetrics) {
	// instencedMetricsResult.put(im, this.databaseService
	// .resultExecutionQuery(im.getDataModel().getDatabaseSource(), "estudiante"));
	// }
	// }
	//
	// Map<Aggregation, List<GenericObject>> aggregationsResult = new HashMap<Aggregation,
	// List<GenericObject>>();
	// if (aggregationsIds != null) {
	// for (Integer imA : aggregationsIds) {
	// Aggregation aggregation = this.aggregationsService.aggregationGet(imA);
	// aggregationsResult.put(aggregation, this.databaseService
	// .resultExecutionQuery(aggregation.getDataModel().getDatabaseSource(), "estudiante"));
	// }
	// }
	//
	// model.addAttribute("aggregationsResult", aggregationsResult);
	// model.addAttribute("instancedMetricsResult", instencedMetricsResult);
	//
	// model.addAttribute("dataModel", dataModel);
	//
	// return "qualityModel/qualityModelDataModelExecuteResultShow";
	// } catch (ExceptionKiuPlus | InterruptedException e) {
	// model.addAttribute("error", e.getMessage());
	//
	// // TODO: invocar la ejecucion y retornar el resultado
	// model.addAttribute("aggregationsIds", aggregationsIds);
	// model.addAttribute("instancedMetricsIds", instancedMetricsIds);
	// model.addAttribute("instancedMetrics", dataModel.getInstancedMetrics());
	//
	// List<Aggregation> aggregations = dataModel.getAggregations().stream()
	// .filter(a -> a.getType().equals(EnumAggregationType.Partial_Measurement))
	// .collect(Collectors.toList());
	//
	// model.addAttribute("aggregations", aggregations);
	// model.addAttribute("dataModel", dataModel);
	// return "qualityModel/qualityModelDataModelExecute";
	// }
	// }

}
