package com.project.kiuplus.controller.execute;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Future;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.kiuplus.configuration.logic.service.DimensionsService;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.DataModel;
import com.kiuplus.logic.model.Dimension;
import com.kiuplus.logic.model.InstancedMetric;
import com.kiuplus.logic.model.Parameter.EnumParameterType;
import com.kiuplus.logic.model.ParameterAttribute;
import com.kiuplus.logic.model.ParameterValue;
import com.kiuplus.logic.model.ResultGroup;
import com.kiuplus.logic.model.ResultGroup.EnumResultGroup;
//import com.kiuplus.logic.model.Aggregation.EnumAggregationType;
import com.kiuplus.logic.model.Attribute;
import com.kiuplus.logic.model.Attribute.EnumAttributeType;
import com.kiuplus.logic.model.Metrics.EnumGranularity;
import com.kiuplus.logic.model.AttributeColumn;
import com.kiuplus.logic.model.AttributeDatabase;
import com.kiuplus.logic.model.AttributeTable;
import com.kiuplus.logic.objects.*;
import com.kiuplus.qualityModel.logic.service.ResultGroupsService;
import com.kiuplus.qualityModel.logic.service.DataModelsService;
import com.kiuplus.qualityModel.logic.service.DatabaseService;
import com.kiuplus.qualityModel.logic.service.ExecuteService;
import com.kiuplus.qualityModel.logic.service.InstancedMetricsService;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@Controller
public class ExecuteInstancedMetricsController {

	@Autowired
	private DataModelsService dataModelsService;

	@Autowired
	private ExecuteService executeService;

	@Autowired
	private DatabaseService databaseService;

	@Autowired
	private InstancedMetricsService instancedMetricsService;

	@Autowired
	private ResultGroupsService resultGroupsService;

	@Autowired
	private DimensionsService dimensionsService;

	@Autowired
	private Environment enviroment;

	/**
	 * Retorna la pagina para seleccionar el modelo de calidad de datos y el tipo de ejecucion
	 * 
	 * @param model
	 * @return
	 */
	@RequestMapping(value = "/executeInstancedMetrics", method = RequestMethod.GET)
	public ModelAndView executeInstancedMetrics(Model model) {

		ModelAndView mv = new ModelAndView("execute/executeInstancedMetrics");
		List<DataModel> dataQualityModels = this.dataModelsService.dataModelsGet();
		mv.addObject("dataQualityModels", dataQualityModels);
		return mv;
	}

	/**
	 * @return Pagina para seleccionar las metricas instanciadas a ejecutar
	 */
	@RequestMapping(value = "/executeInstancedMetricsSelectPage", method = RequestMethod.GET)
	public ModelAndView executeInstancedMetricsSelectPage(@RequestParam int idDataQualityModel, Model model) {

		ModelAndView mv = new ModelAndView("execute/executeInstancedMetricsSelect");
		DataModel dataModel = this.dataModelsService.dataModelGetFull(idDataQualityModel);

		// Listas para guardar los ids de las metricas instanciadas seleccionadas para ejecutar
		mv.addObject("instancedMetricsIds", new ArrayList<Integer>());

		// Lista de metricas instanciadas
		mv.addObject("instancedMetrics", dataModel.getInstancedMetrics());

		mv.addObject("dataModel", dataModel);

		ObjectMapper mapper = new ObjectMapper();
		try {
			List<Dimension> dimensions = this.dimensionsService.dimensionsDataQualityModelGet(dataModel.getId());
			ObjectNode objectNode1 = mapper.createObjectNode();
			String jsonInString = mapper.writeValueAsString(dimensions);
			objectNode1.put("dimensions", jsonInString);
			mv.addObject("dimensionsObject", dimensions);
			mv.addObject("dimensions", objectNode1);
		} catch (JsonProcessingException e) {
			e.printStackTrace();
			mv.addObject("error", e.getMessage());

			return mv;
		}
		return mv;
	}

	/**
	 * 
	 * 
	 * 
	 * /** Verifica si hay una ejecucion previa de alguna de las metricas instanciadas y si alguno
	 * de los datos involucrados en la ejecucion ya estan cargados en el sistema
	 * 
	 * @param instancedMetricsIds
	 * @return Json con previusExecution indicando si hay una ejecucion previa dataLoad si los datos
	 *         estan cargados en nuestro sistema
	 * @throws Exception
	 */
	@RequestMapping(value = "/executeInstancedMetricsValidateAjax", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public @ResponseBody ObjectNode executeInstancedMetricsValidateAjax(
			@RequestParam(value = "instancedMetricsIds[]", required = false) Integer[] instancedMetricsIds)
			throws Exception {
		try {
			ObjectMapper mapper = new ObjectMapper();
			ObjectNode objectNode1 = mapper.createObjectNode();

			List<InstancedMetric> instancedMetrics = new ArrayList<>();
			if (instancedMetricsIds != null) {
				instancedMetrics = this.instancedMetricsService.instancedMetricsGet(Arrays.asList(instancedMetricsIds));
			}

			// Verifica si hay una ejecucion previa de alguna metrica instanciada o agregacion
			boolean previusExecution = false;

			for (InstancedMetric im : instancedMetrics) {
				if (im.getDateTimeResult() != null) {
					previusExecution = true;
					break;
				}
			}

			objectNode1.put("previusExecution", previusExecution);
			// Verifica si hay datos cargados que involucren alguna de las metricas instanciadas del
			// modelo de datos.
			objectNode1.put("dataLoad", instancedMetricsAttributeDataLoaded(instancedMetrics).size() > 0);

			return objectNode1;
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	/**
	 * Verifica que otra metrica instanciada ejecutada del mismo modelo de datos no utilice los
	 * datos de las metricas instanciadas a ejecutar. Es decir, al recargar los datos se modifiquen
	 * los datos utilizados por las metrias instanciadas ejecutadas.
	 * 
	 * @param instancedMetrics
	 *            Metricas instanciadas que utilizan los datos de las que se van a ejecutar
	 * @return
	 */
	private List<InstancedMetric> instancedMetricsAttributeDataLoaded(List<InstancedMetric> instancedMetrics) {

		List<InstancedMetric> instancedMetricsExecuted = this.instancedMetricsService
				.instancedMetricsGet(instancedMetrics.get(0).getDataModel().getId());

		// Elimino las no ejecutadas
		instancedMetricsExecuted.removeIf(i -> i.getDateTimeResult() == null);

		// Elimino las que se estan por ejecutar, ya que estas ejecuciones se eliminaran
		// obligatoriamente
		for (InstancedMetric im : instancedMetrics)
			instancedMetricsExecuted.removeIf(i -> i.getId() == im.getId());

		// Lista de metricas instanciadas que tienen los datos cargados
		List<InstancedMetric> instancedMetricsDataLoaded = new ArrayList<InstancedMetric>();

		// Recorro las metricas instanciadas ejecutadas
		for (InstancedMetric imExecuted : instancedMetricsExecuted) {
			for (ParameterValue pvExecuted : imExecuted.getParameterValues()) {
				if (pvExecuted.getClass().equals(ParameterAttribute.class)) {
					Attribute aExecuted = ((ParameterAttribute) pvExecuted).getAttribute();

					// Si la granularidad de alguna de las MI ejecutadas es base de datos entonces
					// se va a eliminar la ejecucion
					if (imExecuted.getMethod().getMetrics().getGranularity().equals(EnumGranularity.Database)) {
						if (!instancedMetricsDataLoaded.contains(imExecuted)) {
							instancedMetricsDataLoaded.add(imExecuted);
						}
					} else {
						// Recorro las metricas instanciadas a ejecutar
						for (InstancedMetric imToExecute : instancedMetrics) {
							for (ParameterValue pvToE : imToExecute.getParameterValues()) {
								if (pvToE.getClass().equals(ParameterAttribute.class)) {
									Attribute aToE = ((ParameterAttribute) pvToE).getAttribute();
									switch (aToE.getTypeAttribute()) {
									case DATABASE:
										// MI a ejecutar es base de datos, se elimina la ejecucion
										// de todas las MI ejecutadas
										if (imToExecute.getMethod().getMetrics().getGranularity()
												.equals(EnumGranularity.Database)) {
											for (InstancedMetric im : instancedMetricsExecuted) {
												if (!instancedMetricsDataLoaded.contains(im))
													instancedMetricsDataLoaded.add(im);
											}
											return instancedMetricsDataLoaded;
										}
										break;
									case TABLES:
										AttributeTable aToETable = (AttributeTable) aToE;

										// La metrica a ejecutar utiliza la misma tabla que la
										// ejecutada
										if (aExecuted.getTypeAttribute().equals(EnumAttributeType.TABLES)) {
											AttributeTable aExecutedTable = (AttributeTable) aExecuted;
											if (aToETable.getId().equals(aExecutedTable.getId())) {
												if (!instancedMetricsDataLoaded.contains(imExecuted)) {
													instancedMetricsDataLoaded.add(imExecuted);
												}
											}
										}

										// La metrica instanciada a ejecutar utiliza la tabla de la
										// columna de una MI ejecutada
										if (aExecuted.getTypeAttribute().equals(EnumAttributeType.COLUMNS)) {
											AttributeColumn aExecutedColumn = (AttributeColumn) aExecuted;
											if (aToETable.getId().equals(aExecutedColumn.getAttributeTable().getId())) {
												if (!instancedMetricsDataLoaded.contains(imExecuted)) {
													instancedMetricsDataLoaded.add(imExecuted);
												}
											}
										}
										break;
									case COLUMNS:
										AttributeColumn aToEColumn = (AttributeColumn) aToE;

										// La metrica a ejecutar utiliza una columna de la tabla de
										// la ejecutada
										if (aExecuted.getTypeAttribute().equals(EnumAttributeType.TABLES)) {
											AttributeTable aExecutedTable = (AttributeTable) aExecuted;
											if (aToEColumn.getAttributeTable().getId().equals(aExecutedTable.getId())) {
												if (!instancedMetricsDataLoaded.contains(imExecuted)) {
													instancedMetricsDataLoaded.add(imExecuted);
												}
											}
										}

										// La metrica instanciada a ejecutar utiliza una columna de
										// la tabla a la cual pertenece la columna de la mi
										// ejecutada
										if (aExecuted.getTypeAttribute().equals(EnumAttributeType.COLUMNS)) {
											AttributeColumn aExecutedColumn = (AttributeColumn) aExecuted;
											if (aToEColumn.getAttributeTable().getId()
													.equals(aExecutedColumn.getAttributeTable().getId())) {
												if (!instancedMetricsDataLoaded.contains(imExecuted)) {
													instancedMetricsDataLoaded.add(imExecuted);
												}
											}
										}
										break;
									case UNDEFINED:
										break;

									}
								}
							}
						}
					}
				}
			}
		}

		return instancedMetricsDataLoaded;
	}

	/**
	 * Ejecuta las agregaciones y metricas instanciadas indicadas
	 * 
	 * @param aggregationsIds
	 * @param instancedMetricsIds
	 * @param reloadData
	 *            Los datos se cargan de la base de datos fuentes
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/executeInstancedMetricsAjax", method = RequestMethod.GET)
	public ModelAndView executeInstancedMetricsAjax(
			@RequestParam(value = "instancedMetricsIds[]", required = false) Integer[] instancedMetricsIds,
			@RequestParam(value = "reloadData", required = true) boolean reloadData,
			@RequestParam(value = "idDataQualityModel", required = true) int idDataQualityModel) throws Exception {
		try {

			ModelAndView mv = new ModelAndView("execute/executeInstancedMetricsResultShow");
			if (instancedMetricsIds == null || instancedMetricsIds.length == 0) {
				throw new ExceptionKiuPlus("Select the instantiated metrics to be evaluated");
			}

			// Obtiene las metricas instanciadas identificadas con el id
			List<InstancedMetric> instancedMetrics = new ArrayList<>();

			if (instancedMetricsIds != null) {
				instancedMetrics = this.instancedMetricsService.instancedMetricsGet(Arrays.asList(instancedMetricsIds));
			}

			// Ejecuta las metricas intanciadas
			Future<Boolean> resultExecute = this.executeService.executeInstancedMetrics(instancedMetrics, reloadData);

			// Espera que termine de ejecutarse
			while (true) {
				if (resultExecute.isDone()) {
					break;
				}

				// Si aun no esta el resultado se duerme 1 segundo
				Thread.sleep(1000);

			}

			// Cargo los resultados

			// Obtiene los resultados de la ejecucion
			Map<InstancedMetric, List<GenericObject>> resultIMExecute = new HashMap<InstancedMetric, List<GenericObject>>();
			if (instancedMetrics != null) {
				for (InstancedMetric im : instancedMetrics) {

					resultIMExecute.put(im, this.executeService.resultExecutionQuery(im));
				}
			}

			mv.addObject("resultIMExecute", resultIMExecute);
			mv.addObject("dataModel", this.dataModelsService.dataModelGet(idDataQualityModel));
			return mv;
		} catch (Exception e) {
			e.printStackTrace();
			throw new ExceptionKiuPlus(e.getMessage());
		}
	}

//	/**
//	 * Retorna la pagina para seleccionar el modelo de calidad de datos para mostrar el resultado
//	 * 
//	 * @param model
//	 * @return
//	 */
//	@RequestMapping(value = "/executeInstancedMetricsResultPage", method = RequestMethod.GET)
//	public ModelAndView executeDataQualityModelResultPage(Model model) {
//
//		ModelAndView mv = new ModelAndView("execute/executeDataQualityModelResult");
//		List<DataModel> dataQualityModels = this.dataModelsService.dataModelsGet();
//		mv.addObject("dataQualityModels", dataQualityModels);
//		return mv;
//	}

	/**
	 * Obtiene el resultado de todas las ejecuciones de modelo de datos
	 * 
	 * @param idDataModel
	 *            identificador del modelo de datos
	 * @param model
	 * @return
	 */
	@RequestMapping(value = "/executeInstancedMetricsResultAjax", method = RequestMethod.GET)
	public ModelAndView executeInstancedMetricsResultAjax(@RequestParam int idDataModel, Model model) {
		ModelAndView mv = new ModelAndView("execute/executeInstancedMetricsResultShow");
		try {
			DataModel dataModel = this.dataModelsService.dataModelGetFull(idDataModel);

			// resultado
			Map<InstancedMetric, List<GenericObject>> resultIM = new HashMap<InstancedMetric, List<GenericObject>>();

			for (InstancedMetric instancedMetric : dataModel.getInstancedMetrics()) {
				resultIM.put(instancedMetric, this.executeService.resultExecutionQuery(instancedMetric));
			}

			model.addAttribute("resultIMExecute", resultIM);

			model.addAttribute("dataModel", dataModel);

		} catch (Exception e) {
			e.printStackTrace();
			model.addAttribute("error", e.getMessage());
		}
		return mv;
	}

	// *******************************************//
	// ** Exportar a PDF
	// *******************************************//
	/**
	 * Exporta el resultado de la ejecucion de los modelos de datos a PDF
	 * 
	 * @param id
	 * @param download
	 * @param request
	 * @param response
	 */
	@RequestMapping(value = "/executeInstancedMetricsResultExportPDF", method = RequestMethod.GET)
	public void qualityModelDataModelResultExportPDF(@RequestParam int id, @RequestParam boolean download,
			HttpServletRequest request, HttpServletResponse response) {

		try {
			DataModel dataModel = this.dataModelsService.dataModelGetFull(id);

			Map<InstancedMetric, List<GenericObject>> instencedMetricsResult = new HashMap<InstancedMetric, List<GenericObject>>();
			for (InstancedMetric instancedMetric : dataModel.getInstancedMetrics()) {
				instencedMetricsResult.put(instancedMetric, this.executeService.resultExecutionQuery(instancedMetric));

			}

			Map<ResultGroup, List<GenericObject>> aggregationsResult = new HashMap<ResultGroup, List<GenericObject>>();
			for (ResultGroup aggregation : dataModel.getResultGroups()) {
				aggregationsResult.put(aggregation, this.executeService.resultExecutionQuery(aggregation));

			}
			String path = enviroment.getProperty("images.source");

			ByteArrayOutputStream baos = this.executeService.qualityModelDataModelResultExportPDF(dataModel,
					instencedMetricsResult, aggregationsResult, path);

			// DocFlavor docFormat = DocFlavor.BYTE_ARRAY.AUTOSENSE;
			// Doc document = new SimpleDoc(baos.toByteArray(), docFormat, null);

			// PrintRequestAttributeSet attributeSet = new HashPrintRequestAttributeSet();
			//
			// PrintService defaultPrintService = PrintServiceLookup.lookupPrintServices();
			//
			//
			// if (defaultPrintService != null) {
			// DocPrintJob printJob = defaultPrintService.createPrintJob();
			// try {
			// printJob.print(document, attributeSet);
			//
			// } catch (Exception ex) {
			// ex.printStackTrace();
			// }
			// } else {
			// System.err.println("No existen impresoras instaladas");
			// }

			// Establece los cabezales de la respuesta
			response.setHeader("Expires", "0");
			response.setHeader("Cache-Control", "must-revalidate, post-check=0, pre-check=0");
			String fileName = "Data model " + dataModel.getName() + " result of execute.pdf";
			response.setHeader("Pragma", "public");

			if (download) {
				response.setHeader("Content-disposition", "attachment; filename=" + fileName);
			} else {
				response.setHeader("Content-disposition", "inline; filename=" + fileName);
			}

			response.setContentType("application/pdf");

			response.setContentLength(((ByteArrayOutputStream) baos).size());

			OutputStream os = response.getOutputStream();

			baos.writeTo(os);
			os.flush();
			os.close();

		} catch (Exception e) {
			e.printStackTrace();

		}
	}

}
