package com.project.kiuplus.controller.qualityModel;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.itextpdf.text.DocumentException;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
//import com.kiuplus.logic.model.Aggregation.EnumAggregationType;
import com.kiuplus.logic.model.AttributeTable;
import com.kiuplus.logic.model.DataModel;
import com.kiuplus.logic.model.InstancedMetric;
import com.kiuplus.logic.model.Operation;
import com.kiuplus.logic.model.Parameter;
import com.kiuplus.logic.model.Parameter.EnumDataType;
import com.kiuplus.logic.model.Parameter.EnumParameterType;
import com.kiuplus.logic.model.ParameterValue;
import com.kiuplus.logic.model.Reference;
import com.kiuplus.logic.model.ResultGroup;
import com.kiuplus.logic.model.Metrics.EnumGranularity;
import com.kiuplus.logic.model.ResultGroup.EnumResultGroup;
import com.kiuplus.logic.objects.GenericObject;
import com.kiuplus.qualityModel.logic.service.ResultGroupsService;
import com.kiuplus.qualityModel.logic.service.DataModelsService;
import com.kiuplus.qualityModel.logic.service.DatabaseService;
import com.kiuplus.qualityModel.logic.service.ExecuteService;
import com.kiuplus.qualityModel.logic.service.InstancedMetricsService;
import com.kiuplus.webservices.dataquality.WebServiceService;
import com.project.kiuplus.exceptions.ExceptionMethod;

@SessionAttributes({ "aggregation", "dataModel" })
@Controller
public class QualityModelAggregationsController {

	@Autowired
	private HttpSession session;

	@Autowired
	private InstancedMetricsService instancedMetricsService;

	@Autowired
	private DataModelsService dataModelsService;

	@Autowired
	private DatabaseService databaseService;

	@Autowired
	private ResultGroupsService resultGroupsService;

	@Autowired
	private WebServiceService webServiceService;

	@Autowired
	private ExecuteService executeService;

	@ModelAttribute("enumParameterTypeList")
	public EnumParameterType[] enumParameterTypeListLoad() {
		EnumParameterType[] enumParameterTypeList = EnumParameterType.values();
		return enumParameterTypeList;
	}

	/**
	 * @return Pagina para visualizar la agregacion
	 */
	@RequestMapping(value = "/qualityModelAggregationShowPage", method = RequestMethod.GET)
	public ModelAndView qualityModelAggregationShowPage(@RequestParam int id) {

		ResultGroup aggregation = this.resultGroupsService.resultGroupGet(id);

		ModelAndView mv = new ModelAndView("qualityModel/qualityModelAggregationShow");

		mv.addObject("aggregation", aggregation);

		InstancedMetric instancedMetric = new InstancedMetric();
		for (ParameterValue parameterValue : aggregation.getParameterValues()) {
			if (parameterValue.getParameter().getParameterType().equals(EnumParameterType.TABLE_NAME)
					|| parameterValue.getParameter().getParameterType().equals(EnumParameterType.COLUMN_NAME)) {
				instancedMetric = parameterValue.getInstancedMetric();
			}
		}

		mv.addObject("instancedMetric", instancedMetric);

		return mv;
	}

	@RequestMapping(value = "/qualityModelAggregationCreatePage", method = RequestMethod.GET)
	public ModelAndView qualityModelAggregationCreatePage(@RequestParam int idDataModel) {

		ModelAndView mv;

		mv = new ModelAndView("qualityModel/qualityModelAggregationCreateEdit");

		DataModel dataModel = this.dataModelsService.dataModelGet(idDataModel);

		// Solo se pueden seleccionar metricas instanciadas del tipo1
		List<InstancedMetric> instancedMetricsAux = this.instancedMetricsService
				.instancedMetricsGet(idDataModel);
		List<InstancedMetric> instancedMetrics = new ArrayList<InstancedMetric>();
		for (InstancedMetric im : instancedMetricsAux) {
			if (im.getMethod().getMetrics().getGranularity().isGroup1()) {
				instancedMetrics.add(im);
			}
		}
		
		mv.addObject("instancedMetrics", instancedMetrics);
		mv.addObject("aggregation", new ResultGroup(dataModel, EnumResultGroup.Aggregation));

		mv.addObject("granularityList", EnumGranularity.getGroup2());
		return mv;
	}

	@RequestMapping(value = "/qualityModelAggregationEditPage", method = RequestMethod.GET)
	public String qualityModelAggregationEditPage(@ModelAttribute DataModel dataModel, @RequestParam int idAggregation,
			Model model, BindingResult result) {

		try {

			ResultGroup aggregation = this.resultGroupsService.resultGroupGet(idAggregation);
			if (aggregation == null) {
				model.addAttribute("error", "Aggregation does not exist");

				return "redirect:qualityModelDataModelShowPage?id=" + dataModel.getId();
			}

			// Se busca la metrica instanciada en los parametros
			InstancedMetric instancedMetric = new InstancedMetric();
			for (ParameterValue parameterValue : aggregation.getParameterValues()) {
				if (parameterValue.getParameter().getParameterType().equals(EnumParameterType.TABLE_NAME)
						|| parameterValue.getParameter().getParameterType().equals(EnumParameterType.COLUMN_NAME)) {
					instancedMetric = parameterValue.getInstancedMetric();
				}
			}

			this.qualityAggregationCreatEditLoad(aggregation, instancedMetric.getName(), model);

			model.addAttribute("aggregation", aggregation);

			// Carga las tablas
			List<AttributeTable> tables = this.databaseService
					.tablesColumnsGet(aggregation.getDataModel().getDatabaseSource());
			model.addAttribute("tables", tables);
			return "qualityModel/qualityModelAggregationCreateEdit";
		} catch (ExceptionKiuPlus e) {
			e.printStackTrace();
			model.addAttribute("error", e.getMessage());

			return "qualityModel/qualityModelDataModelShow";
		}
	}

	@RequestMapping(value = "/qualityModelAggregationCreateEdit", method = RequestMethod.POST)
	public String qualityModelAggregationCreateEdit( @ModelAttribute("aggregation") @Valid ResultGroup aggregation,
			 BindingResult result, Model model, @RequestParam(value="instancedMetric", required=true) String nameInstancedMetric,
			RedirectAttributes redirectAttributes) {

		try {

			if (this.qualityAggregationCreateEditValidar(aggregation, nameInstancedMetric, result, model)
					|| result.hasErrors()) {
				this.qualityAggregationCreatEditLoad(aggregation, nameInstancedMetric, model);
				return "qualityModel/qualityModelAggregationCreateEdit";
			}

			// Asigna la metrica instanciada seleccionada a los parametros del tipo TABLE_NAME o
			// COLUMN_NAME
			InstancedMetric instancedMetricSelected = this.instancedMetricsService
					.instancedMetricFind(nameInstancedMetric);

			for (int i = 0; i < aggregation.getParameterValues().size(); i++) {

				ParameterValue parameterValue = aggregation.getParameterValues().get(i);
				if (parameterValue.getParameter().getParameterType().equals(EnumParameterType.TABLE_NAME)
						|| parameterValue.getParameter().getParameterType().equals(EnumParameterType.COLUMN_NAME)) {
					parameterValue.setInstancedMetric(instancedMetricSelected);
				}

				Parameter parameterOperation = (Parameter) aggregation.getOperation().getParameters().stream()
						.filter(p -> p.getName().equals(parameterValue.getParameter().getName())).toArray()[0];
				parameterOperation.setParameterType(parameterValue.getParameter().getParameterType());
				parameterValue.setParameter(parameterOperation);

			}

			// Elimino las operaciones que no son de la aggregacion, estas operaciones se cargaron
			// para que el usuario pudiera elegir
			List<Operation> operations = new ArrayList<>();
			operations.add(aggregation.getOperation());
			aggregation.getOperation().getWebService().setOperations(operations);

			if (aggregation.getId() == 0) {
				this.resultGroupsService.resultGroupCreate(aggregation);
			} else {
				this.resultGroupsService.resultGroupEdit(aggregation);
			}

			model.addAttribute("instancedMetric", instancedMetricSelected);
			return "qualityModel/qualityModelAggregationShow";
		} catch (Exception e) {
			e.printStackTrace();
			model.addAttribute("error", e.getMessage());
			this.qualityAggregationCreatEditLoad(aggregation, nameInstancedMetric, model);
			return "qualityModel/qualityModelAggregationCreateEdit";
		}
	}

	private void qualityAggregationCreatEditLoad(ResultGroup aggregation, String nameInstancedMetric, Model model) {

		List<InstancedMetric> instancedMetricsAux = this.instancedMetricsService
				.instancedMetricsGet(aggregation.getDataModel().getId());
		List<InstancedMetric> instancedMetrics = new ArrayList<InstancedMetric>();
		// Solo se puede seleccionar una metrica instanciada del tipo1
		for (InstancedMetric im : instancedMetricsAux) {
			if (im.getMethod().getMetrics().getGranularity().isGroup1()) {
				instancedMetrics.add(im);
			}
		}
		model.addAttribute("instancedMetrics", instancedMetrics);

		// Carga la metrica instanciada seleccionada por el usuario
		if (!nameInstancedMetric.equals("")) {
			InstancedMetric instancedMetricSelected = this.instancedMetricsService
					.instancedMetricFind(nameInstancedMetric);
			model.addAttribute("instancedMetric", instancedMetricSelected);
		}

		// si no hay conexion o la url no es correcta se muestra solo la operacion de la
		// agregacion

		try {
			List<Operation> operations = this.webServiceService
					.operationsGet(aggregation.getOperation().getWebService().getUrl());
			session.setAttribute("listOperations", operations);
			aggregation.getOperation().getWebService().setOperations(operations);
		} catch (Exception e) {
			e.printStackTrace();
			if (e.getClass().equals(ExceptionKiuPlus.class)) {
				model.addAttribute("error", e.getMessage());
			}
		}

		// Carga las metricas instanciadas a mostrar en los combobox.

		model.addAttribute("granularityList", EnumGranularity.getGroup2());
	}

	/**
	 * Valida la agregacion controlando que el usuario selecciono todos los elementos
	 * 
	 * @param aggregation
	 * @param result
	 * @return retorna si hay un error fuera del binding result
	 */
	private boolean qualityAggregationCreateEditValidar(ResultGroup aggregation, String instancedMetricName,
			BindingResult result, Model model) {

		boolean error = false;

		// Valida que el usuario seleccione la metrica instanciada
		if (instancedMetricName.equals("")) {
			model.addAttribute("instancedMetricError", "Select an instanced metric");
			error = true;
		}

		if (aggregation.getOperation().getWebService().getUrl() == "") {
			result.rejectValue("operation.webService.url", "blanckValue", new Object[] { "'operation.webService.url'" },
					"Can not be empty");
		}

		if (aggregation.getOperation().getName() == null || aggregation.getOperation().getName().equals("")) {
			result.rejectValue("operation.name", "blanckValue", new Object[] { "'operation.name'" },
					"Select an operation");
		} else {

			// Valida los parametros
			int connectToDBCount = 0;

			for (ParameterValue parameterValue : aggregation.getParameterValues()) {
				// Tipo de parametro
				if (parameterValue.getParameter().getParameterType() == null) {
					result.rejectValue("parameterValues", "blanckValue", new Object[] { "'parameterValues'" },
							"Select the type of " + parameterValue.getParameter().getName());
				} else {

					// Valor de refencia
					if (parameterValue.getParameter().getParameterType().equals(EnumParameterType.REFERENCE)
							&& ((Reference) parameterValue).getReferenceValue().equals("")) {
						result.rejectValue("parameterValues", "blanckValue", new Object[] { "'parameterValues'" },
								"Enter the value of " + parameterValue.getParameter().getName());
					}

					// Si es del tipo nombre de columna, nombre de tabla o conexion a base de datos
					// entonces el tipo del parametro tiene que ser string
					if ((parameterValue.getParameter().getParameterType().equals(EnumParameterType.CONNECT_TO_DB)
							|| parameterValue.getParameter().getParameterType().equals(EnumParameterType.COLUMN_NAME)
							|| parameterValue.getParameter().getParameterType().equals(EnumParameterType.TABLE_NAME))
							&& !parameterValue.getParameter().getDataType().equals(EnumDataType.STRING)) {
						
						String parameterTypeName = "";
						switch (parameterValue.getParameter().getParameterType()) {
						case CONNECT_TO_DB:
							parameterTypeName = "Connect to DB";
							break;
						case COLUMN_NAME:
							parameterTypeName = "Name of the results column";
							break;
						case TABLE_NAME:
							parameterTypeName = "Name of the results table";
							break;
						default:
							parameterValue.getParameter().getParameterType().getText();
						}
						
						result.rejectValue("parameterValues", "blanckValue", new Object[] { "'parameterValues'" },
								" The datatype of parameters of type " + parameterTypeName + " must be string.");
					}

					if (parameterValue.getParameter().getParameterType().equals(EnumParameterType.CONNECT_TO_DB)) {
						connectToDBCount += 1;
					}

				}

			}
			// Connect to DB

			// Supongo que si la metrica instanciada es del tipo celda, cellset o tupla entonces el
			// resultado de la MI es una columna
			// y la operacion de la agregacion se tiene que conectar a la base de datos

			// boolean connectTODB = false;
			// List<InstancedMetric> intancedMetricsAux = this.instancedMetricsService
			// .instancedMetricsGet(aggregation.getDataModel().getId());
			// for (InstancedMetric im : instancedMetrics) {
			// InstancedMetric instancedMetric = (InstancedMetric) (intancedMetricsAux.stream()
			// .filter(i -> i.getName().equals(im.getName())).toArray())[0];
			// connectTODB = connectTODB || !instancedMetric.getConnectToDB();
			// }
			//
			// if (connectTODB) {
			if (connectToDBCount == 0) {

				result.rejectValue("parameterValues", "blanckValue", new Object[] { "'parameterValues'" },
						"Select the parameter connect to database");
			}

			if (connectToDBCount > 1) {
				result.rejectValue("parameterValues", "blanckValue", new Object[] { "'parameterValues'" },
						"Select only one parameter connect to database");
			}
			// } else {
			// // Cuando la granularidad de la metrica es Column, columSet, Table o DB la
			// // agregacion no se conecta a la base de datos
			// if (connectToDBCount > 0) {
			//
			// result.rejectValue("parameterValues", "blanckValue", new Object[] {
			// "'parameterValues'" },
			// "The aggregation does not connect to the database");
			// }
			// }
			// El usuario debe seleccionar el nombre de la tabla y el nombre de la columna de
			// resultados como parametro.
			if (!aggregation.getParameterValues().stream()
					.anyMatch(p -> p.getParameter().getParameterType().equals(EnumParameterType.TABLE_NAME))) {
				result.rejectValue("parameterValues", "blanckValue", new Object[] { "'parameterValues'" },
						"Select the parameter name of the table");
			}
			if (!aggregation.getParameterValues().stream()
					.anyMatch(p -> p.getParameter().getParameterType().equals(EnumParameterType.COLUMN_NAME))) {
				result.rejectValue("parameterValues", "blanckValue", new Object[] { "'parameterValues'" },
						"Select the parameter name of the column");
			}
		}
		return error;
	}

	@RequestMapping(value = "/qualityModelAggregationDeletePage", method = RequestMethod.GET)
	public String qualityModelAggregationDeletePage(@RequestParam int id, @RequestParam int idDataModel, Model model,
			RedirectAttributes redirectAttributes) {

		try {

			ResultGroup aggregation = this.resultGroupsService.resultGroupGet(id);
			this.resultGroupsService.resultGroupDelete(id);
			redirectAttributes.addFlashAttribute("message", "Aggregation deleted successfully.");

			return "redirect:qualityModelDataModelShowPage?id=" + aggregation.getDataModel().getId();

		} catch (ExceptionKiuPlus ex) {
			redirectAttributes.addFlashAttribute("error", "Can not delete the aggregation: " + ex.getMessage());
			return "redirect:qualityModelAggregationShowPage?id=" + id;

		}
	}

	/**
	 * Obtiene los metodos de la metrica
	 * 
	 * @param idMetrics
	 * @param model
	 * @return Parametros basicos de las metricas
	 * @throws Exception
	 */
	// @RequestMapping(value = "/instancedMetricGet", method = RequestMethod.GET, produces =
	// MediaType.APPLICATION_JSON_VALUE)
	// public @ResponseBody InstancedMetric instancedMetricGet(
	// @RequestParam(value = "idInstancedMetric", required = true) int idInstancedMetric, Model
	// model) {
	//
	// return this.instancedMetricsService.instancedMetricGet(idInstancedMetric);
	//
	// }

	@RequestMapping(value = "/aggregationTypeParameterSet", method = RequestMethod.GET)
	@ResponseStatus(value = HttpStatus.OK)
	public void aggregationTypeParameterSet(@RequestParam(value = "index", required = true) int index,
			@RequestParam(value = "typeParameter", required = true) String typeParameter, Model model,
			HttpServletRequest request) {
		try {
			ResultGroup aggregation = (ResultGroup) request.getSession().getAttribute("aggregation");
			ParameterValue parameterValue = new ParameterValue();
			switch (typeParameter) {
			case "REFERENCE":
				parameterValue = new Reference();
				break;
			case "TABLE_NAME":
				parameterValue.getParameter().setParameterType(EnumParameterType.TABLE_NAME);
				break;
			case "COLUMN_NAME":
				parameterValue.getParameter().setParameterType(EnumParameterType.COLUMN_NAME);
				break;
			default:
				parameterValue.getParameter().setParameterType(EnumParameterType.CONNECT_TO_DB);
			}
			aggregation.getParameterValues().set(index, parameterValue);
			request.getSession().setAttribute("aggregation", aggregation);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

//	public void aggregationIMSet(String nameInstancedMetric, ResultGroup aggregation) {
//		try {
//
//			for (ParameterValue p : aggregation.getParameterValues()) {
//				if (p.getParameter().getParameterType().equals(EnumParameterType.COLUMN_NAME)
//						|| p.getParameter().getParameterType().equals(EnumParameterType.TABLE_NAME)) {
//
//					p.setInstancedMetric(this.instancedMetricsService.instancedMetricFind(nameInstancedMetric));
//				}
//			}
//
//		} catch (Exception e) {
//			e.printStackTrace();
//		}
//	}

	/**
	 * Carga en la agregacion la operacion seleccionada por el usuario y crea su lista de parametros
	 * de valor.
	 * 
	 * @param operation
	 * @param model
	 * @param request
	 * @throws ExceptionKiuPlus
	 */
	@RequestMapping(value = "/aggregationOperationSelect", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public @ResponseBody Operation aggregationOperationSelect(
			@RequestParam(value = "nameOperation", required = true) String nameOperation, Model model,
			HttpServletRequest request) throws Exception {
		try {
			List<Operation> operations = (ArrayList<Operation>) session.getAttribute("listOperations");
			ResultGroup aggregation = (ResultGroup) request.getSession().getAttribute("aggregation");
			Operation operation = (Operation) operations.stream().filter(o -> o.getName().equals(nameOperation))
					.toArray()[0];

			aggregation.setOperation(operation);
			aggregation.getParameterValues().removeAll(aggregation.getParameterValues());
			for (int i = 0; i < operation.getParameters().size(); i++) {
				aggregation.getParameterValues().add(new ParameterValue());
			}

			request.getSession().setAttribute("aggregation", aggregation);

			return operation;
		} catch (Exception e) {
			throw new ExceptionKiuPlus("The operation was not found");
		}
	}

	// *****************************************//
	// Ejecucion de agregacion
	// *****************************************//

	/**
	 * Ejecuta la agregacion y muestra el resultado
	 * 
	 * @param id
	 *            de la agregacion
	 * @param model
	 * @return
	 */
//	@RequestMapping(value = "/qualityModelDataModelExecuteAggregation", method = RequestMethod.GET)
//	public String qualityModelDataModelExecuteAggregation(@RequestParam int id, Model model) {
//		try {
//
//			ResultGroup aggregation = this.resultGroupsService.resultGroupGet(id);
//
//			// TODO: invocar la ejecucion
//
//			return "redirect:qualityModelDataModelExecuteAggregationResultPage?id=" + id;
//		} catch (Exception e) {
//			model.addAttribute("error", e.getMessage());
//			return "qualityModel/qualityModelAggregationShow";
//		}
//	}
//
//	/**
//	 * Muestra el resultado de la ejecucion si ya se ejecuto
//	 * 
//	 * @param id
//	 *            de la agregacion
//	 * @param model
//	 * @return
//	 */
//	@RequestMapping(value = "/qualityModelDataModelExecuteAggregationResultPage", method = RequestMethod.GET)
//	public String qualityModelDataModelExecuteAggregationResultPage(@RequestParam int id, Model model) {
//
//		try {
//
//			ResultGroup aggregation = this.resultGroupsService.resultGroupGet(id);
//
//			List<GenericObject> genericObjects = this.executeService.resultExecutionQuery(
//					aggregation.getDataModel().getDatabaseMetadata(), aggregation.getMetadataTableResult());
//			Set<String> resultColumnName = new ConcurrentSkipListSet<>();
//			if (!genericObjects.isEmpty()) {
//				resultColumnName = genericObjects.get(0).getProperties().keySet();
//			}
//
//			model.addAttribute("resultColumnName", resultColumnName);
//			model.addAttribute("result", genericObjects);
//			model.addAttribute("aggregation", aggregation);
//
//			return "qualityModel/qualityModelDataModelExecuteAggregationResult";
//		} catch (Exception e) {
//			e.printStackTrace();
//			model.addAttribute("error", e.getMessage());
//			return "qualityModel/qualityModelAggregationShow";
//		}
//	}

	@RequestMapping(value = "/qualityModelAggregationResultExportPDF", method = RequestMethod.GET)
	public void qualityModelAggregationResultExportPDF(@RequestParam int id, HttpServletRequest request,
			HttpServletResponse response) {

		try {
			ResultGroup aggregation = this.resultGroupsService.resultGroupGet(id);

			List<GenericObject> genericObjects = this.executeService.resultExecutionQuery(aggregation);

			ByteArrayOutputStream baos = this.resultGroupsService.resultGroupResultExportToPDF(aggregation,
					genericObjects);

			// setting some response headers
			response.setHeader("Expires", "0");
			response.setHeader("Cache-Control", "must-revalidate, post-check=0, pre-check=0");
			String fileName = "Aggregation " + aggregation.getName() + " result of execute.pdf";
			response.setHeader("Pragma", "public");
			response.setHeader("Content-disposition", "attachment; filename=" + fileName);
			// setting the content type
			response.setContentType("application/pdf");
			// the contentlength
			response.setContentLength(((ByteArrayOutputStream) baos).size());
			// write ByteArrayOutputStream to the ServletOutputStream
			OutputStream os = response.getOutputStream();

			baos.writeTo(os);
			os.flush();
			os.close();

		} catch (DocumentException | IOException | ExceptionKiuPlus e) {
			// TODO Auto-generated catch block
			e.printStackTrace();

		}
	}

//	@RequestMapping(value = "/aggregationsGet", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
//	public @ResponseBody List<ResultGroup> aggregationsGet(
//			@RequestParam(value = "aggregationsIds[]", required = false) Integer[] aggregationsIds) throws Exception {
//		try {
//
//			if (aggregationsIds != null) {
//				return this.resultGroupsService.resultGroupsGet(Arrays.asList(aggregationsIds));
//			} else {
//				return new ArrayList<>();
//			}
//
//		} catch (Exception e) {
//			throw new ExceptionMethod("It is not possible to obtain the instanced metrics.");
//		}
//	}

	@ExceptionHandler({ ExceptionMethod.class })
	public ResponseEntity<String> manageMethodException(Exception e) {
		return new ResponseEntity<String>(e.getMessage(), HttpStatus.NOT_FOUND);
	}
}
