package com.project.kiuplus.controller.qualityModel;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.ResultGroup;
import com.kiuplus.logic.model.Metrics.EnumGranularity;
//import com.kiuplus.logic.model.ResultGroup.EnumResultGroupType;
import com.kiuplus.logic.model.AttributeTable;
import com.kiuplus.logic.model.DataModel;
import com.kiuplus.logic.model.InstancedMetric;
import com.kiuplus.logic.model.Operation;
import com.kiuplus.logic.model.Parameter;
import com.kiuplus.logic.model.Parameter.EnumParameterType;
import com.kiuplus.logic.model.ResultGroup.EnumResultGroup;
import com.kiuplus.logic.model.ParameterValue;
import com.kiuplus.logic.model.Reference;
import com.kiuplus.qualityModel.logic.service.ResultGroupsService;
import com.kiuplus.qualityModel.logic.service.DataModelsService;
import com.kiuplus.qualityModel.logic.service.DatabaseService;
import com.kiuplus.qualityModel.logic.service.ExecuteService;
import com.kiuplus.qualityModel.logic.service.InstancedMetricsService;
import com.kiuplus.webservices.dataquality.WebServiceService;
import com.project.kiuplus.exceptions.ExceptionMethod;

@SessionAttributes({ "composedIM", "dataModel" })
@Controller
public class QualityModelComposedIMController {

	@Autowired
	private HttpSession session;

	@Autowired
	private InstancedMetricsService instancedMetricsService;

	@Autowired
	private DataModelsService dataModelsService;

	@Autowired
	private DatabaseService databaseService;

	@Autowired
	private ResultGroupsService resultGroupsService;

	@Autowired
	private WebServiceService webServiceService;

	@Autowired
	private ExecuteService executeService;

	@ModelAttribute("enumParameterTypeList")
	public EnumParameterType[] enumParameterTypeListLoad() {
		EnumParameterType[] enumParameterTypeList = EnumParameterType.values();
		return enumParameterTypeList;
	}

	/**
	 * @return Pagina para visualizar la MIC
	 */
	@RequestMapping(value = "/qualityModelComposedIMShowPage", method = RequestMethod.GET)
	public ModelAndView qualityModelComposedIMShowPage(@RequestParam int id) {

		ResultGroup composedIM = this.resultGroupsService.resultGroupGet(id);

		ModelAndView mv = new ModelAndView("qualityModel/qualityModelComposedIMShow");

		// Aca no se muestran las metricas intanciadas porque se visualizan en los parametros
		mv.addObject("composedIM", composedIM);
		return mv;
	}

	@RequestMapping(value = "/qualityModelComposedIMCreatePage", method = RequestMethod.GET)
	public ModelAndView qualityModelComposedIMCreatePage(@RequestParam int idDataModel) {

		ModelAndView mv;

		mv = new ModelAndView("qualityModel/qualityModelComposedIMCreateEdit");

		DataModel dataModel = this.dataModelsService.dataModelGet(idDataModel);
		List<InstancedMetric> instancedMetrics = this.instancedMetricsService.instancedMetricsGet(idDataModel);
		mv.addObject("instancedMetrics", instancedMetrics);
		mv.addObject("composedIM", new ResultGroup(dataModel, EnumResultGroup.ComposedInstancedMetric));
		if(instancedMetrics.get(0).getMethod().getMetrics().getGranularity().isGroup1()) {
			mv.addObject("granularityList", EnumGranularity.getGroup1());
		}else {
			mv.addObject("granularityList", EnumGranularity.getGroup2());
		}
		List<InstancedMetric> instancedMetricsSelected = new ArrayList<>();
		instancedMetricsSelected.add(new InstancedMetric());
		instancedMetricsSelected.add(new InstancedMetric());
		mv.addObject("instancedMetricsSelected", instancedMetricsSelected);
		return mv;
	}

	@RequestMapping(value = "/qualityModelComposedIMEditPage", method = RequestMethod.GET)
	public String qualityModelComposedIMEditPage(@ModelAttribute DataModel dataModel, @RequestParam int idComposedIM,
			Model model, BindingResult result) {

		try {

			ResultGroup composedIM = this.resultGroupsService.resultGroupGet(idComposedIM);
			if (composedIM == null) {
				model.addAttribute("error", "ComposedIM does not exist");

				return "redirect:qualityModelDataModelShowPage?id=" + dataModel.getId();
			}

			this.qualityComposedIMCreatEditLoad(composedIM, model, new String[0]);

			
			model.addAttribute("composedIM", composedIM);

			
			return "qualityModel/qualityModelComposedIMCreateEdit";
		} catch (Exception e) {
			e.printStackTrace();
			model.addAttribute("error", e.getMessage());

			return "qualityModel/qualityModelDataModelShow";
		}
	}

	@RequestMapping(value = "/qualityModelComposedIMCreateEdit", method = RequestMethod.POST)
	public String qualityModelComposedIMCreateEdit(@ModelAttribute("composedIM") @Valid ResultGroup composedIM,
			BindingResult result, Model model, @RequestParam("nameInstancedMetrics[]") String[] nameInstancedMetrics,
			RedirectAttributes redirectAttributes) {

		try {
			// Verifica errores

			if (this.qualityComposedIMCreateEditValidar(composedIM, nameInstancedMetrics, result, model)
					|| result.hasErrors()) {
				this.qualityComposedIMCreatEditLoad(composedIM, model, nameInstancedMetrics);
				return "qualityModel/qualityModelComposedIMCreateEdit";
			}

			for (int i = 0; i < composedIM.getParameterValues().size(); i++) {

				ParameterValue parameterValue = composedIM.getParameterValues().get(i);
				if (parameterValue.getParameter().getParameterType().equals(EnumParameterType.ATTRIBUTE)) {

					InstancedMetric instancedMetric = this.instancedMetricsService
							.instancedMetricFind(parameterValue.getInstancedMetric().getName());
					parameterValue.setInstancedMetric(instancedMetric);

				}

				Parameter parameterOperation = (Parameter) composedIM.getOperation().getParameters().stream()
						.filter(p -> p.getName().equals(parameterValue.getParameter().getName())).toArray()[0];
				parameterOperation.setParameterType(parameterValue.getParameter().getParameterType());
				parameterValue.setParameter(parameterOperation);

			}

			// Elimino las operaciones que no son de la aggregacion, estas operaciones se cargaron
			// para que el usuario pudiera elegir
			List<Operation> operations = new ArrayList<>();
			operations.add(composedIM.getOperation());
			composedIM.getOperation().getWebService().setOperations(operations);

			if (composedIM.getId() == 0) {
				this.resultGroupsService.resultGroupCreate(composedIM);
			} else {
				this.resultGroupsService.resultGroupEdit(composedIM);
			}

			return "qualityModel/qualityModelComposedIMShow";
		} catch (Exception e) {
			e.printStackTrace();
			model.addAttribute("error", e.getMessage());
			this.qualityComposedIMCreatEditLoad(composedIM, model, nameInstancedMetrics);
			return "qualityModel/qualityModelComposedIMCreateEdit";
		}
	}

	private void qualityComposedIMCreatEditLoad(ResultGroup composedIM, Model model, String[] nameInstancedMetrics) {

		List<InstancedMetric> instancedMetrics = this.instancedMetricsService
				.instancedMetricsGet(composedIM.getDataModel().getId());
		model.addAttribute("instancedMetrics", instancedMetrics);

		// si no hay conexion o la url no es correcta se muestra solo la operacion de la
		// MIC

		try {
			List<Operation> operations = this.webServiceService
					.operationsGet(composedIM.getOperation().getWebService().getUrl());
			session.setAttribute("listOperations", operations);
			composedIM.getOperation().getWebService().setOperations(operations);
		} catch (Exception e) {
			e.printStackTrace();
			if (e.getClass().equals(ExceptionKiuPlus.class)) {
				model.addAttribute("error", e.getMessage());
			}
		}

		// Carga las metricas instanciadas a mostrar en los combobox de arriba
		List<InstancedMetric> instancedMetricsSelected = new ArrayList<>();
		if (nameInstancedMetrics.length > 0) {
			for (String nameIM : nameInstancedMetrics) {
				instancedMetricsSelected.add(this.instancedMetricsService.instancedMetricFind(nameIM));
			}
		} else {

			for (ParameterValue parameter : composedIM.getParameterValues()) {
				if (parameter.getInstancedMetric() != null) {
					instancedMetricsSelected.add(parameter.getInstancedMetric());
				}
			}

		}
		model.addAttribute("instancedMetricsSelected", instancedMetricsSelected);

		if(instancedMetricsSelected.get(0).getMethod().getMetrics().getGranularity().isGroup1()) {
			model.addAttribute("granularityList", EnumGranularity.getGroup1());
		}else {
			model.addAttribute("granularityList", EnumGranularity.getGroup2());
		}
		
	}

	/**
	 * Valida la MIC controlando que el usuario selecciono todos los elementos
	 * 
	 * @param composedIM
	 * @param result
	 */
	private boolean qualityComposedIMCreateEditValidar(ResultGroup composedIM, String[] nameInstancedMetrics,
			BindingResult result, Model model) {

		boolean error = false;
		int instancedMetricsCount = 0;

		if (composedIM.getOperation().getWebService().getUrl() == "") {
			result.rejectValue("operation.webService.url", "blanckValue", new Object[] { "'operation.webService.url'" },
					"Can not be empty");
		}

		if (composedIM.getOperation().getName() == null || composedIM.getOperation().getName().equals("")) {
			result.rejectValue("operation.name", "blanckValue", new Object[] { "'operation.name'" },
					"Select an operation");
		} else {

			// Contador para la cantidad de tipos instanced metric
			for (ParameterValue parameterValue : composedIM.getParameterValues()) {
				// Seleccione todos los tipos de parametros
				if (parameterValue.getParameter().getParameterType() == null) {
					result.rejectValue("parameterValues", "blanckValue", new Object[] { "'parameterValues'" },
							"Select the type of " + parameterValue.getParameter().getName());
				} else {
					// Instanced metrics
					if (parameterValue.getParameter().getParameterType().equals(EnumParameterType.ATTRIBUTE)) {
						if (parameterValue.getInstancedMetric() == null
								|| parameterValue.getInstancedMetric().getName().isEmpty()) {
							result.rejectValue("parameterValues", "blanckValue", new Object[] { "'parameterValues'" },
									"Select the instanced metric of " + parameterValue.getParameter().getName());

						} else {
							instancedMetricsCount++;
						}
					}
					// Valor de refencia
					if (parameterValue.getParameter().getParameterType().equals(EnumParameterType.REFERENCE)
							&& ((Reference) parameterValue).getReferenceValue().equals("")) {
						result.rejectValue("parameterValues", "blanckValue", new Object[] { "'parameterValues'" },
								"Enter the value of " + parameterValue.getParameter().getName());
					}

				}

			}

		}

		// Metricas instanciadas seleccionadas

		// Lista para guardar las metricas instanciadas de los parametros, sirve para controlar
		// la cantidad que seleccione y que la operacion se tiene que conectar a la BD
		List<InstancedMetric> instancedMetrics = new ArrayList<>();

		// Verifica que todas las metricas intanciadas sean de granularidades compatibles
		int isGroup1Count = 0;
		int isGroup2Count = 0;
		for (String imName : nameInstancedMetrics) {
			InstancedMetric im = this.instancedMetricsService.instancedMetricFind(imName);
			instancedMetrics.add(im);
			if (im.getMethod().getMetrics().getGranularity().isGroup1()) {
				isGroup1Count++;
			} else {
				isGroup2Count++;
			}
		}
		if (isGroup1Count > 0 && isGroup2Count > 0) {
			model.addAttribute("instancedMetricError",
					"All instanced metrics must belong to the group Cell, Cell Set and Tuple or group Column, Column Set, Table and Database.");
			error = true;
		} else {
			// La granularidad final debe ser del grupo de granularidades de la metrica
			// instanciada
			if (isGroup1Count > 0) {
				if (!composedIM.getGranularity().isGroup1()) {
					result.rejectValue("granularity", "blanckValue", new Object[] { "'instancedMetrics'" },
							"The final ganularity must be Cell, Cell Set or Tuple.");
				}
			}

			if (isGroup2Count > 0) {
				if (!composedIM.getGranularity().isGroup2()) {
					result.rejectValue("granularity", "blanckValue", new Object[] { "'instancedMetrics'" },
							"The final ganularity must be Column, Column Set, Table and Database.");
				}
			}
		}

		for (int i = 0; i < nameInstancedMetrics.length; i++) {
			if (nameInstancedMetrics[i].equals("")) {
				model.addAttribute("instancedMetricError", "Select a value for every Instanced Metric");
				error = true;
				break;
			}
		}

		// Controlo que se seleccionen todas las metricas instanciadas
		if (instancedMetrics.size() > instancedMetricsCount) {
			result.rejectValue("parameterValues", "blanckValue", new Object[] { "'parameterValues'" },
					"Select the parameter for all instanced metrics");
		}

		return error;
	}

	@RequestMapping(value = "/qualityModelComposedIMDeletePage", method = RequestMethod.GET)
	public String qualityModelComposedIMDeletePage(@RequestParam int id, @RequestParam int idDataModel, Model model,
			RedirectAttributes redirectAttributes) {

		try {

			ResultGroup composedIM = this.resultGroupsService.resultGroupGet(id);
			this.resultGroupsService.resultGroupDelete(id);
			redirectAttributes.addFlashAttribute("message", "Composed Instanced Metric deleted successfully.");

			return "redirect:qualityModelDataModelShowPage?id=" + composedIM.getDataModel().getId();

		} catch (ExceptionKiuPlus ex) {
			redirectAttributes.addFlashAttribute("error", "Can not delete the composedIM: " + ex.getMessage());
			return "redirect:qualityModelComposedIMShowPage?id=" + id;

		}
	}

	// **************************//
	// llamados ajax
	// **************************//

	

	@RequestMapping(value = "/composedIMTypeParameterSet", method = RequestMethod.GET)
	@ResponseStatus(value = HttpStatus.OK)
	public void resultGroupTypeParameterSet(@RequestParam(value = "index", required = true) int index,
			@RequestParam(value = "typeParameter", required = true) String typeParameter, Model model,
			HttpServletRequest request) {
		try {
			ResultGroup composedIM = (ResultGroup) request.getSession().getAttribute("composedIM");
			ParameterValue parameterValue = new ParameterValue();
			switch (typeParameter) {
			case "REFERENCE":
				parameterValue = new Reference();
				break;
			case "ATTRIBUTE":
				parameterValue.getParameter().setParameterType(EnumParameterType.ATTRIBUTE);
				break;
			default:
			}
			composedIM.getParameterValues().set(index, parameterValue);
			request.getSession().setAttribute("composedIM", composedIM);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Carga en la MIC la operacion seleccionada por el usuario y crea su lista de parametros
	 * de valor.
	 * 
	 * @param operation
	 * @param model
	 * @param request
	 * @throws ExceptionKiuPlus
	 */
	@RequestMapping(value = "/composedIMOperationSelect", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public @ResponseBody Operation composedIMOperationSelect(
			@RequestParam(value = "nameOperation", required = true) String nameOperation, Model model,
			HttpServletRequest request) throws Exception {
		try {
			List<Operation> operations = (ArrayList<Operation>) session.getAttribute("listOperations");
			ResultGroup composedIM = (ResultGroup) request.getSession().getAttribute("composedIM");
			Operation operation = (Operation) operations.stream().filter(o -> o.getName().equals(nameOperation))
					.toArray()[0];

			composedIM.setOperation(operation);
			composedIM.getParameterValues().removeAll(composedIM.getParameterValues());
			for (int i = 0; i < operation.getParameters().size(); i++) {
				composedIM.getParameterValues().add(new ParameterValue());
			}

			request.getSession().setAttribute("composedIM", composedIM);

			return operation;
		} catch (Exception e) {
			throw new ExceptionKiuPlus("The operation was not found");
		}
	}

	// *****************************************//
	// Ejecucion de MIC
	// *****************************************//

	/**
	 * Ejecuta la agregacion y muestra el resultado
	 * 
	 * @param id
	 *            de la agregacion
	 * @param model
	 * @return
	 */
	// @RequestMapping(value = "/qualityModelDataModelExecuteComposedIM", method =
	// RequestMethod.GET)
	// public String qualityModelDataModelExecuteComposedIM(@RequestParam int id, Model model) {
	// try {
	//
	// ResultGroup composedIM = this.resultGroupsService.resultGroupGet(id);
	//
	// // TODO: invocar la ejecucion
	//
	// return "redirect:qualityModelDataModelExecuteComposedIMResultPage?id=" + id;
	// } catch (Exception e) {
	// model.addAttribute("error", e.getMessage());
	// return "qualityModel/qualityModelComposedIMShow";
	// }
	// }

	/**
	 * Muestra el resultado de la ejecucion si ya se ejecuto
	 * 
	 * @param id
	 *            de la agregacion
	 * @param model
	 * @return
	 */
	// @RequestMapping(value = "/qualityModelDataModelExecuteComposedIMResultPage", method =
	// RequestMethod.GET)
	// public String qualityModelDataModelExecuteComposedIMResultPage(@RequestParam int id, Model
	// model) {
	//
	// try {
	//
	// ResultGroup composedIM = this.resultGroupsService.resultGroupGet(id);
	//
	// List<GenericObject> genericObjects = this.executeService.resultExecutionQuery(
	// composedIM.getDataModel().getDatabaseMetadata(), composedIM.getMetadataTableResult());
	// Set<String> resultColumnName = new ConcurrentSkipListSet<>();
	// if (!genericObjects.isEmpty()) {
	// resultColumnName = genericObjects.get(0).getProperties().keySet();
	// }
	//
	// model.addAttribute("resultColumnName", resultColumnName);
	// model.addAttribute("result", genericObjects);
	// model.addAttribute("composedIM", composedIM);
	//
	// return "qualityModel/qualityModelDataModelExecuteComposedIMResult";
	// } catch (Exception e) {
	// e.printStackTrace();
	// model.addAttribute("error", e.getMessage());
	// return "qualityModel/qualityModelComposedIMShow";
	// }
	// }
	//
	// @RequestMapping(value = "/qualityModelComposedIMResultExportPDF", method = RequestMethod.GET)
	// public void qualityModelComposedIMResultExportPDF(@RequestParam int id, HttpServletRequest
	// request,
	// HttpServletResponse response) {
	//
	// try {
	// ResultGroup composedIM = this.resultGroupsService.resultGroupGet(id);
	//
	// List<GenericObject> genericObjects = this.executeService.resultExecutionQuery(
	// composedIM.getDataModel().getDatabaseMetadata(), composedIM.getMetadataTableResult());
	//
	// ByteArrayOutputStream baos =
	// this.resultGroupsService.resultGroupResultExportToPDF(composedIM,
	// genericObjects);
	//
	// // setting some response headers
	// response.setHeader("Expires", "0");
	// response.setHeader("Cache-Control", "must-revalidate, post-check=0, pre-check=0");
	// String fileName = "ComposedIM " + composedIM.getName() + " result of execute.pdf";
	// response.setHeader("Pragma", "public");
	// response.setHeader("Content-disposition", "attachment; filename=" + fileName);
	// // setting the content type
	// response.setContentType("application/pdf");
	// // the contentlength
	// response.setContentLength(((ByteArrayOutputStream) baos).size());
	// // write ByteArrayOutputStream to the ServletOutputStream
	// OutputStream os = response.getOutputStream();
	//
	// baos.writeTo(os);
	// os.flush();
	// os.close();
	//
	// } catch (DocumentException | IOException | ExceptionKiuPlus e) {
	// // TODO Auto-generated catch block
	// e.printStackTrace();
	//
	// }
	// }

	// @RequestMapping(value = "/composedIMGet", method = RequestMethod.GET, produces =
	// MediaType.APPLICATION_JSON_VALUE)
	// public @ResponseBody List<ResultGroup> composedIMsGet(
	// @RequestParam(value = "composedIMIds[]", required = false) Integer[] composedIMIds) throws
	// Exception {
	// try {
	//
	// if (composedIMIds != null) {
	// return this.resultGroupsService.resultGroupsGet(Arrays.asList(composedIMIds));
	// } else {
	// return new ArrayList<>();
	// }
	//
	// } catch (Exception e) {
	// throw new ExceptionMethod("It is not possible to obtain the instanced metrics.");
	// }
	// }

	@ExceptionHandler({ ExceptionMethod.class })
	public ResponseEntity<String> manageMethodException(Exception e) {
		return new ResponseEntity<String>(e.getMessage(), HttpStatus.NOT_FOUND);
	}
}
