package com.project.kiuplus.controller.qualityModel;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.ModelAndView;

import com.kiuplus.configuration.logic.service.DimensionsService;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.AttributeColumn;
import com.kiuplus.logic.model.AttributeDatabase;
import com.kiuplus.logic.model.AttributeTable;
import com.kiuplus.logic.model.DataModel;
import com.kiuplus.logic.model.Dimension;
import com.kiuplus.logic.model.Factor;
import com.kiuplus.logic.model.Method;
import com.kiuplus.logic.model.Metrics;
import com.kiuplus.logic.model.ResultGroup;
import com.kiuplus.logic.model.ResultGroup.EnumResultGroup;
import com.kiuplus.qualityModel.logic.service.DataModelsService;
import com.kiuplus.qualityModel.logic.service.DatabaseService;

@Controller
public class QualityModelController {

	@Autowired
	private DataModelsService dataModelsService;

	@Autowired
	private DatabaseService databaseService;

	@Autowired
	private DimensionsService dimensionsService;


	@RequestMapping("/qualityModel")
	public ModelAndView configurationMethod() {
		ModelAndView mv = new ModelAndView("qualityModel/qualityModel");

		return mv;
	}

	@RequestMapping("/qualityModelMenuSide")
	public ModelAndView qualityModelMenuSide(int idDataModel, String type, int idElement) {
		ModelAndView mv = new ModelAndView("qualityModel/qualityModelMenuSide");

		DataModel dataModel = dataModelsService.dataModelGet(idDataModel);

		try {
			// Database
			AttributeDatabase database = databaseService.databaseGet(dataModel.getDatabaseSource());

			// Para que no entre en un ciclo infinito al enviar el json se seta la base de datos
			// de
			// la tabla y las tablas de las columnas en null
			for (AttributeTable table : database.getAttributesTable()) {
				for (AttributeColumn column : table.getAttributesColumns()) {
					column.setAttributeTable(null);
				}
				table.setAttributeDatabase(null);
				mv.addObject("database", database);
			}
		} catch (ExceptionKiuPlus e) {
			mv.addObject("database", new AttributeDatabase());
			mv.addObject("error", e.getMessage());

		}

		mv.addObject("idDataModel", idDataModel);
		mv.addObject("type", type);
		mv.addObject("idElement", idElement);
		List<Dimension> dimensions = dimensionsService.dimensionsGetAll();

		// Se remueven los elementos que no tienen metricas instanciadas para mostrar en el menu

		Iterator<Dimension> itDimension = dimensions.iterator();
		while (itDimension.hasNext()) {
			Dimension d = itDimension.next();

			Iterator<Factor> itFactor = d.getFactors().iterator();

			while (itFactor.hasNext()) {
				Factor f = itFactor.next();

				Iterator<Metrics> itMetrics = f.getMetrics().iterator();

				while (itMetrics.hasNext()) {

					Metrics me = itMetrics.next();

					Iterator<Method> itMethods = me.getMethods().iterator();

					while (itMethods.hasNext()) {

						Method method = itMethods.next();

						method.getInstancedMetrics().removeIf(im -> im.getDataModel().getId() != idDataModel);

					}

					me.getMethods().removeIf(method -> method.getInstancedMetrics().isEmpty());
				}

				if (f.getName().length() > 38) {
					f.setName(f.getName().substring(0, 38));
				}

				f.getMetrics().removeIf(me -> me.getMethods().isEmpty());
			}

			d.getFactors().removeIf(fa -> fa.getMetrics().isEmpty());
		}

		dimensions.removeIf(d -> d.getFactors().isEmpty());

		mv.addObject("dimensions", dimensions);
		List<ResultGroup> resultGroups = dataModelsService.dataModelGet(idDataModel).getResultGroups();
	
		List<ResultGroup> aggregations = new ArrayList<>();
		List<ResultGroup> composedIMs = new ArrayList<>();
		
		for(ResultGroup rg : resultGroups) {
			if(rg.getType().equals(EnumResultGroup.Aggregation)) {
				aggregations.add(rg);
			}else {
				composedIMs.add(rg);
			}
		}
	
		mv.addObject("aggregations", aggregations);
		mv.addObject("composedIMs", composedIMs);
		return mv;

	}

}
