package com.project.kiuplus.controller.qualityModel;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.util.List;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.kiuplus.configuration.logic.service.DimensionsService;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.AttributeDatabase;
import com.kiuplus.logic.model.AttributeDatabase.EnumDatabaseSystem;
import com.kiuplus.logic.model.DataModel;
import com.kiuplus.logic.model.Dimension;
import com.kiuplus.qualityModel.logic.service.DataModelsService;
import com.kiuplus.qualityModel.logic.service.DatabaseService;

import net.sf.json.JSONException;
import net.sf.json.JSONObject;

//@EnableAsync
@SessionAttributes({ "dataModel" })
@Controller
public class QualityModelDataModelsController {

	@Autowired
	private DataModelsService dataModelsService;

	@Autowired
	private DatabaseService databaseService;

	@Autowired
	private DimensionsService dimensionsService;

	@Autowired
	private Environment enviroment;

	@RequestMapping("/qualityModelDataModelListPage")
	public ModelAndView qualityModelDataModelListPage() {
		ModelAndView mv = new ModelAndView("qualityModel/qualityModelDataModelList");
		mv.addObject("dataModels", dataModelsService.dataModelsGet());
		return mv;
	}

	/**
	 * @return Pagina para visualizar el dataModel
	 */
	@RequestMapping(value = "/qualityModelDataModelShowPage", method = RequestMethod.GET)
	public ModelAndView qualityModelDataModelShowPage(@RequestParam int id, Model model) {

		ModelAndView mv = new ModelAndView("qualityModel/qualityModelDataModelShow");
		try {

			DataModel dataModel = this.dataModelsService.dataModelGet(id);

			ObjectMapper mapper = new ObjectMapper();

			List<Dimension> dimensions = this.dimensionsService.dimensionsDataQualityModelGet(id);
			ObjectNode objectNode1 = mapper.createObjectNode();
			String jsonInString = mapper.writeValueAsString(dimensions);

			objectNode1.put("dimensions", jsonInString);

			mv.addObject("dimensions", objectNode1);

			mv.addObject("dataModel", dataModel);

		} catch (JSONException | JsonProcessingException e) {
			e.printStackTrace();
			mv.addObject("error", e.getMessage());
		}
		return mv;
	}

	@RequestMapping("/qualityModelDataModelCreatePage")
	public ModelAndView qualityModelDataModelCreatePage() {
		ModelAndView mv = new ModelAndView("qualityModel/qualityModelDataModelCreateEdit");

		mv.addObject("dataModel", new DataModel());
		mv.addObject("enumDatabaseSystem", EnumDatabaseSystem.values());
		return mv;
	}

	/**
	 * @return Pagina para editar el dataModel
	 */
	@RequestMapping(value = "/qualityModelDataModelEditPage", method = RequestMethod.GET)
	public ModelAndView qualityModelDataModelEditPage(@RequestParam int id, Model model) {
		ModelAndView mv = new ModelAndView("qualityModel/qualityModelDataModelCreateEdit");

		DataModel dataModel = this.dataModelsService.dataModelGet(id);
		mv.addObject("dataModel", dataModel);
		mv.addObject("enumDatabaseSystem", EnumDatabaseSystem.values());
		mv.addObject("action", "edit");
		return mv;
	}

	/**
	 * Valida la conexion a la base de datos y crea o edita el modelo de calidad y la database segun
	 * corresponda
	 * 
	 * @param dataModel
	 * @param result
	 * @param model
	 * @param redirectAttributes
	 * @return si se creo exitosamente pagina para mostrar la database Si fallo la pagina de
	 *         editar/crear para mostrar el error
	 */
	@RequestMapping(value = "/qualityModelDataModelCreateEdit", method = RequestMethod.POST)
	public String qualityModelDataModelCreateEdit(@ModelAttribute("dataModel") @Valid DataModel dataModel,
			BindingResult result, Model model, RedirectAttributes redirectAttributes) {
		try {

			if (result.hasErrors()) {
				return "qualityModel/qualityModelDataModelCreateEdit";
			}

			// Valida la conexion a la base de datos
			try {
				this.databaseService.dataBaseTestConnection(dataModel.getDatabaseSource());

			} catch (Exception e) {
				model.addAttribute("errorValidation", e.getMessage());
				return "qualityModel/qualityModelDataModelCreateEdit";
			}

			/// Agregar database
			if (dataModel.getId() == 0) {

				// Agrega los datos de la base de metadatos obtenidos del archivo de configuracion

				String user = enviroment.getProperty("metadata.database.username");
				if (user == null || user.isEmpty()) {
					throw new ExceptionKiuPlus(
							"Set the user for the metadata base in kiuplus.properties. Enter property metadata.database.username.");
				}

				String password = enviroment.getProperty("metadata.database.password");
				if (password == null || password.isEmpty()) {
					throw new ExceptionKiuPlus(
							"Set the password for the metadata base in kiuplus.properties. Enter property metadata.database.password.");
				}

				String url = enviroment.getProperty("metadata.database.url");
				if (url == null || url.isEmpty()) {
					throw new ExceptionKiuPlus(
							"Set the url for the metadata base in kiuplus.properties. Enter property metadata.database.url.");
				}

				// String systemString = enviroment.getProperty("metadata.database.system");
				// EnumDatabaseSystem system;
				// if (systemString == null || systemString.isEmpty()) {
				// throw new ExceptionKiuPlus(
				// "Set the system for the metadata base in kiuplus.properties. Enter property
				// metadata.database.system. Values ​​can be MySql or PostgreSQL");
				// } else {
				// try {
				// system = EnumDatabaseSystem.valueOf(systemString);
				// } catch (Exception e) {
				// throw new ExceptionKiuPlus(
				// "Set the system for the metadata base in kiuplus.properties. Enter property
				// metadata.database.system. Values ​​can be MySql or PostgreSQL");
				// }
				// }
				//
				String name = enviroment.getProperty("metadata.database.name");
				if (name == null || name.isEmpty()) {
					throw new ExceptionKiuPlus(
							"Set the name for the metadata base in kiuplus.properties. Enter property metadata.database.name.");
				}
				
				String schema = enviroment.getProperty("metadata.database.schema");
				if (schema == null || schema.isEmpty()) {
					throw new ExceptionKiuPlus(
							"Set the schema for the metadata base in kiuplus.properties. Enter property metadata.database.schema.");
				}

				AttributeDatabase databaseMetadata = new AttributeDatabase(name, url, user, password, schema,
						EnumDatabaseSystem.MySQL);
				// dataModel.getDatabaseMetadata().setPassword(password);
				// dataModel.getDatabaseMetadata().setUser(user);
				// dataModel.getDatabaseMetadata().setUrl(url);
				// dataModel.getDatabaseMetadata().setSystem(EnumDatabaseSystem.MySQL);
				// dataModel.getDatabaseMetadata().setName(name);
				dataModel.setDatabaseMetadata(databaseMetadata);
				dataModelsService.dataModelCreate(dataModel);
			} else {
				dataModel.getDatabaseMetadata().setPassword("admin");
				dataModelsService.dataModelEdit(dataModel);
			}
			model.addAttribute("message", "Data model successfully saved.");

			return "redirect:qualityModelDataModelShowPage?id=" + dataModel.getId();

		} catch (Exception ex) {
			model.addAttribute("error", ex.getMessage());
			return "qualityModel/qualityModelDataModelCreateEdit";
		}
	}

	/**
	 * Elimina el data model
	 * 
	 * @param id
	 * @param redirectAttributes
	 * @return Pagina listando los data model
	 */
	@RequestMapping(value = "/qualityModelDataModelDeletePage", method = RequestMethod.GET)
	public String qualityModelDataModelDelete(@RequestParam int id, Model model,
			RedirectAttributes redirectAttributes) {

		try {

			this.dataModelsService.dataModelDelete(id);
			redirectAttributes.addFlashAttribute("message", "Data model deleted successfully.");

			return "redirect:qualityModelDataModelListPage";

		} catch (Exception ex) {
			redirectAttributes.addFlashAttribute("error", ex.getMessage());
			return "redirect:qualityModelDataModelListPage";

		}
	}

	/**
	 * Verifica el string de conexion
	 * 
	 * @param url
	 * @param user
	 * @param password
	 * @param model
	 * @return
	 */
	@RequestMapping(value = "/qualityModelConnectDatabaseTestConnection", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public @ResponseBody JSONObject qualityModelConnectDatabaseTestConnection(
			@RequestParam(value = "name", required = true) String name,
			@RequestParam(value = "url", required = true) String url,
			@RequestParam(value = "user", required = true) String user,
			@RequestParam(value = "password", required = true) String password,
			@RequestParam(value = "schema", required = true) String schema,
			@RequestParam(value = "system", required = true) String system, Model model) {

		try {

			EnumDatabaseSystem systemEnum = EnumDatabaseSystem.valueOf(system);
			if (this.databaseService
					.dataBaseTestConnection(new AttributeDatabase(0, name, url, user, password, schema, systemEnum))) {
				JSONObject json = new JSONObject();
				json.put("ok", true);
				json.put("response", "Test connection succeeded");

				return json;
			} else {
				throw new Exception("Ha ocurrido un error");
			}

		} catch (Exception e) {
			JSONObject json = new JSONObject();
			json.accumulate("ok", false);
			json.accumulate("response", e.getMessage());

			return json;
		}
	}

	// **************************************************//
	// Export to PDF
	// **************************************************//

	@RequestMapping(value = "/qualityModelDataModelExportPDF", method = RequestMethod.GET)
	public void qualityModelDataModelExportPDF(@RequestParam int idDataModel, @RequestParam boolean download,
			HttpServletRequest request, HttpServletResponse response) {

		try {
			DataModel dataModel = this.dataModelsService.dataModelGet(idDataModel);

			String path = enviroment.getProperty("images.source");

			ByteArrayOutputStream baos = this.dataModelsService.dataQualityModelExportToPDF(dataModel, path);
			// Establece los cabezales de la respuesta
			response.setHeader("Expires", "0");
			response.setHeader("Cache-Control", "must-revalidate, post-check=0, pre-check=0");
			String fileName = "Data model " + dataModel.getName() + ".pdf";
			response.setHeader("Pragma", "public");
			if (download) {
				response.setHeader("Content-disposition", "attachment; filename=" + fileName);
			} else {
				response.setHeader("Content-disposition", "inline; filename=" + fileName);
			}

			response.setContentType("application/pdf");

			response.setContentLength(((ByteArrayOutputStream) baos).size());

			OutputStream os = response.getOutputStream();

			baos.writeTo(os);
			os.flush();
			os.close();

		} catch (Exception e) {
			e.printStackTrace();

		}

	}

}
