package com.project.kiuplus.controller.qualityModel;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

import org.json.simple.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.kiuplus.configuration.logic.service.DimensionsService;
import com.kiuplus.configuration.logic.service.FactorsService;
import com.kiuplus.configuration.logic.service.MethodsService;
import com.kiuplus.configuration.logic.service.MetricsService;
import com.kiuplus.logic.exception.ExceptionKiuPlus;
import com.kiuplus.logic.model.Attribute;
import com.kiuplus.logic.model.Attribute.EnumAttributeType;
import com.kiuplus.logic.model.AttributeColumn;
import com.kiuplus.logic.model.AttributeDatabase;
import com.kiuplus.logic.model.AttributeTable;
import com.kiuplus.logic.model.DataModel;
import com.kiuplus.logic.model.Dimension;
import com.kiuplus.logic.model.Factor;
import com.kiuplus.logic.model.InstancedMetric;
import com.kiuplus.logic.model.Method;
import com.kiuplus.logic.model.Metrics;
import com.kiuplus.logic.model.Metrics.EnumGranularity;
import com.kiuplus.logic.model.Parameter;
import com.kiuplus.logic.model.Parameter.EnumDataType;
import com.kiuplus.logic.model.Parameter.EnumParameterType;
import com.kiuplus.logic.model.ParameterAttribute;
import com.kiuplus.logic.model.ParameterValue;
import com.kiuplus.logic.model.Reference;
import com.kiuplus.qualityModel.logic.service.ResultGroupsService;
import com.kiuplus.qualityModel.logic.service.DataModelsService;
import com.kiuplus.qualityModel.logic.service.DatabaseService;
import com.kiuplus.qualityModel.logic.service.ExecuteService;
import com.kiuplus.qualityModel.logic.service.InstancedMetricsService;
import com.project.kiuplus.exceptions.ExceptionMethod;

@SessionAttributes({ "instancedMetric", "dataModel" })
@Controller
public class QualityModelInstancedMetricsController {

	@Autowired
	private InstancedMetricsService instancedMetricsService;

	@Autowired
	private DataModelsService dataModelsService;

	@Autowired
	private DimensionsService dimensionsService;

	@Autowired
	private FactorsService factorsService;

	@Autowired
	private MetricsService metricsService;

	@Autowired
	private MethodsService methodsService;

	@Autowired
	private DatabaseService databaseService;

	@Autowired
	private ResultGroupsService resultGroupsService;

	@Autowired
	private ExecuteService executeService;

	/**
	 * @return Pagina para visualizar la metrica instamciada
	 */
	@RequestMapping(value = "/qualityModelInstancedMetricShowPage", method = RequestMethod.GET)
	public ModelAndView qualityModelInstancedMetricShowPage(@RequestParam int id, Model model) {
		ModelAndView mv = new ModelAndView("qualityModel/qualityModelInstancedMetricShow");

		InstancedMetric instancedMetric = this.instancedMetricsService.instancedMetricGet(id);
		// instancedMetric.getParameterValues()
		// .sort((p1, p2) -> p1.getParameter().getName().compareTo(p2.getParameter().getName()));
		mv.addObject("instancedMetric", instancedMetric);

		return mv;

	}

	@RequestMapping(value = "/qualityModelInstancedMetricCreatePage", method = RequestMethod.GET)
	public ModelAndView qualityModelInstancedMetricCreatePage(@RequestParam int idDataModel) {
		ModelAndView mv = new ModelAndView("qualityModel/qualityModelInstancedMetricCreateEdit");

		DataModel dataModel = this.dataModelsService.dataModelGet(idDataModel);

		List<Dimension> dimensions = this.dimensionsService.dimensionsGetAll();
		InstancedMetric im = new InstancedMetric(dataModel);
		mv.addObject("instancedMetric", im);
		mv.addObject("dimensions", dimensions);
		mv.addObject("factors", new ArrayList<Factor>());
		mv.addObject("metrics", new ArrayList<Metrics>());
		mv.addObject("methods", new ArrayList<Method>());
		mv.addObject("hasAggregations", false);
		return mv;
	}

	@RequestMapping(value = "/qualityModelInstancedMetricEditPage", method = RequestMethod.GET)
	public String qualityModelInstancedMetricEditPage(@ModelAttribute DataModel dataModel,
			@RequestParam int idInstancedMetrics, Model md, BindingResult result) {

		try {
			// Carga la metrica instanciada a editar
			InstancedMetric instancedMetric = this.instancedMetricsService.instancedMetricGet(idInstancedMetrics);

			// Si la MI tiene dos attributos que son iguales entonces al modificar uno se modifica
			// el otro, para que esto no suceda creo una copia.
			for (ParameterValue parameterValue : instancedMetric.getParameterValues()) {

				if (parameterValue.getClass().equals(ParameterAttribute.class)) {
					ParameterAttribute parameterAttribute = (ParameterAttribute) parameterValue;
					Attribute attribute = parameterAttribute.getAttribute();
					if (attribute.getClass().equals(AttributeColumn.class)) {

						AttributeColumn attributeOriginal = (AttributeColumn) attribute;
						AttributeTable attributeTable = new AttributeTable(0,
								attributeOriginal.getAttributeTable().getName());
						Attribute attributeColumn = new AttributeColumn(0, attributeOriginal.getName(),
								attributeOriginal.getType(), attributeTable);
						parameterAttribute.setAttribute(attributeColumn);
					}

					if (attribute.getClass().equals(AttributeTable.class)) {
						AttributeTable attributeTable = new AttributeTable(0, attribute.getName());
						parameterAttribute.setAttribute(attributeTable);
					}

				}
			}

			// instancedMetric.getParameterValues()
			// .sort((p1, p2) ->
			// p1.getParameter().getName().compareTo(p2.getParameter().getName()));
			md.addAttribute("instancedMetric", instancedMetric);

			// Carga las dimensiones, factores, metricas, metodos
			List<Dimension> dimensions = this.dimensionsService.dimensionsGetAll();

			md.addAttribute("dimensions", dimensions);
			md.addAttribute("factors",
					instancedMetric.getMethod().getMetrics().getFactor().getDimension().getFactors());
			md.addAttribute("metrics", instancedMetric.getMethod().getMetrics().getFactor().getMetrics());
			md.addAttribute("methods", instancedMetric.getMethod().getMetrics().getMethods());

			// Attributo que indica si tiene agregaciones
			boolean hasAggregations = this.resultGroupsService.instancedMetricHasResultGroups(idInstancedMetrics);
			md.addAttribute("hasAggregations", hasAggregations);

			// Carga las tablas
			List<AttributeTable> tables = this.databaseService
					.tablesColumnsGet(instancedMetric.getDataModel().getDatabaseSource());
			md.addAttribute("tables", tables);

			return "qualityModel/qualityModelInstancedMetricCreateEdit";
		} catch (ExceptionKiuPlus e) {
			// result.rejectValue("error", e.getMessage());
			md.addAttribute("error", e.getMessage());

			return "qualityModel/qualityModelDataModelShow";
		}
	}

	@RequestMapping(value = "/qualityModelInstancedMetricCreateEdit", method = RequestMethod.POST)
	public String qualityModelInstancedMetricCreateEdit(
			@ModelAttribute("instancedMetric") @Valid InstancedMetric instancedMetric, BindingResult result,
			Model model, RedirectAttributes redirectAttributes) {

		try {

			// Verifica errores
			this.qualityModelInstancedMetricCreateEditValidar(instancedMetric, result, model);

			if (result.hasErrors()) {
				this.qualityModelInstancedMetricCreateEditLoadAttributes(model, instancedMetric);

				return "qualityModel/qualityModelInstancedMetricCreateEdit";
			}

			Method method = this.methodsService.methodGetFull(instancedMetric.getMethod().getId());

			// Obtiene los datos de la base de datos que ya tengo guardado
			AttributeDatabase database = this.dataModelsService.dataModelGet(instancedMetric.getDataModel().getId())
					.getDatabaseSource();

			instancedMetric.setMethod(method);

			// Recorre los parametros y obtiene las tablas y columnas de cada attributo asociado al
			// parametro
			for (ParameterValue parameterValue : instancedMetric.getParameterValues()) {

				Parameter parameter = (Parameter) method.getOperation()
						.getParameter(parameterValue.getParameter().getName());

				if (parameterValue.getClass().equals(ParameterAttribute.class)) {
					ParameterAttribute parameterAttribute = (ParameterAttribute) parameterValue;

					// Si selecciona la misma tabla varias se crea una sola y se asocia la misma
					// tabla a varias columnas

					// Crea el atributo de la metrica instanciada
					Attribute attribute = new Attribute();
					if (parameterAttribute.getAttribute().getClass().equals(AttributeTable.class)) {

						AttributeTable attributeT = database
								.findAttributeTable(parameterAttribute.getAttribute().getName());
						if (attributeT == null) {
							attributeT = new AttributeTable(parameterAttribute.getAttribute().getName(), database);
							database.getAttributesTable().add(attributeT);
						}

						attribute = attributeT;
					} else {
						if (parameterAttribute.getAttribute().getClass().equals(AttributeColumn.class)) {
							AttributeColumn a = (AttributeColumn) parameterAttribute.getAttribute();
							AttributeTable attributeTable = database
									.findAttributeTable(a.getAttributeTable().getName());

							if (attributeTable == null) {
								attributeTable = new AttributeTable(a.getAttributeTable().getName(), database);
								database.getAttributesTable().add(attributeTable);
							}

							AttributeColumn attributeColumn = attributeTable.findAttributeColumn(a.getName());

							if (attributeColumn == null) {
								// busca la columna en la base original porque no viene el tipo
								// desde la interfaz
								attributeColumn = this.databaseService.columnGet(a.getName(), attributeTable.getName(),
										attributeTable.getAttributeDatabase());
								attributeColumn.setAttributeTable(attributeTable);
								attributeTable.getAttributesColumns().add(attributeColumn);

							}
							attribute = attributeColumn;

						} else {

							if (parameterAttribute.getAttribute().getClass().equals(AttributeDatabase.class)) {
								attribute = database;
							}
						}
					}
					parameterAttribute.setAttribute(attribute);
				}

				parameterValue.setParameter(parameter);

			}

			// Agregar metrica instanciada
			if (instancedMetric.getId() == 0) {

				this.instancedMetricsService.instancedMetricCreate(instancedMetric);

			} else {

				if (instancedMetric.getDateTimeResult() != null) {
					this.executeService.deleteResultExecutionInstancedMetric(instancedMetric);
					instancedMetric.setDateTimeResult(null);
					instancedMetric.setMetadataTableResult(null);
				}

				this.instancedMetricsService.instancedMetricEdit(instancedMetric);

			}

			return "qualityModel/qualityModelInstancedMetricShow";
		} catch (Exception ex) {
			ex.printStackTrace();
			model.addAttribute("error", ex.getMessage());
			try {
				this.qualityModelInstancedMetricCreateEditLoadAttributes(model, instancedMetric);
			} catch (ExceptionKiuPlus e) {
				model.addAttribute("error", e.getMessage());
			}

			return "qualityModel/qualityModelInstancedMetricCreateEdit";

		}
	}

	/**
	 * Valida los campos ingresados por el usuario No se controla si se conecta a la base de datos,
	 * ya que lo controla cuando se crea el metodo. Controla que el tipo de dato cuando es una
	 * atributo coincida con el del parametro.
	 * 
	 * @param instancedMetric
	 * @param result
	 * @throws ExceptionKiuPlus
	 */
	private void qualityModelInstancedMetricCreateEditValidar(InstancedMetric instancedMetric, BindingResult result,
			Model model) throws ExceptionKiuPlus {

		if (instancedMetric.getMethod().getId() == 0) {
			result.rejectValue("method", "blanckValue", new Object[] { "'method'" }, "Select a method");
		}

		if (instancedMetric.getMethod().getMetrics().getId() == 0) {
			result.rejectValue("method.metrics", "blanckValue", new Object[] { "'metrics'" }, "Select a metrics");
		}

		if (instancedMetric.getMethod().getMetrics().getFactor().getId() == 0) {
			result.rejectValue("method.metrics.factor", "blanckValue", new Object[] { "'factor'" }, "Select a factor");
		}

		if (instancedMetric.getMethod().getMetrics().getFactor().getDimension().getId() == 0) {
			result.rejectValue("method.metrics.factor.dimension", "blanckValue", new Object[] { "'dimension'" },
					"Select a dimension");
		}

		AttributeDatabase database = this.dataModelsService.dataModelGet(instancedMetric.getDataModel().getId())
				.getDatabaseSource();

		// Controla que se seleccione una columna
		// Si el parametro es del tipo atributo controla que sea del tipo que corresponde
		// si el parametro es del tipo referencia controla que se ingrese el valor de referencia y
		// que tenga el formato del parametro
		for (ParameterValue parameterValue : instancedMetric.getParameterValues()) {
			if (parameterValue.getClass().equals(ParameterAttribute.class)) {
				Attribute a = ((ParameterAttribute) parameterValue).getAttribute();
				if (a.getName().equals("")) {

					result.rejectValue("parameterValues", "blanckValue", new Object[] { "'parameterValues'" },
							"Select the attribute for " + parameterValue.getParameter().getName());
				} else {
					if (a.getClass().equals(AttributeColumn.class)) {
						if (parameterValue.getParameter().getParameterType().equals(EnumParameterType.ATTRIBUTE)) {
							AttributeColumn attributeColumn = (AttributeColumn) a;
							AttributeColumn attributeColumnOriginal = this.databaseService.columnGet(
									attributeColumn.getName(), attributeColumn.getAttributeTable().getName(), database);

							// Por el tema de que el web service no acepta las fechas como vienen
							// desde la base de datos,
							// se permite que el parametro del web service sea del tipo string y ahi
							// se realize la conversion
							if (!((attributeColumnOriginal.getType().equals(EnumDataType.DATETIME)
									&& parameterValue.getParameter().getDataType().equals(EnumDataType.STRING))
									|| attributeColumnOriginal.getType()
											.equals(parameterValue.getParameter().getDataType()))) {
								if (parameterValue.getParameter().getDataType().equals(EnumDataType.STRING)) {
									result.rejectValue("parameterValues", "blanckValue",
											new Object[] { "'parameterValues'" },
											"The attribute  " + parameterValue.getParameter().getName() + " must be "
													+ parameterValue.getParameter().getDataType().getText() + " or " + EnumDataType.DATETIME.getText());
								} else {
									result.rejectValue("parameterValues", "blanckValue",
											new Object[] { "'parameterValues'" },
											"The attribute  " + parameterValue.getParameter().getName() + " must be "
													+ parameterValue.getParameter().getDataType().getText());
								}
							}
						}
					}
				}
			}

			if (parameterValue.getClass().equals(Reference.class)) {
				Reference reference = (Reference) parameterValue;
				if (reference.getReferenceValue() == null || reference.getReferenceValue().isEmpty()) {
					result.rejectValue("parameterValues", "blanckValue", new Object[] { "'parameterValues'" },
							"Enter the value for " + parameterValue.getParameter().getName());

				} else {
					String referenceValue = reference.getReferenceValue();
					try {
						switch (parameterValue.getParameter().getDataType()) {
						case BOOL:
							if (!(referenceValue.toUpperCase().trim().equals("TRUE")
									|| referenceValue.toUpperCase().trim().equals("FALSE"))) {
								result.rejectValue("parameterValues", "blanckValue",
										new Object[] { "'parameterValues'" }, "The reference value of "
												+ parameterValue.getParameter().getName() + " must be true or false");
							}
							break;
						case DOUBLE:
							Double.parseDouble(referenceValue);
							break;
						case INTEGER:
							Integer.parseInt(referenceValue);
							break;
						case DATETIME:
							try {
								SimpleDateFormat parser = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
								parser.parse(referenceValue);
							} catch (Exception e) {
								result.rejectValue("parameterValues", "blanckValue",
										new Object[] { "'parameterValues'" },
										"The reference value of " + parameterValue.getParameter().getName()
												+ " must be yyyy-MM-dd HH:mm:ss");
							}
							break;
						default:
							break;
						}
					} catch (Exception e) {
						result.rejectValue("parameterValues", "blanckValue", new Object[] { "'parameterValues'" },
								"The reference value of " + parameterValue.getParameter().getName() + " must be "
										+ parameterValue.getParameter().getDataType().getText());
					}
				}
			}
		}

		// Si es tupla se controla que todas las columnas de la tabla esten asociadas a un parametro
		// de la operacion, no se repitan y no se seleccione otra tabla
		if (instancedMetric.getMethod().getMetrics().getGranularity().equals(EnumGranularity.Tuple)
				|| instancedMetric.getMethod().getMetrics().getGranularity().equals(EnumGranularity.CellSet)) {

			int attributecount = 0;
			String tableName = "";
			boolean error = false;
			List<String> columnsName = new ArrayList<String>();
			ParameterAttribute pa = null;

			for (ParameterValue parameterValue : instancedMetric.getParameterValues()) {
				if (parameterValue.getClass().equals(ParameterAttribute.class)) {
					pa = (ParameterAttribute) parameterValue;
					AttributeColumn a = (AttributeColumn) pa.getAttribute();
					// controla que sea la misma tabla si es tupla oconjunto de celdas
					if (tableName.isEmpty()) {
						tableName = a.getAttributeTable().getName();
					} else {
						if (!tableName.equals(a.getAttributeTable().getName())
								&& !a.getAttributeTable().getName().isEmpty()) {
							error = true;
						}
					}

					// Controla que no se repitan las columnas si es tupla
					if (instancedMetric.getMethod().getMetrics().getGranularity().equals(EnumGranularity.Tuple)
							&& !a.getName().isEmpty()) {
						if (!columnsName.contains(a.getName())) {
							columnsName.add(a.getName());
						} else {
							result.rejectValue("parameterValues", "blanckValue", new Object[] { "'parameterValues'" },
									"Columns can not be repeated");
							break;
						}
					}

					attributecount++;
				}
			}

			if (error) {
				result.rejectValue("parameterValues", "blanckValue", new Object[] { "'parameterValues'" },
						"Select the same table for all parameters");
			} else {
				// controla que todos los atributos esten asociados a un parametro cuando la
				// granularidad es tupla
				if (instancedMetric.getMethod().getMetrics().getGranularity().equals(EnumGranularity.Tuple)
						&& !instancedMetric.getParameterValues().isEmpty()) {

					if (pa != null) {

						AttributeTable attributeTable = ((AttributeColumn) pa.getAttribute()).getAttributeTable();
						try {
							int columnsCount = this.databaseService.columnsGet(database, attributeTable).size();

							if (columnsCount != attributecount) {
								result.rejectValue("parameterValues", "blanckValue",
										new Object[] { "'parameterValues'" },
										"Select the parameters for all the attributes of the table");
							}
						} catch (ExceptionKiuPlus e) {
							model.addAttribute("error", e.getMessage());
						}

					}
				}
			}
		}

	}

	/**
	 * Carga los atributos necesarios para mostrar en la pagina
	 * 
	 * @param model
	 * @param instancedMetric
	 * @throws ExceptionKiuPlus
	 */
	private void qualityModelInstancedMetricCreateEditLoadAttributes(Model model, InstancedMetric instancedMetric)
			throws ExceptionKiuPlus {

		// Attributo que indica si tiene agregaciones
		model.addAttribute("hasAggregations",
				this.resultGroupsService.instancedMetricHasResultGroups(instancedMetric.getId()));

		// Carga las dimensiones, factores, metricas, metodos
		List<Dimension> dimensions = this.dimensionsService.dimensionsGetAll();
		model.addAttribute("dimensions", dimensions);

		Dimension dimension = this.dimensionsService
				.dimensionFindFullDById(instancedMetric.getMethod().getMetrics().getFactor().getDimension().getId());
		if (dimension != null) {
			List<Factor> factors = dimension.getFactors();
			model.addAttribute("factors", factors);

			List<Metrics> metrics = this.metricsService
					.metricsGetAll(instancedMetric.getMethod().getMetrics().getFactor().getId());
			model.addAttribute("metrics", metrics);

			List<Method> methods = this.methodsService.methodsGetAll(instancedMetric.getMethod().getMetrics().getId());
			model.addAttribute("methods", methods);

			// int cantParametros = 0;

			// Si el metodo esta seleccionado cargo las tablas
			if (instancedMetric.getMethod().getId() != 0) {

				Method method = this.methodsService.methodGetFull(instancedMetric.getMethod().getId());
				instancedMetric.setMethod(method);

				// for (Parameter p : method.getOperation().getParameters()) {
				// if (p.getClass().equals(ParameterAttribute.class)) {
				// cantParametros++;
				// }
				// }

				List<AttributeTable> tables = this.databaseService
						.tablesColumnsGet(instancedMetric.getDataModel().getDatabaseSource());
				model.addAttribute("tables", tables);
			} else {
				instancedMetric.setParameterValues(new ArrayList<ParameterValue>());
			}

		} else {
			// Si no hay nada seleccionado cargo las listas vacias
			model.addAttribute("factors", new ArrayList<Factor>());
			model.addAttribute("metrics", new ArrayList<Metrics>());
			model.addAttribute("methods", new ArrayList<Method>());
			model.addAttribute("hasAggregations", false);
		}
	}

	@RequestMapping(value = "/qualityModelInstancedMetricDeletePage", method = RequestMethod.GET)
	public String qualityModelInstancedMetricDeletePage(@RequestParam int id, Model model,
			RedirectAttributes redirectAttributes) {
		InstancedMetric im = this.instancedMetricsService.instancedMetricGet(id);
		try {

			if (im.getDateTimeResult() != null) {
				this.executeService.deleteResultExecutionInstancedMetric(im);
			}
			this.instancedMetricsService.instancedMetricDelete(id);
			redirectAttributes.addFlashAttribute("message", "Instanced metric deleted successfully.");

			return "redirect:qualityModelDataModelShowPage?id=" + im.getDataModel().getId();

		} catch (Exception ex) {
			redirectAttributes.addFlashAttribute("error", ex.getMessage());
			String retorno = "redirect:qualityModelInstancedMetricShowPage?id=" + id;
			return retorno;
		}
	}

	/**
	 * Obtiene los factores de la dimension
	 * 
	 * @param idDimension
	 * @param model
	 * @return Los parametros basicos de los factores
	 * @throws Exception
	 */
	@RequestMapping(value = "/factorsGet", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public @ResponseBody List<Factor> factorsGet(@RequestParam(value = "idDimension", required = true) int idDimension,
			Model model) throws Exception {

		try {

			List<Factor> factors = this.factorsService.factorsGetAll(idDimension);
			// List<Factor> factorsAux = new ArrayList<Factor>();
			// for (Factor f : factors) {
			// factorsAux.add(new Factor(f.getId(), f.getName()));
			// }

			// List<Factor> factorsAux = new ArrayList<Factor>();
			return factors;
		} catch (Exception e) {
			throw new ExceptionMethod("It is not possible to obtain the factors.");
		}
	}

	/**
	 * Obtiene las metricas de los factores
	 * 
	 * @param idFactor
	 * @param model
	 * @return Parametros basicos de las metricas
	 * @throws Exception
	 */
	@RequestMapping(value = "/metricsGet", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public @ResponseBody List<Metrics> metricsGet(@RequestParam(value = "idFactor", required = true) int idFactor,
			Model model) throws Exception {

		try {

			List<Metrics> metrics = this.metricsService.metricsGetAll(idFactor);
			// ArrayList<Metrics> metricsAux = new ArrayList<Metrics>();
			// for (Metrics m : metrics) {
			// metricsAux.add(new Metrics(m.getId(), m.getName(), m.getGranularity()));
			// }
			return metrics;
		} catch (Exception e) {
			throw new ExceptionMethod("It is not possible to obtain the metrics.");
		}
	}

	/**
	 * Obtiene los metodos de la metrica
	 * 
	 * @param idMetrics
	 * @param model
	 * @return Parametros basicos de las metricas
	 * @throws Exception
	 */
	@RequestMapping(value = "/methodsGet", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public @ResponseBody JSONObject methodsGet(@RequestParam(value = "idMetrics", required = true) int idMetrics,
			Model model) throws Exception {

		try {

			List<Method> methods = this.methodsService.methodsGetAll(idMetrics);
			// List<Method> methodsAux = new ArrayList<Method>();
			// for (Method m : methods) {
			// methodsAux.add(new Method(m.getId(), m.getName(), new Metrics(m.getMetrics().getId(),
			// m.getMetrics().getName(), m.getMetrics().getGranularity())));
			// }

			JSONObject json = new JSONObject();
			json.put("methods", methods);
			Metrics metrics = this.metricsService.metricsGet(idMetrics);
			json.put("metrics", metrics);

			return json;
		} catch (Exception e) {
			throw new ExceptionMethod("It is not possible to obtain the methods.");
		}
	}

	/**
	 * Obtiene el metodo
	 * 
	 * @param idMetrics
	 * @param model
	 * @return Parametros basicos del metodo
	 * @throws Exception
	 */
	// public Method methodGet(int idMethod) throws Exception {
	//
	// try {
	// Method method = this.methodsService.methodGet(idMethod);
	// // List<Parameter> parameters =
	// // method.getOperation().getParameters();
	// // Metrics metrics = new Metrics(method.getMetrics().getId(),
	// // method.getMetrics().getName(),
	// // method.getMetrics().getGranularity());
	// // return new Method(method.getId(), method.getName(), method.getOperation(), metrics);
	//
	// return method;
	// } catch (Exception e) {
	// throw new ExceptionMethod("It is not possible to obtain the method.");
	// }
	// }

	/**
	 * Obtiene las tablas de la base de datos source y los parametros de la operacion asociada al
	 * metodo
	 * 
	 * @param idDataModel
	 * @param model
	 * @param idMethod
	 * @return Nombre de las tablas de la base de datos
	 * @throws Exception
	 */
	@SuppressWarnings("unchecked")
	@RequestMapping(value = "/methodParametersGet", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public @ResponseBody JSONObject methodParametersGet(@RequestParam(value = "idMethod", required = true) int idMethod,
			@RequestParam(value = "idDataModel", required = true) int idDataModel, Model model,
			HttpServletRequest request) throws Exception {

		try {
			AttributeDatabase database = this.dataModelsService.dataModelGet(idDataModel).getDatabaseSource();
			JSONObject json = new JSONObject();

			Method method = this.methodsService.methodGet(idMethod);
			json.put("method", method);
			json.put("webService", method.getOperation().getWebService());
			json.put("typeAttribute", method.getMetrics().getTypeAttribute());

			List<AttributeTable> tables = this.databaseService.tablesFind(database);

			json.put("tables", tables);

			// Crea los parameters values del tipo que corresponda para hacer el bean en la vista
			InstancedMetric instancedMetric = (InstancedMetric) request.getSession().getAttribute("instancedMetric");
			instancedMetric.getParameterValues().removeAll(instancedMetric.getParameterValues());

			// method.getOperation().getParameters().sort((p1, p2) ->
			// p1.getName().compareTo(p2.getName()));

			for (Parameter parameter : method.getOperation().getParameters()) {

				switch (parameter.getParameterType()) {
				case ATTRIBUTE:
					Attribute attribute = new Attribute();
					switch (method.getMetrics().getTypeAttribute()) {
					case COLUMNS:
						attribute = new AttributeColumn();
						break;
					case TABLES:
						attribute = new AttributeTable();
						break;
					default:
					}
					instancedMetric.getParameterValues().add(new ParameterAttribute(attribute));
					break;
				case COLUMN_NAME:
					instancedMetric.getParameterValues().add(new ParameterAttribute(new AttributeColumn()));
					break;
				case TABLE_NAME:
					instancedMetric.getParameterValues().add(new ParameterAttribute(new AttributeTable()));
					break;
				case REFERENCE:
					instancedMetric.getParameterValues().add(new Reference());
					break;
				default:
					instancedMetric.getParameterValues().add(new ParameterAttribute(database));

				}
			}

			request.getSession().setAttribute("instancedMetric", instancedMetric);
			return json;

		} catch (Exception e) {
			e.printStackTrace();
			throw new ExceptionKiuPlus(e.getMessage());
		}
	}

	/**
	 * Obtiene las columnas de la tabla
	 * 
	 * @param idDataModel
	 * @param model
	 * @return Nombre de las tablas de la base de datos
	 * @throws Exception
	 */
	@RequestMapping(value = "/columnsFind", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public @ResponseBody List<AttributeColumn> columnsFind(
			@RequestParam(value = "tableName", required = true) String tableName,
			@RequestParam(value = "idDataModel", required = true) int idDataModel, Model model) throws Exception {

		try {

			AttributeDatabase database = this.dataModelsService.dataModelGet(idDataModel).getDatabaseSource();
			AttributeTable attributeTable = new AttributeTable(0, tableName);
			return this.databaseService.columnsGet(database, attributeTable);

		} catch (Exception e) {
			throw new ExceptionMethod("It is not possible to obtain the tables.");
		}
	}

	// @RequestMapping(value = "/instancedMetricsGet", method = RequestMethod.GET, produces =
	// MediaType.APPLICATION_JSON_VALUE)
	// public @ResponseBody List<InstancedMetric> instancedMetricsGet(
	// @RequestParam(value = "idsInstancedMetrics[]", required = false) Integer[]
	// idsInstancedMetrics)
	// throws Exception {
	// try {
	//
	// if (idsInstancedMetrics != null) {
	// return this.instancedMetricsService.instancedMetricsGet(Arrays.asList(idsInstancedMetrics));
	// } else {
	// return new ArrayList<>();
	// }
	//
	// } catch (Exception e) {
	// throw new ExceptionMethod("It is not possible to obtain the instanced metrics.");
	// }
	// }

	// Captura las excepciones del tipo ExceptionMethod para enviarlas a la
	// llamada ajax

	/**
	 * Retorna las metricas instanciadas del modelo de datos
	 * 
	 * @param idDataModel
	 * @return Las metricas instanciadas del Data model
	 * @throws ExceptionKiuPlus
	 */
	@RequestMapping(value = "/instancedMetricsGet", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public @ResponseBody List<InstancedMetric> instancedMetricsGet(
			@RequestParam(value = "idDataModel", required = true) int idDataModel, Model model,
			HttpServletRequest request) throws Exception {
		try {

			return this.instancedMetricsService.instancedMetricsGet(idDataModel);
		} catch (Exception e) {
			throw new ExceptionMethod("It is not possible to obtain the instanced metrics.");
		}
	}

	@ExceptionHandler({ ExceptionMethod.class })
	public ResponseEntity<String> manageMethodException(Exception e) {
		return new ResponseEntity<String>(e.getMessage(), HttpStatus.NOT_FOUND);
	}

}
